const ok$1 = (data) => ({ ok: true, data });
const err$1 = (error) => ({
  ok: false,
  error
});
const wrapThrows$1 = (fn) => (...args) => {
  try {
    return {
      ok: true,
      data: fn(...args)
    };
  } catch (error) {
    return {
      ok: false,
      error
    };
  }
};
async function makeRequest(apiHost, endpoint, method, data) {
  const url = new URL(endpoint, apiHost);
  const body = JSON.stringify(data);
  const res = wrapThrows$1(fetch)(url.toString(), {
    method,
    headers: {
      "Content-Type": "application/json"
    },
    body
  });
  if (res.ok === false)
    return err$1(res.error);
  const response = await res.data;
  const { data: innerData } = await response.json();
  if (!response.ok) {
    return err$1({
      code: "network_error",
      message: response.statusText,
      status: response.status,
      url
    });
  }
  return ok$1(innerData);
}
class ActionAPI {
  constructor(apiHost, environmentId) {
    this.apiHost = apiHost;
    this.environmentId = environmentId;
  }
  async create(actionInput) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/actions`, "POST", actionInput);
  }
}
class DisplayAPI {
  constructor(baseUrl, environmentId) {
    this.apiHost = baseUrl;
    this.environmentId = environmentId;
  }
  async create(displayInput) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/displays`, "POST", displayInput);
  }
  async update(displayId, displayInput) {
    return makeRequest(
      this.apiHost,
      `/api/v1/client/${this.environmentId}/displays/${displayId}`,
      "PUT",
      displayInput
    );
  }
}
class PeopleAPI {
  constructor(apiHost, environmentId) {
    this.apiHost = apiHost;
    this.environmentId = environmentId;
  }
  async create(userId) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/people`, "POST", {
      environmentId: this.environmentId,
      userId
    });
  }
  async update(userId, personInput) {
    return makeRequest(
      this.apiHost,
      `/api/v1/client/${this.environmentId}/people/${userId}`,
      "POST",
      personInput
    );
  }
}
class ResponseAPI {
  constructor(apiHost, environmentId) {
    this.apiHost = apiHost;
    this.environmentId = environmentId;
  }
  async create(responseInput) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/responses`, "POST", responseInput);
  }
  async update({
    responseId,
    finished,
    data,
    ttc
  }) {
    return makeRequest(this.apiHost, `/api/v1/client/${this.environmentId}/responses/${responseId}`, "PUT", {
      finished,
      data,
      ttc
    });
  }
}
class StorageAPI {
  constructor(apiHost, environmentId) {
    this.apiHost = apiHost;
    this.environmentId = environmentId;
  }
  async uploadFile(file, { allowedFileExtensions, surveyId } = {}) {
    if (!(file instanceof Blob) || !(file instanceof File)) {
      throw new Error(`Invalid file type. Expected Blob or File, but received ${typeof file}`);
    }
    const payload = {
      fileName: file.name,
      fileType: file.type,
      allowedFileExtensions,
      surveyId
    };
    const response = await fetch(`${this.apiHost}/api/v1/client/${this.environmentId}/storage`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json"
      },
      body: JSON.stringify(payload)
    });
    if (!response.ok) {
      throw new Error(`Upload failed with status: ${response.status}`);
    }
    const json = await response.json();
    const { data } = json;
    const { signedUrl, fileUrl, signingData, presignedFields } = data;
    let requestHeaders = {};
    if (signingData) {
      const { signature, timestamp, uuid } = signingData;
      requestHeaders = {
        "X-File-Type": file.type,
        "X-File-Name": encodeURIComponent(file.name),
        "X-Survey-ID": surveyId ?? "",
        "X-Signature": signature,
        "X-Timestamp": String(timestamp),
        "X-UUID": uuid
      };
    }
    const formData = new FormData();
    if (presignedFields) {
      Object.keys(presignedFields).forEach((key) => {
        formData.append(key, presignedFields[key]);
      });
    }
    formData.append("file", file);
    const uploadResponse = await fetch(signedUrl, {
      method: "POST",
      ...signingData ? { headers: requestHeaders } : {},
      body: formData
    });
    if (!uploadResponse.ok) {
      const uploadJson = await uploadResponse.json();
      throw new Error(`${uploadJson.message}`);
    }
    return fileUrl;
  }
}
class Client {
  constructor(options) {
    const { apiHost, environmentId } = options;
    this.response = new ResponseAPI(apiHost, environmentId);
    this.display = new DisplayAPI(apiHost, environmentId);
    this.action = new ActionAPI(apiHost, environmentId);
    this.people = new PeopleAPI(apiHost, environmentId);
    this.storage = new StorageAPI(apiHost, environmentId);
  }
}
class FormbricksAPI {
  constructor(options) {
    this.client = new Client(options);
  }
}
class Logger {
  constructor() {
    this.logLevel = "error";
  }
  static getInstance() {
    if (!Logger.instance) {
      Logger.instance = new Logger();
    }
    return Logger.instance;
  }
  configure(config2) {
    if (config2 && config2.logLevel !== void 0) {
      this.logLevel = config2.logLevel;
    }
  }
  logger(message, level) {
    if (level === "debug" && this.logLevel !== "debug") {
      return;
    }
    const timestamp = (/* @__PURE__ */ new Date()).toISOString();
    const logMessage = `🧱 Formbricks - ${timestamp} [${level.toUpperCase()}] - ${message}`;
    if (level === "error") {
      console.error(logMessage);
    } else {
      console.log(logMessage);
    }
  }
  debug(message) {
    this.logger(message, "debug");
  }
  error(message) {
    this.logger(message, "error");
  }
}
const ok = (value) => ({ ok: true, value });
const okVoid = () => ({ ok: true, value: void 0 });
const err = (error) => ({
  ok: false,
  error
});
function match(result, onSuccess, onError) {
  if (result.ok === true) {
    return onSuccess(result.value);
  }
  return onError(result.error);
}
const wrapThrows = (fn) => (...args) => {
  try {
    return {
      ok: true,
      value: fn(...args)
    };
  } catch (error) {
    return {
      ok: false,
      error
    };
  }
};
const logger$7 = Logger.getInstance();
const _ErrorHandler = class _ErrorHandler {
  constructor(errorHandler2) {
    this.customized = false;
    if (errorHandler2) {
      this.handleError = errorHandler2;
      this.customized = true;
    } else {
      this.handleError = (err2) => Logger.getInstance().error(JSON.stringify(err2));
    }
  }
  static getInstance() {
    if (!_ErrorHandler.instance) {
      _ErrorHandler.instance = new _ErrorHandler();
    }
    return _ErrorHandler.instance;
  }
  static init(errorHandler2) {
    this.initialized = true;
    _ErrorHandler.instance = new _ErrorHandler(errorHandler2);
  }
  printStatus() {
    logger$7.debug(`Custom error handler: ${this.customized ? "yes" : "no"}`);
  }
  handle(error) {
    this.handleError(error);
  }
};
_ErrorHandler.initialized = false;
let ErrorHandler = _ErrorHandler;
const LOCAL_STORAGE_KEY = "formbricks-js";
class Config {
  constructor() {
    this.config = null;
    const localConfig = this.loadFromLocalStorage();
    if (localConfig.ok) {
      this.config = localConfig.value;
    }
  }
  static getInstance() {
    if (!Config.instance) {
      Config.instance = new Config();
    }
    return Config.instance;
  }
  update(newConfig) {
    if (newConfig) {
      const expiresAt = new Date((/* @__PURE__ */ new Date()).getTime() + 15 * 6e4);
      this.config = {
        ...this.config,
        ...newConfig,
        expiresAt
      };
      this.saveToLocalStorage();
    }
  }
  get() {
    if (!this.config) {
      throw new Error("config is null, maybe the init function was not called?");
    }
    return this.config;
  }
  loadFromLocalStorage() {
    if (typeof window !== "undefined") {
      const savedConfig = localStorage.getItem(LOCAL_STORAGE_KEY);
      if (savedConfig) {
        const parsedConfig = JSON.parse(savedConfig);
        if (parsedConfig.expiresAt && new Date(parsedConfig.expiresAt) <= /* @__PURE__ */ new Date()) {
          return err(new Error("Config in local storage has expired"));
        }
        return ok(JSON.parse(savedConfig));
      }
    }
    return err(new Error("No or invalid config in local storage"));
  }
  saveToLocalStorage() {
    return wrapThrows(() => localStorage.setItem(LOCAL_STORAGE_KEY, JSON.stringify(this.config)))();
  }
  // reset the config
  resetConfig() {
    this.config = null;
    return wrapThrows(() => localStorage.removeItem(LOCAL_STORAGE_KEY))();
  }
}
class ResponseQueue {
  constructor(config2, surveyState) {
    this.queue = [];
    this.isRequestInProgress = false;
    this.config = config2;
    this.surveyState = surveyState;
    this.api = new FormbricksAPI({
      apiHost: config2.apiHost,
      environmentId: config2.environmentId
    });
  }
  add(responseUpdate) {
    this.surveyState.accumulateResponse(responseUpdate);
    if (this.config.setSurveyState) {
      this.config.setSurveyState(this.surveyState);
    }
    this.queue.push(responseUpdate);
    this.processQueue();
  }
  async processQueue() {
    if (this.isRequestInProgress)
      return;
    if (this.queue.length === 0)
      return;
    this.isRequestInProgress = true;
    const responseUpdate = this.queue[0];
    let attempts = 0;
    while (attempts < this.config.retryAttempts) {
      const success = await this.sendResponse(responseUpdate);
      if (success) {
        this.queue.shift();
        break;
      }
      console.error("Formbricks: Failed to send response. Retrying...", attempts);
      attempts++;
    }
    if (attempts >= this.config.retryAttempts) {
      console.error("Failed to send response after 2 attempts.");
      if (this.surveyState.responseAcc.finished && this.config.onResponseSendingFailed) {
        this.config.onResponseSendingFailed(this.surveyState.responseAcc);
      }
      this.queue.shift();
    }
    this.isRequestInProgress = false;
    this.processQueue();
  }
  async sendResponse(responseUpdate) {
    try {
      if (this.surveyState.responseId !== null) {
        await this.api.client.response.update({ ...responseUpdate, responseId: this.surveyState.responseId });
      } else {
        const response = await this.api.client.response.create({
          ...responseUpdate,
          surveyId: this.surveyState.surveyId,
          userId: this.surveyState.userId || null,
          singleUseId: this.surveyState.singleUseId || null
        });
        if (!response.ok) {
          throw new Error("Could not create response");
        }
        if (this.surveyState.displayId) {
          await this.api.client.display.update(this.surveyState.displayId, { responseId: response.data.id });
        }
        this.surveyState.updateResponseId(response.data.id);
        if (this.config.setSurveyState) {
          this.config.setSurveyState(this.surveyState);
        }
      }
      return true;
    } catch (error) {
      console.error(error);
      return false;
    }
  }
  // update surveyState
  updateSurveyState(surveyState) {
    this.surveyState = surveyState;
  }
}
class SurveyState {
  constructor(surveyId, singleUseId, responseId, userId) {
    this.responseId = null;
    this.displayId = null;
    this.userId = null;
    this.responseAcc = { finished: false, data: {}, ttc: {} };
    this.surveyId = surveyId;
    this.userId = userId ?? null;
    this.singleUseId = singleUseId ?? null;
    this.responseId = responseId ?? null;
  }
  /**
   * Set the current survey ID
   * @param id - The survey ID
   */
  setSurveyId(id) {
    this.surveyId = id;
    this.clear();
  }
  /**
   * Get a copy of the current state
   */
  copy() {
    const copyInstance = new SurveyState(
      this.surveyId,
      this.singleUseId ?? void 0,
      this.responseId ?? void 0,
      this.userId ?? void 0
    );
    copyInstance.responseId = this.responseId;
    copyInstance.responseAcc = this.responseAcc;
    return copyInstance;
  }
  /**
   * Update the response ID after a successful response creation
   * @param id - The response ID
   */
  updateResponseId(id) {
    this.responseId = id;
  }
  /**
   * Update the response ID after a successful response creation
   * @param id - The response ID
   */
  updateDisplayId(id) {
    this.displayId = id;
  }
  /**
   * Update the user ID
   * @param id - The user ID
   */
  updateUserId(id) {
    this.userId = id;
  }
  /**
   * Accumulate the responses
   * @param responseUpdate - The new response data to add
   */
  accumulateResponse(responseUpdate) {
    this.responseAcc = {
      finished: responseUpdate.finished,
      ttc: responseUpdate.ttc,
      data: { ...this.responseAcc.data, ...responseUpdate.data }
    };
  }
  /**
   * Check if the current accumulated response is finished
   */
  isResponseFinished() {
    return this.responseAcc.finished;
  }
  /**
   * Clear the current state
   */
  clear() {
    this.responseId = null;
    this.responseAcc = { finished: false, data: {}, ttc: {} };
  }
}
const SurveyState$1 = SurveyState;
var n, l$2, u$2, i$1, o$1, r$2, f$2, c$1 = {}, s$1 = [], a$1 = /acit|ex(?:s|g|n|p|$)|rph|grid|ows|mnc|ntw|ine[ch]|zoo|^ord|itera/i, h$1 = Array.isArray;
function v$1(n2, l2) {
  for (var u2 in l2)
    n2[u2] = l2[u2];
  return n2;
}
function p$1(n2) {
  var l2 = n2.parentNode;
  l2 && l2.removeChild(n2);
}
function y(l2, u2, t2) {
  var i2, o2, r2, f2 = {};
  for (r2 in u2)
    "key" == r2 ? i2 = u2[r2] : "ref" == r2 ? o2 = u2[r2] : f2[r2] = u2[r2];
  if (arguments.length > 2 && (f2.children = arguments.length > 3 ? n.call(arguments, 2) : t2), "function" == typeof l2 && null != l2.defaultProps)
    for (r2 in l2.defaultProps)
      void 0 === f2[r2] && (f2[r2] = l2.defaultProps[r2]);
  return d$1(l2, f2, i2, o2, null);
}
function d$1(n2, t2, i2, o2, r2) {
  var f2 = { type: n2, props: t2, key: i2, ref: o2, __k: null, __: null, __b: 0, __e: null, __d: void 0, __c: null, constructor: void 0, __v: null == r2 ? ++u$2 : r2, __i: -1, __u: 0 };
  return null == r2 && null != l$2.vnode && l$2.vnode(f2), f2;
}
function g$2(n2) {
  return n2.children;
}
function b$1(n2, l2) {
  this.props = n2, this.context = l2;
}
function m$1(n2, l2) {
  if (null == l2)
    return n2.__ ? m$1(n2.__, n2.__i + 1) : null;
  for (var u2; l2 < n2.__k.length; l2++)
    if (null != (u2 = n2.__k[l2]) && null != u2.__e)
      return u2.__e;
  return "function" == typeof n2.type ? m$1(n2) : null;
}
function k$1(n2) {
  var l2, u2;
  if (null != (n2 = n2.__) && null != n2.__c) {
    for (n2.__e = n2.__c.base = null, l2 = 0; l2 < n2.__k.length; l2++)
      if (null != (u2 = n2.__k[l2]) && null != u2.__e) {
        n2.__e = n2.__c.base = u2.__e;
        break;
      }
    return k$1(n2);
  }
}
function w$1(n2) {
  (!n2.__d && (n2.__d = true) && i$1.push(n2) && !x$1.__r++ || o$1 !== l$2.debounceRendering) && ((o$1 = l$2.debounceRendering) || r$2)(x$1);
}
function x$1() {
  var n2, u2, t2, o2, r2, e2, c2, s2, a2;
  for (i$1.sort(f$2); n2 = i$1.shift(); )
    n2.__d && (u2 = i$1.length, o2 = void 0, e2 = (r2 = (t2 = n2).__v).__e, s2 = [], a2 = [], (c2 = t2.__P) && ((o2 = v$1({}, r2)).__v = r2.__v + 1, l$2.vnode && l$2.vnode(o2), L$1(c2, o2, r2, t2.__n, void 0 !== c2.ownerSVGElement, 32 & r2.__u ? [e2] : null, s2, null == e2 ? m$1(r2) : e2, !!(32 & r2.__u), a2), o2.__.__k[o2.__i] = o2, M$1(s2, o2, a2), o2.__e != e2 && k$1(o2)), i$1.length > u2 && i$1.sort(f$2));
  x$1.__r = 0;
}
function C$1(n2, l2, u2, t2, i2, o2, r2, f2, e2, a2, h2) {
  var v2, p2, y2, d2, _2, g2 = t2 && t2.__k || s$1, b2 = l2.length;
  for (u2.__d = e2, P(u2, l2, g2), e2 = u2.__d, v2 = 0; v2 < b2; v2++)
    null != (y2 = u2.__k[v2]) && "boolean" != typeof y2 && "function" != typeof y2 && (p2 = -1 === y2.__i ? c$1 : g2[y2.__i] || c$1, y2.__i = v2, L$1(n2, y2, p2, i2, o2, r2, f2, e2, a2, h2), d2 = y2.__e, y2.ref && p2.ref != y2.ref && (p2.ref && z$3(p2.ref, null, y2), h2.push(y2.ref, y2.__c || d2, y2)), null == _2 && null != d2 && (_2 = d2), 65536 & y2.__u || p2.__k === y2.__k ? e2 = S(y2, e2, n2) : "function" == typeof y2.type && void 0 !== y2.__d ? e2 = y2.__d : d2 && (e2 = d2.nextSibling), y2.__d = void 0, y2.__u &= -196609);
  u2.__d = e2, u2.__e = _2;
}
function P(n2, l2, u2) {
  var t2, i2, o2, r2, f2, e2 = l2.length, c2 = u2.length, s2 = c2, a2 = 0;
  for (n2.__k = [], t2 = 0; t2 < e2; t2++)
    null != (i2 = n2.__k[t2] = null == (i2 = l2[t2]) || "boolean" == typeof i2 || "function" == typeof i2 ? null : "string" == typeof i2 || "number" == typeof i2 || "bigint" == typeof i2 || i2.constructor == String ? d$1(null, i2, null, null, i2) : h$1(i2) ? d$1(g$2, { children: i2 }, null, null, null) : void 0 === i2.constructor && i2.__b > 0 ? d$1(i2.type, i2.props, i2.key, i2.ref ? i2.ref : null, i2.__v) : i2) ? (i2.__ = n2, i2.__b = n2.__b + 1, f2 = H$1(i2, u2, r2 = t2 + a2, s2), i2.__i = f2, o2 = null, -1 !== f2 && (s2--, (o2 = u2[f2]) && (o2.__u |= 131072)), null == o2 || null === o2.__v ? (-1 == f2 && a2--, "function" != typeof i2.type && (i2.__u |= 65536)) : f2 !== r2 && (f2 === r2 + 1 ? a2++ : f2 > r2 ? s2 > e2 - r2 ? a2 += f2 - r2 : a2-- : a2 = f2 < r2 && f2 == r2 - 1 ? f2 - r2 : 0, f2 !== t2 + a2 && (i2.__u |= 65536))) : (o2 = u2[t2]) && null == o2.key && o2.__e && (o2.__e == n2.__d && (n2.__d = m$1(o2)), N(o2, o2, false), u2[t2] = null, s2--);
  if (s2)
    for (t2 = 0; t2 < c2; t2++)
      null != (o2 = u2[t2]) && 0 == (131072 & o2.__u) && (o2.__e == n2.__d && (n2.__d = m$1(o2)), N(o2, o2));
}
function S(n2, l2, u2) {
  var t2, i2;
  if ("function" == typeof n2.type) {
    for (t2 = n2.__k, i2 = 0; t2 && i2 < t2.length; i2++)
      t2[i2] && (t2[i2].__ = n2, l2 = S(t2[i2], l2, u2));
    return l2;
  }
  return n2.__e != l2 && (u2.insertBefore(n2.__e, l2 || null), l2 = n2.__e), l2 && l2.nextSibling;
}
function $$1(n2, l2) {
  return l2 = l2 || [], null == n2 || "boolean" == typeof n2 || (h$1(n2) ? n2.some(function(n3) {
    $$1(n3, l2);
  }) : l2.push(n2)), l2;
}
function H$1(n2, l2, u2, t2) {
  var i2 = n2.key, o2 = n2.type, r2 = u2 - 1, f2 = u2 + 1, e2 = l2[u2];
  if (null === e2 || e2 && i2 == e2.key && o2 === e2.type)
    return u2;
  if (t2 > (null != e2 && 0 == (131072 & e2.__u) ? 1 : 0))
    for (; r2 >= 0 || f2 < l2.length; ) {
      if (r2 >= 0) {
        if ((e2 = l2[r2]) && 0 == (131072 & e2.__u) && i2 == e2.key && o2 === e2.type)
          return r2;
        r2--;
      }
      if (f2 < l2.length) {
        if ((e2 = l2[f2]) && 0 == (131072 & e2.__u) && i2 == e2.key && o2 === e2.type)
          return f2;
        f2++;
      }
    }
  return -1;
}
function I$1(n2, l2, u2) {
  "-" === l2[0] ? n2.setProperty(l2, null == u2 ? "" : u2) : n2[l2] = null == u2 ? "" : "number" != typeof u2 || a$1.test(l2) ? u2 : u2 + "px";
}
function T$2(n2, l2, u2, t2, i2) {
  var o2;
  n:
    if ("style" === l2)
      if ("string" == typeof u2)
        n2.style.cssText = u2;
      else {
        if ("string" == typeof t2 && (n2.style.cssText = t2 = ""), t2)
          for (l2 in t2)
            u2 && l2 in u2 || I$1(n2.style, l2, "");
        if (u2)
          for (l2 in u2)
            t2 && u2[l2] === t2[l2] || I$1(n2.style, l2, u2[l2]);
      }
    else if ("o" === l2[0] && "n" === l2[1])
      o2 = l2 !== (l2 = l2.replace(/(PointerCapture)$|Capture$/, "$1")), l2 = l2.toLowerCase() in n2 ? l2.toLowerCase().slice(2) : l2.slice(2), n2.l || (n2.l = {}), n2.l[l2 + o2] = u2, u2 ? t2 ? u2.u = t2.u : (u2.u = Date.now(), n2.addEventListener(l2, o2 ? D : A, o2)) : n2.removeEventListener(l2, o2 ? D : A, o2);
    else {
      if (i2)
        l2 = l2.replace(/xlink(H|:h)/, "h").replace(/sName$/, "s");
      else if ("width" !== l2 && "height" !== l2 && "href" !== l2 && "list" !== l2 && "form" !== l2 && "tabIndex" !== l2 && "download" !== l2 && "rowSpan" !== l2 && "colSpan" !== l2 && "role" !== l2 && l2 in n2)
        try {
          n2[l2] = null == u2 ? "" : u2;
          break n;
        } catch (n3) {
        }
      "function" == typeof u2 || (null == u2 || false === u2 && "-" !== l2[4] ? n2.removeAttribute(l2) : n2.setAttribute(l2, u2));
    }
}
function A(n2) {
  var u2 = this.l[n2.type + false];
  if (n2.t) {
    if (n2.t <= u2.u)
      return;
  } else
    n2.t = Date.now();
  return u2(l$2.event ? l$2.event(n2) : n2);
}
function D(n2) {
  return this.l[n2.type + true](l$2.event ? l$2.event(n2) : n2);
}
function L$1(n2, u2, t2, i2, o2, r2, f2, e2, c2, s2) {
  var a2, p2, y2, d2, _2, m2, k2, w2, x2, P2, S2, $2, H2, I2, T2, A2 = u2.type;
  if (void 0 !== u2.constructor)
    return null;
  128 & t2.__u && (c2 = !!(32 & t2.__u), r2 = [e2 = u2.__e = t2.__e]), (a2 = l$2.__b) && a2(u2);
  n:
    if ("function" == typeof A2)
      try {
        if (w2 = u2.props, x2 = (a2 = A2.contextType) && i2[a2.__c], P2 = a2 ? x2 ? x2.props.value : a2.__ : i2, t2.__c ? k2 = (p2 = u2.__c = t2.__c).__ = p2.__E : ("prototype" in A2 && A2.prototype.render ? u2.__c = p2 = new A2(w2, P2) : (u2.__c = p2 = new b$1(w2, P2), p2.constructor = A2, p2.render = O$1), x2 && x2.sub(p2), p2.props = w2, p2.state || (p2.state = {}), p2.context = P2, p2.__n = i2, y2 = p2.__d = true, p2.__h = [], p2._sb = []), null == p2.__s && (p2.__s = p2.state), null != A2.getDerivedStateFromProps && (p2.__s == p2.state && (p2.__s = v$1({}, p2.__s)), v$1(p2.__s, A2.getDerivedStateFromProps(w2, p2.__s))), d2 = p2.props, _2 = p2.state, p2.__v = u2, y2)
          null == A2.getDerivedStateFromProps && null != p2.componentWillMount && p2.componentWillMount(), null != p2.componentDidMount && p2.__h.push(p2.componentDidMount);
        else {
          if (null == A2.getDerivedStateFromProps && w2 !== d2 && null != p2.componentWillReceiveProps && p2.componentWillReceiveProps(w2, P2), !p2.__e && (null != p2.shouldComponentUpdate && false === p2.shouldComponentUpdate(w2, p2.__s, P2) || u2.__v === t2.__v)) {
            for (u2.__v !== t2.__v && (p2.props = w2, p2.state = p2.__s, p2.__d = false), u2.__e = t2.__e, u2.__k = t2.__k, u2.__k.forEach(function(n3) {
              n3 && (n3.__ = u2);
            }), S2 = 0; S2 < p2._sb.length; S2++)
              p2.__h.push(p2._sb[S2]);
            p2._sb = [], p2.__h.length && f2.push(p2);
            break n;
          }
          null != p2.componentWillUpdate && p2.componentWillUpdate(w2, p2.__s, P2), null != p2.componentDidUpdate && p2.__h.push(function() {
            p2.componentDidUpdate(d2, _2, m2);
          });
        }
        if (p2.context = P2, p2.props = w2, p2.__P = n2, p2.__e = false, $2 = l$2.__r, H2 = 0, "prototype" in A2 && A2.prototype.render) {
          for (p2.state = p2.__s, p2.__d = false, $2 && $2(u2), a2 = p2.render(p2.props, p2.state, p2.context), I2 = 0; I2 < p2._sb.length; I2++)
            p2.__h.push(p2._sb[I2]);
          p2._sb = [];
        } else
          do {
            p2.__d = false, $2 && $2(u2), a2 = p2.render(p2.props, p2.state, p2.context), p2.state = p2.__s;
          } while (p2.__d && ++H2 < 25);
        p2.state = p2.__s, null != p2.getChildContext && (i2 = v$1(v$1({}, i2), p2.getChildContext())), y2 || null == p2.getSnapshotBeforeUpdate || (m2 = p2.getSnapshotBeforeUpdate(d2, _2)), C$1(n2, h$1(T2 = null != a2 && a2.type === g$2 && null == a2.key ? a2.props.children : a2) ? T2 : [T2], u2, t2, i2, o2, r2, f2, e2, c2, s2), p2.base = u2.__e, u2.__u &= -161, p2.__h.length && f2.push(p2), k2 && (p2.__E = p2.__ = null);
      } catch (n3) {
        u2.__v = null, c2 || null != r2 ? (u2.__e = e2, u2.__u |= c2 ? 160 : 32, r2[r2.indexOf(e2)] = null) : (u2.__e = t2.__e, u2.__k = t2.__k), l$2.__e(n3, u2, t2);
      }
    else
      null == r2 && u2.__v === t2.__v ? (u2.__k = t2.__k, u2.__e = t2.__e) : u2.__e = j$1(t2.__e, u2, t2, i2, o2, r2, f2, c2, s2);
  (a2 = l$2.diffed) && a2(u2);
}
function M$1(n2, u2, t2) {
  u2.__d = void 0;
  for (var i2 = 0; i2 < t2.length; i2++)
    z$3(t2[i2], t2[++i2], t2[++i2]);
  l$2.__c && l$2.__c(u2, n2), n2.some(function(u3) {
    try {
      n2 = u3.__h, u3.__h = [], n2.some(function(n3) {
        n3.call(u3);
      });
    } catch (n3) {
      l$2.__e(n3, u3.__v);
    }
  });
}
function j$1(l2, u2, t2, i2, o2, r2, f2, e2, s2) {
  var a2, v2, y2, d2, _2, g2, b2, k2 = t2.props, w2 = u2.props, x2 = u2.type;
  if ("svg" === x2 && (o2 = true), null != r2) {
    for (a2 = 0; a2 < r2.length; a2++)
      if ((_2 = r2[a2]) && "setAttribute" in _2 == !!x2 && (x2 ? _2.localName === x2 : 3 === _2.nodeType)) {
        l2 = _2, r2[a2] = null;
        break;
      }
  }
  if (null == l2) {
    if (null === x2)
      return document.createTextNode(w2);
    l2 = o2 ? document.createElementNS("http://www.w3.org/2000/svg", x2) : document.createElement(x2, w2.is && w2), r2 = null, e2 = false;
  }
  if (null === x2)
    k2 === w2 || e2 && l2.data === w2 || (l2.data = w2);
  else {
    if (r2 = r2 && n.call(l2.childNodes), k2 = t2.props || c$1, !e2 && null != r2)
      for (k2 = {}, a2 = 0; a2 < l2.attributes.length; a2++)
        k2[(_2 = l2.attributes[a2]).name] = _2.value;
    for (a2 in k2)
      _2 = k2[a2], "children" == a2 || ("dangerouslySetInnerHTML" == a2 ? y2 = _2 : "key" === a2 || a2 in w2 || T$2(l2, a2, null, _2, o2));
    for (a2 in w2)
      _2 = w2[a2], "children" == a2 ? d2 = _2 : "dangerouslySetInnerHTML" == a2 ? v2 = _2 : "value" == a2 ? g2 = _2 : "checked" == a2 ? b2 = _2 : "key" === a2 || e2 && "function" != typeof _2 || k2[a2] === _2 || T$2(l2, a2, _2, k2[a2], o2);
    if (v2)
      e2 || y2 && (v2.__html === y2.__html || v2.__html === l2.innerHTML) || (l2.innerHTML = v2.__html), u2.__k = [];
    else if (y2 && (l2.innerHTML = ""), C$1(l2, h$1(d2) ? d2 : [d2], u2, t2, i2, o2 && "foreignObject" !== x2, r2, f2, r2 ? r2[0] : t2.__k && m$1(t2, 0), e2, s2), null != r2)
      for (a2 = r2.length; a2--; )
        null != r2[a2] && p$1(r2[a2]);
    e2 || (a2 = "value", void 0 !== g2 && (g2 !== l2[a2] || "progress" === x2 && !g2 || "option" === x2 && g2 !== k2[a2]) && T$2(l2, a2, g2, k2[a2], false), a2 = "checked", void 0 !== b2 && b2 !== l2[a2] && T$2(l2, a2, b2, k2[a2], false));
  }
  return l2;
}
function z$3(n2, u2, t2) {
  try {
    "function" == typeof n2 ? n2(u2) : n2.current = u2;
  } catch (n3) {
    l$2.__e(n3, t2);
  }
}
function N(n2, u2, t2) {
  var i2, o2;
  if (l$2.unmount && l$2.unmount(n2), (i2 = n2.ref) && (i2.current && i2.current !== n2.__e || z$3(i2, null, u2)), null != (i2 = n2.__c)) {
    if (i2.componentWillUnmount)
      try {
        i2.componentWillUnmount();
      } catch (n3) {
        l$2.__e(n3, u2);
      }
    i2.base = i2.__P = null, n2.__c = void 0;
  }
  if (i2 = n2.__k)
    for (o2 = 0; o2 < i2.length; o2++)
      i2[o2] && N(i2[o2], u2, t2 || "function" != typeof n2.type);
  t2 || null == n2.__e || p$1(n2.__e), n2.__ = n2.__e = n2.__d = void 0;
}
function O$1(n2, l2, u2) {
  return this.constructor(n2, u2);
}
function q(u2, t2, i2) {
  var o2, r2, f2, e2;
  l$2.__ && l$2.__(u2, t2), r2 = (o2 = "function" == typeof i2) ? null : i2 && i2.__k || t2.__k, f2 = [], e2 = [], L$1(t2, u2 = (!o2 && i2 || t2).__k = y(g$2, null, [u2]), r2 || c$1, c$1, void 0 !== t2.ownerSVGElement, !o2 && i2 ? [i2] : r2 ? null : t2.firstChild ? n.call(t2.childNodes) : null, f2, !o2 && i2 ? i2 : r2 ? r2.__e : t2.firstChild, o2, e2), M$1(f2, u2, e2);
}
n = s$1.slice, l$2 = { __e: function(n2, l2, u2, t2) {
  for (var i2, o2, r2; l2 = l2.__; )
    if ((i2 = l2.__c) && !i2.__)
      try {
        if ((o2 = i2.constructor) && null != o2.getDerivedStateFromError && (i2.setState(o2.getDerivedStateFromError(n2)), r2 = i2.__d), null != i2.componentDidCatch && (i2.componentDidCatch(n2, t2 || {}), r2 = i2.__d), r2)
          return i2.__E = i2;
      } catch (l3) {
        n2 = l3;
      }
  throw n2;
} }, u$2 = 0, b$1.prototype.setState = function(n2, l2) {
  var u2;
  u2 = null != this.__s && this.__s !== this.state ? this.__s : this.__s = v$1({}, this.state), "function" == typeof n2 && (n2 = n2(v$1({}, u2), this.props)), n2 && v$1(u2, n2), null != n2 && this.__v && (l2 && this._sb.push(l2), w$1(this));
}, b$1.prototype.forceUpdate = function(n2) {
  this.__v && (this.__e = true, n2 && this.__h.push(n2), w$1(this));
}, b$1.prototype.render = g$2, i$1 = [], r$2 = "function" == typeof Promise ? Promise.prototype.then.bind(Promise.resolve()) : setTimeout, f$2 = function(n2, l2) {
  return n2.__v.__b - l2.__v.__b;
}, x$1.__r = 0;
var f$1 = 0;
function u$1(e2, t2, n2, o2, i2, u2) {
  var a2, c2, p2 = {};
  for (c2 in t2)
    "ref" == c2 ? a2 = t2[c2] : p2[c2] = t2[c2];
  var l2 = { type: e2, props: p2, key: n2, ref: a2, __k: null, __: null, __b: 0, __e: null, __d: void 0, __c: null, constructor: void 0, __v: --f$1, __i: -1, __u: 0, __source: i2, __self: u2 };
  if ("function" == typeof e2 && (a2 = e2.defaultProps))
    for (c2 in a2)
      void 0 === p2[c2] && (p2[c2] = a2[c2]);
  return l$2.vnode && l$2.vnode(l2), l2;
}
function FormbricksBranding() {
  return u$1("a", {
    href: "https://formbricks.com?utm_source=survey_branding",
    target: "_blank",
    tabIndex: -1,
    className: "mb-5 mt-2 flex justify-center",
    children: u$1("p", {
      className: "text-signature text-xs",
      children: ["Powered by", " ", u$1("b", {
        children: u$1("span", {
          className: "text-info-text hover:text-heading",
          children: "Formbricks"
        })
      })]
    })
  });
}
const cn = (...classes) => {
  return classes.filter(Boolean).join(" ");
};
function isLight(color) {
  let r2, g2, b2;
  if (color.length === 4) {
    r2 = parseInt(color[1] + color[1], 16);
    g2 = parseInt(color[2] + color[2], 16);
    b2 = parseInt(color[3] + color[3], 16);
  } else if (color.length === 7) {
    r2 = parseInt(color[1] + color[2], 16);
    g2 = parseInt(color[3] + color[4], 16);
    b2 = parseInt(color[5] + color[6], 16);
  }
  if (r2 === void 0 || g2 === void 0 || b2 === void 0) {
    throw new Error("Invalid color");
  }
  return r2 * 0.299 + g2 * 0.587 + b2 * 0.114 > 128;
}
const shuffle = (array) => {
  for (let i2 = 0; i2 < array.length; i2++) {
    const j2 = Math.floor(Math.random() * (i2 + 1));
    [array[i2], array[j2]] = [array[j2], array[i2]];
  }
};
const shuffleQuestions = (array, shuffleOption) => {
  const arrayCopy = [...array];
  const otherIndex = arrayCopy.findIndex((element) => element.id === "other");
  const otherElement = otherIndex !== -1 ? arrayCopy.splice(otherIndex, 1)[0] : null;
  if (shuffleOption === "all") {
    shuffle(arrayCopy);
  } else if (shuffleOption === "exceptLast") {
    const lastElement = arrayCopy.pop();
    shuffle(arrayCopy);
    arrayCopy.push(lastElement);
  }
  if (otherElement) {
    arrayCopy.push(otherElement);
  }
  return arrayCopy;
};
const calculateElementIdx = (survey, currentQustionIdx) => {
  var _a;
  const currentQuestion = survey.questions[currentQustionIdx];
  const surveyLength = survey.questions.length;
  const middleIdx = Math.floor(surveyLength / 2);
  const possibleNextQuestions = ((_a = currentQuestion == null ? void 0 : currentQuestion.logic) == null ? void 0 : _a.map((l2) => l2.destination)) || [];
  const getLastQuestionIndex = () => {
    const lastQuestion = survey.questions.filter((q2) => possibleNextQuestions.includes(q2.id)).sort((a2, b2) => survey.questions.indexOf(a2) - survey.questions.indexOf(b2)).pop();
    return survey.questions.findIndex((e2) => e2.id === (lastQuestion == null ? void 0 : lastQuestion.id));
  };
  let elementIdx = currentQustionIdx || 0.5;
  const lastprevQuestionIdx = getLastQuestionIndex();
  if (lastprevQuestionIdx > 0)
    elementIdx = Math.min(middleIdx, lastprevQuestionIdx - 1);
  if (possibleNextQuestions.includes("end"))
    elementIdx = middleIdx;
  return elementIdx;
};
var t, r$1, u, i, o = 0, f = [], c = [], e = l$2.__b, a = l$2.__r, v = l$2.diffed, l$1 = l$2.__c, m = l$2.unmount;
function d(t2, u2) {
  l$2.__h && l$2.__h(r$1, t2, o || u2), o = 0;
  var i2 = r$1.__H || (r$1.__H = { __: [], __h: [] });
  return t2 >= i2.__.length && i2.__.push({ __V: c }), i2.__[t2];
}
function h(n2) {
  return o = 1, s(B$1, n2);
}
function s(n2, u2, i2) {
  var o2 = d(t++, 2);
  if (o2.t = n2, !o2.__c && (o2.__ = [i2 ? i2(u2) : B$1(void 0, u2), function(n3) {
    var t2 = o2.__N ? o2.__N[0] : o2.__[0], r2 = o2.t(t2, n3);
    t2 !== r2 && (o2.__N = [r2, o2.__[1]], o2.__c.setState({}));
  }], o2.__c = r$1, !r$1.u)) {
    var f2 = function(n3, t2, r2) {
      if (!o2.__c.__H)
        return true;
      var u3 = o2.__c.__H.__.filter(function(n4) {
        return n4.__c;
      });
      if (u3.every(function(n4) {
        return !n4.__N;
      }))
        return !c2 || c2.call(this, n3, t2, r2);
      var i3 = false;
      return u3.forEach(function(n4) {
        if (n4.__N) {
          var t3 = n4.__[0];
          n4.__ = n4.__N, n4.__N = void 0, t3 !== n4.__[0] && (i3 = true);
        }
      }), !(!i3 && o2.__c.props === n3) && (!c2 || c2.call(this, n3, t2, r2));
    };
    r$1.u = true;
    var c2 = r$1.shouldComponentUpdate, e2 = r$1.componentWillUpdate;
    r$1.componentWillUpdate = function(n3, t2, r2) {
      if (this.__e) {
        var u3 = c2;
        c2 = void 0, f2(n3, t2, r2), c2 = u3;
      }
      e2 && e2.call(this, n3, t2, r2);
    }, r$1.shouldComponentUpdate = f2;
  }
  return o2.__N || o2.__;
}
function p(u2, i2) {
  var o2 = d(t++, 3);
  !l$2.__s && z$2(o2.__H, i2) && (o2.__ = u2, o2.i = i2, r$1.__H.__h.push(o2));
}
function _(n2) {
  return o = 5, F$1(function() {
    return { current: n2 };
  }, []);
}
function F$1(n2, r2) {
  var u2 = d(t++, 7);
  return z$2(u2.__H, r2) ? (u2.__V = n2(), u2.i = r2, u2.__h = n2, u2.__V) : u2.__;
}
function T$1(n2, t2) {
  return o = 8, F$1(function() {
    return n2;
  }, t2);
}
function b() {
  for (var t2; t2 = f.shift(); )
    if (t2.__P && t2.__H)
      try {
        t2.__H.__h.forEach(k), t2.__H.__h.forEach(w), t2.__H.__h = [];
      } catch (r2) {
        t2.__H.__h = [], l$2.__e(r2, t2.__v);
      }
}
l$2.__b = function(n2) {
  r$1 = null, e && e(n2);
}, l$2.__r = function(n2) {
  a && a(n2), t = 0;
  var i2 = (r$1 = n2.__c).__H;
  i2 && (u === r$1 ? (i2.__h = [], r$1.__h = [], i2.__.forEach(function(n3) {
    n3.__N && (n3.__ = n3.__N), n3.__V = c, n3.__N = n3.i = void 0;
  })) : (i2.__h.forEach(k), i2.__h.forEach(w), i2.__h = [], t = 0)), u = r$1;
}, l$2.diffed = function(t2) {
  v && v(t2);
  var o2 = t2.__c;
  o2 && o2.__H && (o2.__H.__h.length && (1 !== f.push(o2) && i === l$2.requestAnimationFrame || ((i = l$2.requestAnimationFrame) || j)(b)), o2.__H.__.forEach(function(n2) {
    n2.i && (n2.__H = n2.i), n2.__V !== c && (n2.__ = n2.__V), n2.i = void 0, n2.__V = c;
  })), u = r$1 = null;
}, l$2.__c = function(t2, r2) {
  r2.some(function(t3) {
    try {
      t3.__h.forEach(k), t3.__h = t3.__h.filter(function(n2) {
        return !n2.__ || w(n2);
      });
    } catch (u2) {
      r2.some(function(n2) {
        n2.__h && (n2.__h = []);
      }), r2 = [], l$2.__e(u2, t3.__v);
    }
  }), l$1 && l$1(t2, r2);
}, l$2.unmount = function(t2) {
  m && m(t2);
  var r2, u2 = t2.__c;
  u2 && u2.__H && (u2.__H.__.forEach(function(n2) {
    try {
      k(n2);
    } catch (n3) {
      r2 = n3;
    }
  }), u2.__H = void 0, r2 && l$2.__e(r2, u2.__v));
};
var g$1 = "function" == typeof requestAnimationFrame;
function j(n2) {
  var t2, r2 = function() {
    clearTimeout(u2), g$1 && cancelAnimationFrame(t2), setTimeout(n2);
  }, u2 = setTimeout(r2, 100);
  g$1 && (t2 = requestAnimationFrame(r2));
}
function k(n2) {
  var t2 = r$1, u2 = n2.__c;
  "function" == typeof u2 && (n2.__c = void 0, u2()), r$1 = t2;
}
function w(n2) {
  var t2 = r$1;
  n2.__c = n2.__(), r$1 = t2;
}
function z$2(n2, t2) {
  return !n2 || n2.length !== t2.length || t2.some(function(t3, r2) {
    return t3 !== n2[r2];
  });
}
function B$1(n2, t2) {
  return "function" == typeof t2 ? t2(n2) : t2;
}
function Progress({
  progress
}) {
  return u$1("div", {
    className: "bg-accent-bg h-2 w-full overflow-hidden rounded-full",
    children: u$1("div", {
      className: "transition-width bg-brand z-20 h-2 rounded-full duration-500",
      style: {
        width: `${Math.floor(progress * 100)}%`
      }
    })
  });
}
function ProgressBar({
  survey,
  questionId
}) {
  const currentQuestionIdx = F$1(() => survey.questions.findIndex((e2) => e2.id === questionId), [survey, questionId]);
  const calculateProgress = T$1((questionId2, survey2, progress) => {
    if (survey2.questions.length === 0)
      return 0;
    let currentQustionIdx = survey2.questions.findIndex((e2) => e2.id === questionId2);
    if (currentQustionIdx === -1)
      currentQustionIdx = 0;
    const elementIdx = calculateElementIdx(survey2, currentQustionIdx);
    const newProgress = elementIdx / survey2.questions.length;
    let updatedProgress = progress;
    if (newProgress > progress) {
      updatedProgress = newProgress;
    } else if (newProgress <= progress && progress + 0.1 <= 1) {
      updatedProgress = progress + 0.1;
    }
    return updatedProgress;
  }, []);
  const progressArray = F$1(() => {
    let progress = 0;
    let progressArrayTemp = [];
    survey.questions.forEach((question) => {
      progress = calculateProgress(question.id, survey, progress);
      progressArrayTemp.push(progress);
    });
    return progressArrayTemp;
  }, [calculateProgress, survey]);
  return u$1(Progress, {
    progress: questionId === "end" ? 1 : progressArray[currentQuestionIdx]
  });
}
function AutoCloseWrapper({
  survey,
  onClose,
  children
}) {
  const [countdownProgress, setCountdownProgress] = h(100);
  const [countdownStop, setCountdownStop] = h(false);
  const startRef = _(performance.now());
  const frameRef = _(null);
  const handleStopCountdown = () => {
    if (frameRef.current !== null) {
      setCountdownStop(true);
      cancelAnimationFrame(frameRef.current);
      frameRef.current = null;
    }
  };
  p(() => {
    if (!survey.autoClose)
      return;
    const updateCountdown = () => {
      const timeout = survey.autoClose * 1e3;
      const elapsed = performance.now() - startRef.current;
      const remaining = Math.max(0, timeout - elapsed);
      setCountdownProgress(remaining / timeout);
      if (remaining > 0) {
        frameRef.current = requestAnimationFrame(updateCountdown);
      } else {
        handleStopCountdown();
        onClose();
      }
    };
    setCountdownProgress(1);
    frameRef.current = requestAnimationFrame(updateCountdown);
    return () => handleStopCountdown();
  }, [survey.autoClose, onClose]);
  return u$1(g$2, {
    children: [!countdownStop && survey.autoClose && u$1(Progress, {
      progress: countdownProgress
    }), u$1("div", {
      onClick: handleStopCountdown,
      onMouseOver: handleStopCountdown,
      className: "h-full w-full",
      children
    })]
  });
}
function evaluateCondition(logic, responseValue) {
  switch (logic.condition) {
    case "equals":
      return Array.isArray(responseValue) && responseValue.length === 1 && responseValue.includes(logic.value) || (responseValue == null ? void 0 : responseValue.toString()) === logic.value;
    case "notEquals":
      return responseValue !== logic.value;
    case "lessThan":
      return logic.value !== void 0 && responseValue < logic.value;
    case "lessEqual":
      return logic.value !== void 0 && responseValue <= logic.value;
    case "greaterThan":
      return logic.value !== void 0 && responseValue > logic.value;
    case "greaterEqual":
      return logic.value !== void 0 && responseValue >= logic.value;
    case "includesAll":
      return Array.isArray(responseValue) && Array.isArray(logic.value) && logic.value.every((v2) => responseValue.includes(v2));
    case "includesOne":
      return Array.isArray(responseValue) && Array.isArray(logic.value) && logic.value.some((v2) => responseValue.includes(v2));
    case "accepted":
      return responseValue === "accepted";
    case "clicked":
      return responseValue === "clicked";
    case "submitted":
      if (typeof responseValue === "string") {
        return responseValue !== "dismissed" && responseValue !== "" && responseValue !== null;
      } else if (Array.isArray(responseValue)) {
        return responseValue.length > 0;
      } else if (typeof responseValue === "number") {
        return responseValue !== null;
      }
      return false;
    case "skipped":
      return Array.isArray(responseValue) && responseValue.length === 0 || responseValue === "" || responseValue === null || responseValue === void 0 || responseValue === "dismissed";
    case "uploaded":
      if (Array.isArray(responseValue)) {
        return responseValue.length > 0;
      } else {
        return responseValue !== "skipped" && responseValue !== "" && responseValue !== null;
      }
    case "notUploaded":
      return Array.isArray(responseValue) && responseValue.length === 0 || responseValue === "" || responseValue === null || responseValue === "skipped";
    default:
      return false;
  }
}
function BackButton({
  onClick,
  backButtonLabel,
  tabIndex = 2
}) {
  return u$1("button", {
    tabIndex,
    type: "button",
    className: cn("border-back-button-border text-heading focus:ring-focus flex items-center rounded-md border px-3 py-3 text-base font-medium leading-4 shadow-sm hover:opacity-90 focus:outline-none focus:ring-2 focus:ring-offset-2"),
    onClick,
    children: backButtonLabel || "Back"
  });
}
function SubmitButton({
  buttonLabel,
  isLastQuestion,
  onClick,
  tabIndex = 1,
  focus = false,
  type = "submit"
}) {
  const buttonRef = T$1((currentButton) => {
    if (currentButton && focus) {
      setTimeout(() => {
        currentButton.focus();
      }, 200);
    }
  }, [focus]);
  return u$1("button", {
    ref: buttonRef,
    type,
    tabIndex,
    autoFocus: focus,
    className: "bg-brand border-submit-button-border text-on-brand focus:ring-focus flex items-center rounded-md border px-3 py-3 text-base font-medium leading-4 shadow-sm hover:opacity-90 focus:outline-none focus:ring-2 focus:ring-offset-2",
    onClick,
    children: buttonLabel || (isLastQuestion ? "Finish" : "Next")
  });
}
function Headline({
  headline,
  questionId,
  required = true,
  alignTextCenter = false
}) {
  return u$1("label", {
    htmlFor: questionId,
    className: "text-heading mb-1.5 block text-base font-semibold leading-6",
    children: u$1("div", {
      className: `flex items-center  ${alignTextCenter ? "justify-center" : "justify-between"}`,
      children: [headline, !required && u$1("span", {
        className: "text-info-text ml-2 self-start text-sm font-normal leading-7",
        tabIndex: -1,
        children: "Optional"
      })]
    })
  });
}
/*!
 * Sanitize an HTML string
 * (c) 2021 Chris Ferdinandi, MIT License, https://gomakethings.com
 * @param  {String}          str   The HTML string to sanitize
 * @return {String}          The sanitized string
 */
function cleanHtml(str) {
  function stringToHTML() {
    let parser = new DOMParser();
    let doc = parser.parseFromString(str, "text/html");
    return doc.body || document.createElement("body");
  }
  function removeScripts(html2) {
    let scripts = html2.querySelectorAll("script");
    scripts.forEach((script) => {
      script.remove();
    });
  }
  function isPossiblyDangerous(name, value) {
    let val = value.replace(/\s+/g, "").toLowerCase();
    if (["src", "href", "xlink:href", "srcdoc"].includes(name) && (val.includes("javascript:") || val.includes("data:") || val.includes("<script>"))) {
      return true;
    }
    if (name.startsWith("on")) {
      return true;
    }
    return false;
  }
  function removeAttributes(elem) {
    let atts = elem.attributes;
    for (let i2 = atts.length - 1; i2 >= 0; i2--) {
      let {
        name,
        value
      } = atts[i2];
      if (isPossiblyDangerous(name, value)) {
        elem.removeAttribute(name);
      } else if (name === "srcdoc") {
        elem.setAttribute(name, cleanHtml(value));
      }
    }
  }
  function clean(html2) {
    let nodes = Array.from(html2.children);
    for (let node of nodes) {
      removeAttributes(node);
      clean(node);
    }
  }
  let html = stringToHTML();
  removeScripts(html);
  clean(html);
  return html.innerHTML;
}
function HtmlBody({
  htmlString,
  questionId
}) {
  if (!htmlString)
    return null;
  return u$1("label", {
    htmlFor: questionId,
    className: "fb-htmlbody",
    dangerouslySetInnerHTML: {
      __html: cleanHtml(htmlString)
    }
  });
}
function QuestionImage({
  imgUrl,
  altText = "Image"
}) {
  return u$1("div", {
    className: "mb-4 rounded-md",
    children: u$1("img", {
      src: imgUrl,
      alt: altText,
      className: "mb-4 rounded-md"
    })
  });
}
function g(n2, t2) {
  for (var e2 in t2)
    n2[e2] = t2[e2];
  return n2;
}
function C(n2, t2) {
  for (var e2 in n2)
    if ("__source" !== e2 && !(e2 in t2))
      return true;
  for (var r2 in t2)
    if ("__source" !== r2 && n2[r2] !== t2[r2])
      return true;
  return false;
}
function E(n2) {
  this.props = n2;
}
(E.prototype = new b$1()).isPureReactComponent = true, E.prototype.shouldComponentUpdate = function(n2, t2) {
  return C(this.props, n2) || C(this.state, t2);
};
var x = l$2.__b;
l$2.__b = function(n2) {
  n2.type && n2.type.__f && n2.ref && (n2.props.ref = n2.ref, n2.ref = null), x && x(n2);
};
var O = l$2.__e;
l$2.__e = function(n2, t2, e2, r2) {
  if (n2.then) {
    for (var u2, o2 = t2; o2 = o2.__; )
      if ((u2 = o2.__c) && u2.__c)
        return null == t2.__e && (t2.__e = e2.__e, t2.__k = e2.__k), u2.__c(n2, t2);
  }
  O(n2, t2, e2, r2);
};
var T = l$2.unmount;
function F(n2, t2, e2) {
  return n2 && (n2.__c && n2.__c.__H && (n2.__c.__H.__.forEach(function(n3) {
    "function" == typeof n3.__c && n3.__c();
  }), n2.__c.__H = null), null != (n2 = g({}, n2)).__c && (n2.__c.__P === e2 && (n2.__c.__P = t2), n2.__c = null), n2.__k = n2.__k && n2.__k.map(function(n3) {
    return F(n3, t2, e2);
  })), n2;
}
function I(n2, t2, e2) {
  return n2 && e2 && (n2.__v = null, n2.__k = n2.__k && n2.__k.map(function(n3) {
    return I(n3, t2, e2);
  }), n2.__c && n2.__c.__P === t2 && (n2.__e && e2.appendChild(n2.__e), n2.__c.__e = true, n2.__c.__P = e2)), n2;
}
function L() {
  this.__u = 0, this.t = null, this.__b = null;
}
function U(n2) {
  var t2 = n2.__.__c;
  return t2 && t2.__a && t2.__a(n2);
}
function M() {
  this.u = null, this.o = null;
}
l$2.unmount = function(n2) {
  var t2 = n2.__c;
  t2 && t2.__R && t2.__R(), t2 && 32 & n2.__u && (n2.type = null), T && T(n2);
}, (L.prototype = new b$1()).__c = function(n2, t2) {
  var e2 = t2.__c, r2 = this;
  null == r2.t && (r2.t = []), r2.t.push(e2);
  var u2 = U(r2.__v), o2 = false, i2 = function() {
    o2 || (o2 = true, e2.__R = null, u2 ? u2(l2) : l2());
  };
  e2.__R = i2;
  var l2 = function() {
    if (!--r2.__u) {
      if (r2.state.__a) {
        var n3 = r2.state.__a;
        r2.__v.__k[0] = I(n3, n3.__c.__P, n3.__c.__O);
      }
      var t3;
      for (r2.setState({ __a: r2.__b = null }); t3 = r2.t.pop(); )
        t3.forceUpdate();
    }
  };
  r2.__u++ || 32 & t2.__u || r2.setState({ __a: r2.__b = r2.__v.__k[0] }), n2.then(i2, i2);
}, L.prototype.componentWillUnmount = function() {
  this.t = [];
}, L.prototype.render = function(n2, e2) {
  if (this.__b) {
    if (this.__v.__k) {
      var r2 = document.createElement("div"), o2 = this.__v.__k[0].__c;
      this.__v.__k[0] = F(this.__b, r2, o2.__O = o2.__P);
    }
    this.__b = null;
  }
  var i2 = e2.__a && y(g$2, null, n2.fallback);
  return i2 && (i2.__u &= -33), [y(g$2, null, e2.__a ? null : n2.children), i2];
};
var V = function(n2, t2, e2) {
  if (++e2[1] === e2[0] && n2.o.delete(t2), n2.props.revealOrder && ("t" !== n2.props.revealOrder[0] || !n2.o.size))
    for (e2 = n2.u; e2; ) {
      for (; e2.length > 3; )
        e2.pop()();
      if (e2[1] < e2[0])
        break;
      n2.u = e2 = e2[2];
    }
};
(M.prototype = new b$1()).__a = function(n2) {
  var t2 = this, e2 = U(t2.__v), r2 = t2.o.get(n2);
  return r2[0]++, function(u2) {
    var o2 = function() {
      t2.props.revealOrder ? (r2.push(u2), V(t2, n2, r2)) : u2();
    };
    e2 ? e2(o2) : o2();
  };
}, M.prototype.render = function(n2) {
  this.u = null, this.o = /* @__PURE__ */ new Map();
  var t2 = $$1(n2.children);
  n2.revealOrder && "b" === n2.revealOrder[0] && t2.reverse();
  for (var e2 = t2.length; e2--; )
    this.o.set(t2[e2], this.u = [1, 0, this.u]);
  return n2.children;
}, M.prototype.componentDidUpdate = M.prototype.componentDidMount = function() {
  var n2 = this;
  this.o.forEach(function(t2, e2) {
    V(n2, e2, t2);
  });
};
var z$1 = "undefined" != typeof Symbol && Symbol.for && Symbol.for("react.element") || 60103, B = /^(?:accent|alignment|arabic|baseline|cap|clip(?!PathU)|color|dominant|fill|flood|font|glyph(?!R)|horiz|image(!S)|letter|lighting|marker(?!H|W|U)|overline|paint|pointer|shape|stop|strikethrough|stroke|text(?!L)|transform|underline|unicode|units|v|vector|vert|word|writing|x(?!C))[A-Z]/, H = /^on(Ani|Tra|Tou|BeforeInp|Compo)/, Z = /[A-Z0-9]/g, Y = "undefined" != typeof document, $ = function(n2) {
  return ("undefined" != typeof Symbol && "symbol" == typeof Symbol() ? /fil|che|rad/ : /fil|che|ra/).test(n2);
};
b$1.prototype.isReactComponent = {}, ["componentWillMount", "componentWillReceiveProps", "componentWillUpdate"].forEach(function(t2) {
  Object.defineProperty(b$1.prototype, t2, { configurable: true, get: function() {
    return this["UNSAFE_" + t2];
  }, set: function(n2) {
    Object.defineProperty(this, t2, { configurable: true, writable: true, value: n2 });
  } });
});
var J = l$2.event;
function K() {
}
function Q() {
  return this.cancelBubble;
}
function X() {
  return this.defaultPrevented;
}
l$2.event = function(n2) {
  return J && (n2 = J(n2)), n2.persist = K, n2.isPropagationStopped = Q, n2.isDefaultPrevented = X, n2.nativeEvent = n2;
};
var tn = { enumerable: false, configurable: true, get: function() {
  return this.class;
} }, en = l$2.vnode;
l$2.vnode = function(n2) {
  "string" == typeof n2.type && function(n3) {
    var t2 = n3.props, e2 = n3.type, u2 = {};
    for (var o2 in t2) {
      var i2 = t2[o2];
      if (!("value" === o2 && "defaultValue" in t2 && null == i2 || Y && "children" === o2 && "noscript" === e2 || "class" === o2 || "className" === o2)) {
        var l2 = o2.toLowerCase();
        "defaultValue" === o2 && "value" in t2 && null == t2.value ? o2 = "value" : "download" === o2 && true === i2 ? i2 = "" : "ondoubleclick" === l2 ? o2 = "ondblclick" : "onchange" !== l2 || "input" !== e2 && "textarea" !== e2 || $(t2.type) ? "onfocus" === l2 ? o2 = "onfocusin" : "onblur" === l2 ? o2 = "onfocusout" : H.test(o2) ? o2 = l2 : -1 === e2.indexOf("-") && B.test(o2) ? o2 = o2.replace(Z, "-$&").toLowerCase() : null === i2 && (i2 = void 0) : l2 = o2 = "oninput", "oninput" === l2 && u2[o2 = l2] && (o2 = "oninputCapture"), u2[o2] = i2;
      }
    }
    "select" == e2 && u2.multiple && Array.isArray(u2.value) && (u2.value = $$1(t2.children).forEach(function(n4) {
      n4.props.selected = -1 != u2.value.indexOf(n4.props.value);
    })), "select" == e2 && null != u2.defaultValue && (u2.value = $$1(t2.children).forEach(function(n4) {
      n4.props.selected = u2.multiple ? -1 != u2.defaultValue.indexOf(n4.props.value) : u2.defaultValue == n4.props.value;
    })), t2.class && !t2.className ? (u2.class = t2.class, Object.defineProperty(u2, "className", tn)) : (t2.className && !t2.class || t2.class && t2.className) && (u2.class = u2.className = t2.className), n3.props = u2;
  }(n2), n2.$$typeof = z$1, en && en(n2);
};
var rn = l$2.__r;
l$2.__r = function(n2) {
  rn && rn(n2), n2.__c;
};
var un = l$2.diffed;
l$2.diffed = function(n2) {
  un && un(n2);
  var t2 = n2.props, e2 = n2.__e;
  null != e2 && "textarea" === n2.type && "value" in t2 && t2.value !== e2.value && (e2.value = null == t2.value ? "" : t2.value);
};
const getUpdatedTtc = (ttc, questionId, time) => {
  if (ttc.hasOwnProperty(questionId)) {
    return {
      ...ttc,
      [questionId]: ttc[questionId] + time
    };
  } else {
    return {
      ...ttc,
      [questionId]: time
    };
  }
};
const useTtc = (questionId, ttc, setTtc, startTime, setStartTime) => {
  p(() => {
    setStartTime(performance.now());
  }, [questionId]);
  p(() => {
    const handleVisibilityChange = () => {
      if (document.visibilityState === "visible") {
        setStartTime(performance.now());
      } else {
        const updatedTtc = getUpdatedTtc(ttc, questionId, performance.now() - startTime);
        setTtc(updatedTtc);
      }
    };
    document.addEventListener("visibilitychange", handleVisibilityChange);
    return () => {
      document.removeEventListener("visibilitychange", handleVisibilityChange);
    };
  }, []);
};
function CTAQuestion({
  question,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  return u$1("div", {
    children: [question.imageUrl && u$1(QuestionImage, {
      imgUrl: question.imageUrl
    }), u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(HtmlBody, {
      htmlString: question.html,
      questionId: question.id
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        backButtonLabel: question.backButtonLabel,
        onClick: () => {
          const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
          setTtc(updatedTtcObj);
          onSubmit({
            [question.id]: ""
          }, updatedTtcObj);
          onBack();
        }
      }), u$1("div", {
        className: "flex w-full justify-end",
        children: [!question.required && u$1("button", {
          tabIndex: 0,
          type: "button",
          onClick: () => {
            const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
            setTtc(updatedTtcObj);
            onSubmit({
              [question.id]: "dismissed"
            }, updatedTtcObj);
          },
          className: "text-heading focus:ring-focus mr-4 flex items-center rounded-md px-3 py-3 text-base font-medium leading-4 hover:opacity-90 focus:outline-none focus:ring-2 focus:ring-offset-2",
          children: question.dismissButtonLabel || "Skip"
        }), u$1(SubmitButton, {
          buttonLabel: question.buttonLabel,
          isLastQuestion,
          focus: true,
          onClick: () => {
            var _a;
            if (question.buttonExternal && question.buttonUrl) {
              (_a = window == null ? void 0 : window.open(question.buttonUrl, "_blank")) == null ? void 0 : _a.focus();
            }
            const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
            setTtc(updatedTtcObj);
            onSubmit({
              [question.id]: "clicked"
            }, updatedTtcObj);
          },
          type: "button"
        })]
      })]
    })]
  });
}
const l = "https://app.cal.com/embed/embed.js";
function r(d2 = l) {
  (function(a2, p2, s2) {
    let i2 = function(t2, n2) {
      t2.q.push(n2);
    }, o2 = a2.document;
    a2.Cal = a2.Cal || function() {
      let t2 = a2.Cal, n2 = arguments;
      if (t2.loaded || (t2.ns = {}, t2.q = t2.q || [], o2.head.appendChild(o2.createElement("script")).src = p2, t2.loaded = true), n2[0] === s2) {
        const e2 = function() {
          i2(e2, arguments);
        }, c2 = n2[1];
        e2.q = e2.q || [], typeof c2 == "string" ? (
          // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
          (t2.ns[c2] = e2) && i2(e2, n2)
        ) : i2(t2, n2);
        return;
      }
      i2(t2, n2);
    };
  })(
    window,
    //! Replace it with "https://cal.com/embed.js" or the URL where you have embed.js installed
    d2,
    "init"
  );
  /*!  Copying ends here. */
  return window.Cal;
}
r.toString();
function CalEmbed({
  question,
  onSuccessfulBooking
}) {
  const cal = F$1(() => {
    const calInline = r("https://cal.com/embed.js");
    const calCssVars = {
      "cal-border-subtle": "transparent",
      "cal-border-booker": "transparent"
    };
    calInline("ui", {
      theme: "light",
      cssVarsPerTheme: {
        light: {
          ...calCssVars
        },
        dark: {
          "cal-bg-muted": "transparent",
          "cal-bg": "transparent",
          ...calCssVars
        }
      }
    });
    calInline("on", {
      action: "bookingSuccessful",
      callback: () => {
        onSuccessfulBooking();
      }
    });
    return calInline;
  }, [onSuccessfulBooking]);
  p(() => {
    document.querySelectorAll("cal-inline").forEach((el) => el.remove());
    cal("inline", {
      elementOrSelector: "#fb-cal-embed",
      calLink: question.calUserName
    });
  }, [cal, question.calUserName]);
  return u$1("div", {
    className: "relative mt-4",
    children: u$1("div", {
      id: "fb-cal-embed",
      className: cn("h-96 overflow-auto rounded-lg border border-slate-200")
    })
  });
}
function Subheader({
  subheader,
  questionId
}) {
  return u$1("label", {
    htmlFor: questionId,
    className: "text-subheading block text-sm font-normal leading-6",
    children: subheader
  });
}
function CalQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  const [errorMessage, setErrorMessage] = h("");
  const onSuccessfulBooking = T$1(() => {
    onChange({
      [question.id]: "booked"
    });
    const updatedttc = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
    setTtc(updatedttc);
    onSubmit({
      [question.id]: "booked"
    }, updatedttc);
  }, [onChange, onSubmit, question.id, setTtc, startTime, ttc]);
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      if (question.required && !value) {
        setErrorMessage("Please book an appointment");
        return;
      }
      const updatedttc = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedttc);
      onChange({
        [question.id]: value
      });
      onSubmit({
        [question.id]: value
      }, updatedttc);
    },
    className: "w-full",
    children: [u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1(g$2, {
      children: [errorMessage && u$1("span", {
        className: "text-red-500",
        children: errorMessage
      }), u$1(CalEmbed, {
        question,
        onSuccessfulBooking
      }, question.id)]
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        backButtonLabel: question.backButtonLabel,
        onClick: () => {
          onBack();
        }
      }), u$1("div", {}), !question.required && u$1(SubmitButton, {
        buttonLabel: question.buttonLabel,
        isLastQuestion,
        onClick: () => {
        }
      })]
    })]
  });
}
function ConsentQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  return u$1("div", {
    children: [question.imageUrl && u$1(QuestionImage, {
      imgUrl: question.imageUrl
    }), u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(HtmlBody, {
      htmlString: question.html || "",
      questionId: question.id
    }), u$1("form", {
      onSubmit: (e2) => {
        e2.preventDefault();
        const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
        setTtc(updatedTtcObj);
        onSubmit({
          [question.id]: value
        }, updatedTtcObj);
      },
      children: [u$1("label", {
        tabIndex: 1,
        onKeyDown: (e2) => {
          if (e2.key == "Enter") {
            onChange({
              [question.id]: "accepted"
            });
          }
        },
        className: "border-border bg-survey-bg text-heading hover:bg-accent-bg focus:bg-accent-bg focus:ring-border-highlight relative z-10 mt-4 flex w-full cursor-pointer items-center rounded-md border p-4 text-sm focus:outline-none focus:ring-2 focus:ring-offset-2",
        children: [u$1("input", {
          type: "checkbox",
          id: question.id,
          name: question.id,
          value: question.label,
          onChange: (e2) => {
            if (e2.target instanceof HTMLInputElement && e2.target.checked) {
              onChange({
                [question.id]: "accepted"
              });
            } else {
              onChange({
                [question.id]: "dismissed"
              });
            }
          },
          checked: value === "accepted",
          className: "border-brand text-brand h-4 w-4 border focus:ring-0 focus:ring-offset-0",
          "aria-labelledby": `${question.id}-label`,
          required: question.required
        }), u$1("span", {
          id: `${question.id}-label`,
          className: "ml-3 font-medium",
          children: question.label
        })]
      }), u$1("div", {
        className: "mt-4 flex w-full justify-between",
        children: [!isFirstQuestion && u$1(BackButton, {
          tabIndex: 3,
          backButtonLabel: question.backButtonLabel,
          onClick: () => {
            const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
            setTtc(updatedTtcObj);
            onSubmit({
              [question.id]: value
            }, updatedTtcObj);
            onBack();
          }
        }), u$1("div", {}), u$1(SubmitButton, {
          tabIndex: 2,
          buttonLabel: question.buttonLabel,
          isLastQuestion,
          onClick: () => {
          }
        })]
      })]
    })]
  });
}
var define_process_env_default = { SURVEYS_PACKAGE_BUILD: "surveys", NVM_INC: "/Users/matthiasnannt/.nvm/versions/node/v20.10.0/include/node", MANPATH: "/Users/matthiasnannt/.nvm/versions/node/v20.10.0/share/man:/opt/homebrew/share/man:/usr/share/man:/usr/local/share/man:/Users/matthiasnannt/.nvm/versions/node/v20.10.0/share/man:/opt/homebrew/share/man::", npm_config_legacy_peer_deps: "true", TERM_PROGRAM: "vscode", NODE: "/Users/matthiasnannt/Library/pnpm/pnpm", npm_package_homepage: "https://formbricks.com", INIT_CWD: "/Users/matthiasnannt/Developer/formbricks/packages/surveys", NVM_CD_FLAGS: "-q", TURBO_INVOCATION_DIR: "/Users/matthiasnannt/Developer/formbricks", _P9K_TTY: "/dev/ttys005", npm_package_devDependencies_vite: "^5.0.10", SHELL: "/bin/zsh", TERM: "xterm-256color", npm_config_shamefully_hoist: "true", HOMEBREW_REPOSITORY: "/opt/homebrew", TMPDIR: "/var/folders/zk/_n2g5c910zzccj1m8q1mg58h0000gn/T/", npm_package_devDependencies_eslint_config_turbo: "latest", npm_package_scripts_lint: "eslint . --ext .ts,.js,.tsx,.jsx", CONDA_SHLVL: "1", npm_package_devDependencies_concurrently: "8.2.2", CONDA_PROMPT_MODIFIER: "(base) ", TERM_PROGRAM_VERSION: "1.85.1", npm_package_scripts_dev: "SURVEYS_PACKAGE_MODE=development vite build --watch", MallocNanoZone: "0", ORIGINAL_XDG_CURRENT_DESKTOP: "undefined", ZDOTDIR: "/Users/matthiasnannt", npm_package_devDependencies__formbricks_types: "workspace:*", npm_config_registry: "https://registry.npmjs.org/", npm_package_sideEffects: "false", npm_package_devDependencies__formbricks_lib: "workspace:*", npm_package_repository_url: "https://github.com/formbricks/formbricks", PNPM_HOME: "/Users/matthiasnannt/Library/pnpm", ZSH: "/Users/matthiasnannt/.oh-my-zsh", npm_package_scripts_build_question_date: "tsc && SURVEYS_PACKAGE_BUILD=question-date vite build", npm_package_description: "Formbricks-surveys is a helper library to embed surveys into your application", NVM_DIR: "/Users/matthiasnannt/.nvm", USER: "matthiasnannt", npm_package_license: "MIT", COMMAND_MODE: "unix2003", npm_package_exports___import: "./dist/index.mjs", CONDA_EXE: "/Users/matthiasnannt/anaconda3/bin/conda", PNPM_SCRIPT_SRC_DIR: "/Users/matthiasnannt/Developer/formbricks/packages/surveys", npm_package_devDependencies_vite_tsconfig_paths: "^4.2.3", SSH_AUTH_SOCK: "/private/tmp/com.apple.launchd.0aMot443u0/Listeners", npm_package_devDependencies_postcss: "^8.4.32", __CF_USER_TEXT_ENCODING: "0x1F5:0x0:0x0", npm_execpath: "/Users/matthiasnannt/Library/pnpm/pnpm", npm_package_scripts_go: 'concurrently "pnpm dev" "serve dist -p 3003"', npm_package_devDependencies_preact: "^10.19.3", npm_package_module: "./dist/index.mjs", PAGER: "less", npm_config_frozen_lockfile: "", LSCOLORS: "Gxfxcxdxbxegedabagacad", _CE_CONDA: "", npm_config_access: "public", npm_package_devDependencies_react_date_picker: "^10.6.0", npm_package_devDependencies__preact_preset_vite: "^2.8.1", PATH: "/Users/matthiasnannt/Developer/formbricks/packages/surveys/node_modules/.bin:/snapshot/dist/node-gyp-bin:/Users/matthiasnannt/Developer/formbricks/node_modules/.bin:/Users/matthiasnannt/Developer/formbricks/packages/surveys/node_modules/.bin:/snapshot/dist/node-gyp-bin:/Users/matthiasnannt/Developer/formbricks/node_modules/.bin:/Users/matthiasnannt/Developer/formbricks/node_modules/.bin:/snapshot/dist/node-gyp-bin:/Users/matthiasnannt/Developer/formbricks/node_modules/.bin:/Users/matthiasnannt/.docker/bin:/Users/matthiasnannt/Library/pnpm:/Users/matthiasnannt/.yarn/bin:/Users/matthiasnannt/.config/yarn/global/node_modules/.bin:/Users/matthiasnannt/anaconda3/bin:/Users/matthiasnannt/.nvm/versions/node/v20.10.0/bin:/opt/homebrew/bin:/opt/homebrew/sbin:/usr/local/bin:/System/Cryptexes/App/usr/bin:/usr/bin:/bin:/usr/sbin:/sbin:/var/run/com.apple.security.cryptexd/codex.system/bootstrap/usr/local/bin:/var/run/com.apple.security.cryptexd/codex.system/bootstrap/usr/bin:/var/run/com.apple.security.cryptexd/codex.system/bootstrap/usr/appleinternal/bin:/Library/Apple/usr/bin", npm_config_auto_install_peers: "true", CONDA_PREFIX: "/Users/matthiasnannt/anaconda3", USER_ZDOTDIR: "/Users/matthiasnannt", __CFBundleIdentifier: "com.microsoft.VSCode", npm_command: "run-script", npm_package_devDependencies_terser: "^5.26.0", npm_package_devDependencies_tailwindcss: "^3.4.0", PWD: "/Users/matthiasnannt/Developer/formbricks/packages/surveys", npm_package_scripts_preview: "vite preview", npm_lifecycle_event: "build:surveys", P9K_SSH: "0", npm_package_types: "./dist/index.d.ts", npm_package_repository_type: "git", npm_package_name: "@formbricks/surveys", LANG: "en_US.UTF-8", P9K_TTY: "old", npm_package_scripts_build: "pnpm run build:surveys && pnpm run build:question-date", npm_config_node_linker: "hoisted", TURBO_HASH: "0916a53b9f093b5c", VSCODE_GIT_ASKPASS_EXTRA_ARGS: "--ms-enable-electron-run-as-node", XPC_FLAGS: "0x0", npm_package_devDependencies_vite_plugin_dts: "^3.7.0", npm_package_devDependencies_eslint_config_prettier: "^9.1.0", npm_package_main: "./dist/index.js", npm_package_version: "1.4.1", XPC_SERVICE_NAME: "0", _CE_M: "", npm_package_devDependencies_autoprefixer: "^10.4.16", VSCODE_INJECTION: "1", HOME: "/Users/matthiasnannt", SHLVL: "4", npm_package_source: "./src/index.ts", VSCODE_GIT_ASKPASS_MAIN: "/Applications/Visual Studio Code.app/Contents/Resources/app/extensions/git/dist/askpass-main.js", npm_config_enable_pre_post_scripts: "true", npm_package_devDependencies__calcom_embed_snippet: "1.1.2", HOMEBREW_PREFIX: "/opt/homebrew", CONDA_PYTHON_EXE: "/Users/matthiasnannt/anaconda3/bin/python", LESS: "-R", LOGNAME: "matthiasnannt", npm_lifecycle_script: "tsc && SURVEYS_PACKAGE_BUILD=surveys vite build", npm_package_exports___require: "./dist/index.umd.js", VSCODE_GIT_IPC_HANDLE: "/var/folders/zk/_n2g5c910zzccj1m8q1mg58h0000gn/T/vscode-git-4d21f99dcc.sock", npm_config_shared_workspace_shrinkwrap: "true", npm_package_devDependencies__formbricks_tsconfig: "workspace:*", CONDA_DEFAULT_ENV: "base", NVM_BIN: "/Users/matthiasnannt/.nvm/versions/node/v20.10.0/bin", npm_config_user_agent: "pnpm/8.12.0 npm/? node/v18.5.0 darwin arm64", GIT_ASKPASS: "/Applications/Visual Studio Code.app/Contents/Resources/app/extensions/git/dist/askpass.sh", HOMEBREW_CELLAR: "/opt/homebrew/Cellar", INFOPATH: "/opt/homebrew/share/info:/opt/homebrew/share/info:", VSCODE_GIT_ASKPASS_NODE: "/Applications/Visual Studio Code.app/Contents/Frameworks/Code Helper (Plugin).app/Contents/MacOS/Code Helper (Plugin)", npm_config_link_workspace_packages: "true", npm_package_files_0: "dist", npm_config_init_author_name: "Matthias Nannt", npm_package_devDependencies_serve: "14.2.1", npm_package_scripts_clean: "rimraf .turbo node_modules dist", npm_package_scripts_build_surveys: "tsc && SURVEYS_PACKAGE_BUILD=surveys vite build", npm_node_execpath: "/Users/matthiasnannt/Library/pnpm/pnpm", COLORTERM: "truecolor", _: "/Users/matthiasnannt/Developer/formbricks/node_modules/.bin/vite", NODE_ENV: "production" };
function DateQuestion({
  question,
  value,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  onChange,
  setTtc,
  ttc
}) {
  const [startTime, setStartTime] = h(performance.now());
  const [errorMessage, setErrorMessage] = h("");
  const [loading, setLoading] = h(true);
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  const defaultDate = value ? new Date(value) : void 0;
  p(() => {
    if (!window.initDatePicker) {
      const script = document.createElement("script");
      script.src = define_process_env_default.SURVEYS_PACKAGE_MODE === "development" ? "http://localhost:3003/question-date.umd.js" : "https://unpkg.com/@formbricks/surveys@^1.4.0/dist/question-date.umd.js";
      script.async = true;
      document.body.appendChild(script);
      script.onload = () => {
        window.initDatePicker(document.getElementById("date-picker-root"), defaultDate, question.format);
        setLoading(false);
      };
      return () => {
        document.body.removeChild(script);
      };
    } else {
      setLoading(false);
      const datePickerContainer = document.getElementById("datePickerContainer");
      if (datePickerContainer) {
        datePickerContainer.remove();
      }
      window.initDatePicker(document.getElementById("date-picker-root"), defaultDate, question.format);
    }
    return () => {
    };
  }, [question.format, question.id]);
  p(() => {
    window.addEventListener("dateChange", (e2) => {
      const date = e2.detail;
      const timezoneOffset = date.getTimezoneOffset() * 6e4;
      const adjustedDate = new Date(date.getTime() - timezoneOffset);
      const dateString = adjustedDate.toISOString().split("T")[0];
      onChange({
        [question.id]: dateString
      });
    });
  }, [onChange, question.id]);
  p(() => {
    if (value && errorMessage) {
      setErrorMessage("");
    }
  }, [value]);
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      if (question.required && !value) {
        setErrorMessage("Please select a date.");
        return;
      }
      const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedTtcObj);
      onSubmit({
        [question.id]: value
      }, updatedTtcObj);
    },
    className: "w-full",
    children: [u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1("div", {
      className: "text-red-600",
      children: u$1("span", {
        children: errorMessage
      })
    }), u$1("div", {
      className: cn("my-4", errorMessage && "rounded-lg border-2 border-red-500"),
      id: "date-picker-root",
      children: loading && u$1("div", {
        className: "relative flex h-12 w-full cursor-pointer appearance-none items-center justify-center rounded-lg border border-slate-300 bg-white text-left text-base font-normal text-slate-900 hover:bg-slate-50 focus:outline-none focus:ring-2 focus:ring-neutral-900 focus:ring-offset-1",
        children: u$1("span", {
          className: "h-6 w-6 animate-spin rounded-full border-b-2 border-neutral-900",
          style: {
            borderTopColor: "transparent"
          }
        })
      })
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [u$1("div", {
        children: !isFirstQuestion && u$1(BackButton, {
          backButtonLabel: question.backButtonLabel,
          onClick: () => {
            const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
            setTtc(updatedTtcObj);
            onBack();
          }
        })
      }), u$1(SubmitButton, {
        isLastQuestion,
        onClick: () => {
        },
        buttonLabel: question.buttonLabel
      })]
    })]
  });
}
function FileInput({
  allowedFileExtensions,
  surveyId,
  onUploadCallback,
  onFileUpload,
  fileUrls,
  maxSizeInMB,
  allowMultipleFiles
}) {
  const [selectedFiles, setSelectedFiles] = h([]);
  const [isUploading, setIsUploading] = h(false);
  const handleFileUpload = async (file) => {
    if (file) {
      if (maxSizeInMB) {
        const fileBuffer = await file.arrayBuffer();
        const bufferBytes = fileBuffer.byteLength;
        const bufferKB = bufferBytes / 1024;
        if (bufferKB > maxSizeInMB * 1024) {
          alert(`File should be less than ${maxSizeInMB} MB`);
        } else {
          setIsUploading(true);
          try {
            const response = await onFileUpload(file, {
              allowedFileExtensions,
              surveyId
            });
            setSelectedFiles([...selectedFiles, file]);
            setIsUploading(false);
            if (fileUrls) {
              onUploadCallback([...fileUrls, response]);
            } else {
              onUploadCallback([response]);
            }
          } catch (err2) {
            setIsUploading(false);
            if (err2.message === "File size exceeds the 10 MB limit") {
              alert(err2.message);
            } else {
              alert("Upload failed! Please try again.");
            }
          }
        }
      } else {
        setIsUploading(true);
        try {
          const response = await onFileUpload(file, {
            allowedFileExtensions,
            surveyId
          });
          setSelectedFiles([...selectedFiles, file]);
          setIsUploading(false);
          if (fileUrls) {
            onUploadCallback([...fileUrls, response]);
          } else {
            onUploadCallback([response]);
          }
        } catch (err2) {
          setIsUploading(false);
          if (err2.message === "File size exceeds the 10 MB limit") {
            alert(err2.message);
          } else {
            alert("Upload failed! Please try again.");
          }
        }
      }
    } else {
      alert("Please select a file");
    }
  };
  const handleDragOver = (e2) => {
    e2.preventDefault();
    e2.stopPropagation();
    e2.dataTransfer.dropEffect = "copy";
  };
  const handleDrop = async (e2) => {
    e2.preventDefault();
    e2.stopPropagation();
    const files = Array.from(e2.dataTransfer.files);
    if (!allowMultipleFiles && files.length > 1) {
      alert("Only one file can be uploaded at a time.");
      return;
    }
    if (files.length > 0) {
      const validFiles = files.filter((file) => allowedFileExtensions && allowedFileExtensions.length > 0 ? allowedFileExtensions.includes(file.type.substring(file.type.lastIndexOf("/") + 1)) : true);
      if (validFiles.length > 0) {
        const uploadedUrls = [];
        for (const file of validFiles) {
          if (maxSizeInMB) {
            const fileBuffer = await file.arrayBuffer();
            const bufferBytes = fileBuffer.byteLength;
            const bufferKB = bufferBytes / 1024;
            if (bufferKB > maxSizeInMB * 1024) {
              alert(`File should be less than ${maxSizeInMB} MB`);
            } else {
              setIsUploading(true);
              try {
                const response = await onFileUpload(file, {
                  allowedFileExtensions,
                  surveyId
                });
                setSelectedFiles([...selectedFiles, file]);
                uploadedUrls.push(response);
              } catch (err2) {
                setIsUploading(false);
                if (err2.message === "File size exceeds the 10 MB limit") {
                  alert(err2.message);
                } else {
                  alert("Upload failed! Please try again.");
                }
              }
            }
          } else {
            setIsUploading(true);
            try {
              const response = await onFileUpload(file, {
                allowedFileExtensions,
                surveyId
              });
              setSelectedFiles([...selectedFiles, file]);
              uploadedUrls.push(response);
            } catch (err2) {
              setIsUploading(false);
              if (err2.message === "File size exceeds the 10 MB limit") {
                alert(err2.message);
              } else {
                alert("Upload failed! Please try again.");
              }
            }
          }
        }
        setIsUploading(false);
        if (fileUrls) {
          onUploadCallback([...fileUrls, ...uploadedUrls]);
        } else {
          onUploadCallback(uploadedUrls);
        }
      } else {
        alert("no selected files are valid");
      }
    }
  };
  const handleDeleteFile = (index, event) => {
    event.stopPropagation();
    if (fileUrls) {
      const newFiles = [...selectedFiles];
      newFiles.splice(index, 1);
      setSelectedFiles(newFiles);
      const updatedFileUrls = [...fileUrls];
      updatedFileUrls.splice(index, 1);
      onUploadCallback(updatedFileUrls);
    }
  };
  const showUploader = F$1(() => {
    if (isUploading) {
      return false;
    }
    if (allowMultipleFiles) {
      return true;
    }
    if (fileUrls && fileUrls.length > 0) {
      return false;
    }
    return true;
  }, [allowMultipleFiles, fileUrls, isUploading]);
  return u$1("div", {
    className: "items-left relative mt-3 flex w-full cursor-pointer flex-col justify-center rounded-lg border-2 border-dashed border-slate-300 bg-slate-50 hover:bg-slate-100 dark:border-slate-600 dark:bg-slate-700 dark:hover:border-slate-500 dark:hover:bg-slate-800",
    children: [u$1("div", {
      children: fileUrls && (fileUrls == null ? void 0 : fileUrls.map((file, index) => u$1("div", {
        className: "relative m-2 rounded-md bg-slate-200",
        children: [u$1("div", {
          className: "absolute right-0 top-0 m-2",
          children: u$1("div", {
            className: "flex h-5 w-5 items-center justify-center rounded-md bg-slate-100 hover:bg-slate-50",
            children: u$1("svg", {
              xmlns: "http://www.w3.org/2000/svg",
              fill: "none",
              viewBox: "0 0 26 26",
              strokeWidth: 1,
              stroke: "currentColor",
              className: "h-5 text-slate-700 hover:text-slate-900",
              onClick: (e2) => handleDeleteFile(index, e2),
              children: u$1("path", {
                strokeLinecap: "round",
                strokeLinejoin: "round",
                d: "M9 9l10 10m0-10L9 19"
              })
            })
          })
        }), u$1("div", {
          className: "flex flex-col items-center justify-center p-2",
          children: [u$1("svg", {
            xmlns: "http://www.w3.org/2000/svg",
            width: "24",
            height: "24",
            viewBox: "0 0 24 24",
            fill: "none",
            stroke: "currentColor",
            "stroke-width": "2",
            "stroke-linecap": "round",
            "stroke-linejoin": "round",
            class: "lucide lucide-file",
            className: "h-6 text-slate-500",
            children: [u$1("path", {
              d: "M14.5 2H6a2 2 0 0 0-2 2v16a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V7.5L14.5 2z"
            }), u$1("polyline", {
              points: "14 2 14 8 20 8"
            })]
          }), u$1("p", {
            className: "mt-1 text-sm text-slate-600 dark:text-slate-400",
            children: decodeURIComponent(file).split("/").pop()
          })]
        })]
      }, index)))
    }), u$1("div", {
      children: [isUploading && u$1("div", {
        className: "inset-0 flex animate-pulse items-center justify-center rounded-lg bg-slate-100 py-4",
        children: u$1("label", {
          htmlFor: "selectedFile",
          className: "text-sm font-medium text-slate-500",
          children: "Uploading..."
        })
      }), u$1("label", {
        htmlFor: "selectedFile",
        onDragOver: (e2) => handleDragOver(e2),
        onDrop: (e2) => handleDrop(e2),
        children: showUploader && u$1("div", {
          className: "flex flex-col items-center justify-center py-6",
          children: [u$1("svg", {
            xmlns: "http://www.w3.org/2000/svg",
            fill: "none",
            viewBox: "0 0 24 24",
            strokeWidth: 1.5,
            stroke: "currentColor",
            className: "h-6 text-slate-500",
            children: u$1("path", {
              strokeLinecap: "round",
              strokeLinejoin: "round",
              d: "M3 16.5v2.25A2.25 2.25 0 005.25 21h13.5A2.25 2.25 0 0021 18.75V16.5m-13.5-9L12 3m0 0l4.5 4.5M12 3v13.5"
            })
          }), u$1("p", {
            className: "mt-2 text-sm text-slate-500 dark:text-slate-400",
            children: u$1("span", {
              className: "font-medium",
              children: "Click or drag to upload files."
            })
          }), u$1("input", {
            type: "file",
            id: "selectedFile",
            name: "selectedFile",
            accept: allowedFileExtensions == null ? void 0 : allowedFileExtensions.map((ext) => `.${ext}`).join(","),
            className: "hidden",
            onChange: (e2) => {
              const inputElement = e2.target;
              if (inputElement.files) {
                handleFileUpload(inputElement.files[0]);
              }
            }
          })]
        })
      })]
    })]
  });
}
function FileUploadQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  surveyId,
  onFileUpload,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedTtcObj);
      if (question.required) {
        if (value && (typeof value === "string" || Array.isArray(value)) && value.length > 0) {
          onSubmit({
            [question.id]: typeof value === "string" ? [value] : value
          }, updatedTtcObj);
        } else {
          alert("Please upload a file");
        }
      } else {
        if (value) {
          onSubmit({
            [question.id]: typeof value === "string" ? [value] : value
          }, updatedTtcObj);
        } else {
          onSubmit({
            [question.id]: "skipped"
          }, updatedTtcObj);
        }
      }
    },
    className: "w-full",
    children: [u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1(FileInput, {
      surveyId,
      onFileUpload,
      onUploadCallback: (urls) => {
        if (urls) {
          onChange({
            [question.id]: urls
          });
        } else {
          onChange({
            [question.id]: "skipped"
          });
        }
      },
      fileUrls: value,
      allowMultipleFiles: question.allowMultipleFiles,
      ...!!question.allowedFileExtensions ? {
        allowedFileExtensions: question.allowedFileExtensions
      } : {},
      ...!!question.maxSizeInMB ? {
        maxSizeInMB: question.maxSizeInMB
      } : {}
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        backButtonLabel: question.backButtonLabel,
        onClick: () => {
          onBack();
        }
      }), u$1("div", {}), u$1(SubmitButton, {
        buttonLabel: question.buttonLabel,
        isLastQuestion,
        onClick: () => {
        }
      })]
    })]
  });
}
function MultipleChoiceMultiQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  const getChoicesWithoutOtherLabels = T$1(() => question.choices.filter((choice) => choice.id !== "other").map((item) => item.label), [question]);
  const [otherSelected, setOtherSelected] = h(false);
  const [otherValue, setOtherValue] = h("");
  p(() => {
    setOtherSelected(!!value && (Array.isArray(value) ? value : [value]).some((item) => {
      return getChoicesWithoutOtherLabels().includes(item) === false;
    }));
    setOtherValue(Array.isArray(value) && value.filter((v2) => !question.choices.find((c2) => c2.label === v2))[0] || "");
  }, [question.id]);
  const questionChoices = F$1(() => {
    if (!question.choices) {
      return [];
    }
    const choicesWithoutOther = question.choices.filter((choice) => choice.id !== "other");
    if (question.shuffleOption) {
      return shuffleQuestions(choicesWithoutOther, question.shuffleOption);
    }
    return choicesWithoutOther;
  }, [question.choices, question.shuffleOption]);
  const questionChoiceLabels = questionChoices.map((questionChoice) => {
    return questionChoice.label;
  });
  const otherOption = F$1(() => question.choices.find((choice) => choice.id === "other"), [question.choices]);
  const otherSpecify = _(null);
  p(() => {
    var _a;
    if (otherSelected) {
      (_a = otherSpecify.current) == null ? void 0 : _a.focus();
    }
  }, [otherSelected]);
  const addItem = (item) => {
    const isOtherValue = !questionChoiceLabels.includes(item);
    if (Array.isArray(value)) {
      if (isOtherValue) {
        const newValue = value.filter((v2) => {
          return questionChoiceLabels.includes(v2);
        });
        return onChange({
          [question.id]: [...newValue, item]
        });
      } else {
        return onChange({
          [question.id]: [...value, item]
        });
      }
    }
    return onChange({
      [question.id]: [item]
    });
  };
  const removeItem = (item) => {
    if (Array.isArray(value)) {
      return onChange({
        [question.id]: value.filter((i2) => i2 !== item)
      });
    }
    return onChange({
      [question.id]: []
    });
  };
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      const newValue = value == null ? void 0 : value.filter((item) => {
        return getChoicesWithoutOtherLabels().includes(item) || item === otherValue;
      });
      onChange({
        [question.id]: newValue
      });
      const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedTtcObj);
      onSubmit({
        [question.id]: value
      }, updatedTtcObj);
    },
    className: "w-full",
    children: [question.imageUrl && u$1(QuestionImage, {
      imgUrl: question.imageUrl
    }), u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1("div", {
      className: "mt-4",
      children: u$1("fieldset", {
        children: [u$1("legend", {
          className: "sr-only",
          children: "Options"
        }), u$1("div", {
          className: "bg-survey-bg relative max-h-[42vh] space-y-2 overflow-y-auto rounded-md py-0.5 pr-2",
          children: [questionChoices.map((choice, idx) => u$1("label", {
            tabIndex: idx + 1,
            onKeyDown: (e2) => {
              if (e2.key == "Enter") {
                if (Array.isArray(value) && value.includes(choice.label)) {
                  removeItem(choice.label);
                } else {
                  addItem(choice.label);
                }
              }
            },
            className: cn(value === choice.label ? "border-border-highlight bg-accent-selected-bg z-10" : "border-border", "text-heading focus-within:border-border-highlight hover:bg-accent-bg focus:bg-accent-bg relative flex cursor-pointer flex-col rounded-md border p-4 focus:outline-none"),
            children: u$1("span", {
              className: "flex items-center text-sm",
              children: [u$1("input", {
                type: "checkbox",
                id: choice.id,
                name: question.id,
                tabIndex: -1,
                value: choice.label,
                className: "border-brand text-brand h-4 w-4 border focus:ring-0 focus:ring-offset-0",
                "aria-labelledby": `${choice.id}-label`,
                onChange: (e2) => {
                  var _a;
                  if ((_a = e2.target) == null ? void 0 : _a.checked) {
                    addItem(choice.label);
                  } else {
                    removeItem(choice.label);
                  }
                },
                checked: Array.isArray(value) && value.includes(choice.label),
                required: question.required && Array.isArray(value) && value.length ? false : question.required
              }), u$1("span", {
                id: `${choice.id}-label`,
                className: "ml-3 font-medium",
                children: choice.label
              })]
            })
          }, choice.id)), otherOption && u$1("label", {
            tabIndex: questionChoices.length + 1,
            className: cn(value === otherOption.label ? "border-border-highlight bg-accent-selected-bg z-10" : "border-border", "text-heading focus-within:border-border-highlight focus-within:bg-accent-bg hover:bg-accent-bg relative flex cursor-pointer flex-col rounded-md border p-4 focus:outline-none"),
            onKeyDown: (e2) => {
              if (e2.key == "Enter") {
                setOtherSelected(!otherSelected);
              }
            },
            children: [u$1("span", {
              className: "flex items-center text-sm",
              children: [u$1("input", {
                type: "checkbox",
                tabIndex: -1,
                id: otherOption.id,
                name: question.id,
                value: otherOption.label,
                className: "border-brand text-brand h-4 w-4 border focus:ring-0 focus:ring-offset-0",
                "aria-labelledby": `${otherOption.id}-label`,
                onChange: (e2) => {
                  var _a;
                  setOtherSelected(!otherSelected);
                  if ((_a = e2.target) == null ? void 0 : _a.checked) {
                    if (!otherValue)
                      return;
                    addItem(otherValue);
                  } else {
                    removeItem(otherValue);
                  }
                },
                checked: otherSelected
              }), u$1("span", {
                id: `${otherOption.id}-label`,
                className: "ml-3 font-medium",
                children: otherOption.label
              })]
            }), otherSelected && u$1("input", {
              ref: otherSpecify,
              id: `${otherOption.id}-label`,
              name: question.id,
              tabIndex: questionChoices.length + 1,
              value: otherValue,
              onChange: (e2) => {
                setOtherValue(e2.currentTarget.value);
                addItem(e2.currentTarget.value);
              },
              onKeyDown: (e2) => {
                if (e2.key == "Enter") {
                  const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
                  setTtc(updatedTtcObj);
                  setTimeout(() => {
                    onSubmit({
                      [question.id]: value
                    }, updatedTtcObj);
                  }, 100);
                }
              },
              placeholder: "Please specify",
              className: "placeholder:text-placeholder border-border bg-survey-bg text-heading focus:ring-focus mt-3 flex h-10 w-full rounded-md border px-3 py-2 text-sm  focus:outline-none focus:ring-2 focus:ring-offset-2 disabled:cursor-not-allowed disabled:opacity-50",
              required: question.required,
              "aria-labelledby": `${otherOption.id}-label`
            })]
          })]
        })]
      })
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        tabIndex: questionChoices.length + 3,
        backButtonLabel: question.backButtonLabel,
        onClick: () => {
          const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
          setTtc(updatedTtcObj);
          onBack();
        }
      }), u$1("div", {}), u$1(SubmitButton, {
        tabIndex: questionChoices.length + 2,
        buttonLabel: question.buttonLabel,
        isLastQuestion,
        onClick: () => {
        }
      })]
    })]
  });
}
function MultipleChoiceSingleQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  const [otherSelected, setOtherSelected] = h(false);
  const questionChoices = F$1(() => {
    if (!question.choices) {
      return [];
    }
    const choicesWithoutOther = question.choices.filter((choice) => choice.id !== "other");
    if (question.shuffleOption) {
      return shuffleQuestions(choicesWithoutOther, question.shuffleOption);
    }
    return choicesWithoutOther;
  }, [question.choices, question.shuffleOption]);
  const otherOption = F$1(() => question.choices.find((choice) => choice.id === "other"), [question.choices]);
  p(() => {
    setOtherSelected(!!value && !question.choices.find((c2) => c2.label === value));
  }, [question.id]);
  const otherSpecify = _(null);
  p(() => {
    var _a;
    if (otherSelected) {
      (_a = otherSpecify.current) == null ? void 0 : _a.focus();
    }
  }, [otherSelected]);
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedTtcObj);
      onSubmit({
        [question.id]: value
      }, updatedTtcObj);
    },
    className: "w-full",
    children: [question.imageUrl && u$1(QuestionImage, {
      imgUrl: question.imageUrl
    }), u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1("div", {
      className: "mt-4",
      children: u$1("fieldset", {
        children: [u$1("legend", {
          className: "sr-only",
          children: "Options"
        }), u$1("div", {
          className: "bg-survey-bg relative max-h-[42vh] space-y-2 overflow-y-auto rounded-md py-0.5 pr-2",
          role: "radiogroup",
          children: [questionChoices.map((choice, idx) => u$1("label", {
            tabIndex: idx + 1,
            onKeyDown: (e2) => {
              if (e2.key == "Enter") {
                onChange({
                  [question.id]: choice.label
                });
                const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
                setTtc(updatedTtcObj);
                setTimeout(() => {
                  onSubmit({
                    [question.id]: choice.label
                  }, updatedTtcObj);
                }, 350);
              }
            },
            className: cn(value === choice.label ? "border-border-highlight bg-accent-selected-bg z-10" : "border-border", "text-heading focus-within:border-border-highlight focus-within:bg-accent-bg hover:bg-accent-bg relative flex cursor-pointer flex-col rounded-md border p-4 focus:outline-none"),
            children: u$1("span", {
              className: "flex items-center text-sm",
              children: [u$1("input", {
                tabIndex: -1,
                type: "radio",
                id: choice.id,
                name: question.id,
                value: choice.label,
                className: "border-brand text-brand h-4 w-4 border focus:ring-0 focus:ring-offset-0",
                "aria-labelledby": `${choice.id}-label`,
                onChange: () => {
                  setOtherSelected(false);
                  onChange({
                    [question.id]: choice.label
                  });
                },
                checked: value === choice.label,
                required: question.required && idx === 0
              }), u$1("span", {
                id: `${choice.id}-label`,
                className: "ml-3 font-medium",
                children: choice.label
              })]
            })
          }, choice.id)), otherOption && u$1("label", {
            tabIndex: questionChoices.length + 1,
            className: cn(value === otherOption.label ? "border-border-highlight bg-accent-selected-bg z-10" : "border-border", "text-heading focus-within:border-border-highlight focus-within:bg-accent-bg hover:bg-accent-bg relative flex cursor-pointer flex-col rounded-md border p-4 focus:outline-none"),
            onKeyDown: (e2) => {
              if (e2.key == "Enter") {
                setOtherSelected(!otherSelected);
                if (!otherSelected)
                  onChange({
                    [question.id]: ""
                  });
              }
            },
            children: [u$1("span", {
              className: "flex items-center text-sm",
              children: [u$1("input", {
                type: "radio",
                id: otherOption.id,
                tabIndex: -1,
                name: question.id,
                value: otherOption.label,
                className: "border-brand text-brand h-4 w-4 border focus:ring-0 focus:ring-offset-0",
                "aria-labelledby": `${otherOption.id}-label`,
                onChange: () => {
                  setOtherSelected(!otherSelected);
                  onChange({
                    [question.id]: ""
                  });
                },
                checked: otherSelected
              }), u$1("span", {
                id: `${otherOption.id}-label`,
                className: "ml-3 font-medium",
                children: otherOption.label
              })]
            }), otherSelected && u$1("input", {
              ref: otherSpecify,
              tabIndex: questionChoices.length + 1,
              id: `${otherOption.id}-label`,
              name: question.id,
              value,
              onChange: (e2) => {
                onChange({
                  [question.id]: e2.currentTarget.value
                });
              },
              onKeyDown: (e2) => {
                if (e2.key == "Enter") {
                  const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
                  setTtc(updatedTtcObj);
                  setTimeout(() => {
                    onSubmit({
                      [question.id]: value
                    }, updatedTtcObj);
                  }, 100);
                }
              },
              placeholder: "Please specify",
              className: "placeholder:text-placeholder border-border bg-survey-bg text-heading focus:ring-focus mt-3 flex h-10 w-full rounded-md border px-3 py-2 text-sm  focus:outline-none focus:ring-2 focus:ring-offset-2 disabled:cursor-not-allowed disabled:opacity-50",
              required: question.required,
              "aria-labelledby": `${otherOption.id}-label`
            })]
          })]
        })]
      })
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        backButtonLabel: question.backButtonLabel,
        tabIndex: questionChoices.length + 3,
        onClick: () => {
          const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
          setTtc(updatedTtcObj);
          onBack();
        }
      }), u$1("div", {}), u$1(SubmitButton, {
        tabIndex: questionChoices.length + 2,
        buttonLabel: question.buttonLabel,
        isLastQuestion,
        onClick: () => {
        }
      })]
    })]
  });
}
function NPSQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedTtcObj);
      onSubmit({
        [question.id]: value
      }, updatedTtcObj);
    },
    children: [question.imageUrl && u$1(QuestionImage, {
      imgUrl: question.imageUrl
    }), u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1("div", {
      className: "my-4",
      children: u$1("fieldset", {
        children: [u$1("legend", {
          className: "sr-only",
          children: "Options"
        }), u$1("div", {
          className: "flex",
          children: Array.from({
            length: 11
          }, (_2, i2) => i2).map((number, idx) => u$1("label", {
            tabIndex: idx + 1,
            onKeyDown: (e2) => {
              if (e2.key == "Enter") {
                const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
                setTtc(updatedTtcObj);
                onSubmit({
                  [question.id]: number
                }, updatedTtcObj);
              }
            },
            className: cn(value === number ? "border-border-highlight bg-accent-selected-bg z-10" : "border-border", "bg-survey-bg text-heading hover:bg-accent-bg relative h-10 flex-1 cursor-pointer border text-center text-sm leading-10 first:rounded-l-md last:rounded-r-md focus:outline-none"),
            children: [u$1("input", {
              type: "radio",
              name: "nps",
              value: number,
              checked: value === number,
              className: "absolute h-full w-full cursor-pointer opacity-0",
              onClick: () => {
                if (question.required) {
                  const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
                  setTtc(updatedTtcObj);
                  onSubmit({
                    [question.id]: number
                  }, updatedTtcObj);
                }
                onChange({
                  [question.id]: number
                });
              },
              required: question.required
            }), number]
          }, number))
        }), u$1("div", {
          className: "text-info-text flex justify-between px-1.5 text-xs leading-6",
          children: [u$1("p", {
            children: question.lowerLabel
          }), u$1("p", {
            children: question.upperLabel
          })]
        })]
      })
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        tabIndex: isLastQuestion ? 12 : 13,
        backButtonLabel: question.backButtonLabel,
        onClick: () => {
          const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
          setTtc(updatedTtcObj);
          onBack();
        }
      }), u$1("div", {}), !question.required && u$1(SubmitButton, {
        tabIndex: 12,
        buttonLabel: question.buttonLabel,
        isLastQuestion,
        onClick: () => {
        }
      })]
    })]
  });
}
function OpenTextQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  autoFocus = true,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  const handleInputChange = (inputValue) => {
    onChange({
      [question.id]: inputValue
    });
  };
  const openTextRef = T$1((currentElement) => {
    if (currentElement && autoFocus) {
      currentElement.focus();
    }
  }, [question.id]);
  const isInputEmpty = (value2) => {
    return question.required && !(value2 == null ? void 0 : value2.trim());
  };
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      const updatedttc = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedttc);
      onSubmit({
        [question.id]: value,
        inputType: question.inputType
      }, updatedttc);
    },
    className: "w-full",
    children: [question.imageUrl && u$1(QuestionImage, {
      imgUrl: question.imageUrl
    }), u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1("div", {
      className: "mt-4",
      children: question.longAnswer === false ? u$1("input", {
        ref: openTextRef,
        tabIndex: 1,
        name: question.id,
        id: question.id,
        placeholder: question.placeholder,
        required: question.required,
        value: value ? value : "",
        type: question.inputType,
        onInput: (e2) => handleInputChange(e2.currentTarget.value),
        autoFocus,
        className: "border-border bg-survey-bg focus:border-border-highlight block w-full rounded-md border p-2 shadow-sm focus:outline-none focus:ring-0 sm:text-sm",
        onKeyDown: (e2) => {
          if (e2.key === "Enter" && isInputEmpty(value)) {
            e2.preventDefault();
          } else if (e2.key === "Enter") {
            const updatedttc = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
            setTtc(updatedttc);
            onSubmit({
              [question.id]: value,
              inputType: question.inputType
            }, updatedttc);
          }
        },
        pattern: question.inputType === "phone" ? "[+][0-9 ]+" : ".*",
        title: question.inputType === "phone" ? "Enter a valid phone number" : void 0
      }) : u$1("textarea", {
        ref: openTextRef,
        rows: 3,
        name: question.id,
        tabIndex: 1,
        id: question.id,
        placeholder: question.placeholder,
        required: question.required,
        value,
        type: question.inputType,
        onInput: (e2) => handleInputChange(e2.currentTarget.value),
        autoFocus,
        className: "border-border bg-survey-bg text-subheading focus:border-border-highlight block w-full rounded-md border p-2 shadow-sm focus:ring-0 sm:text-sm",
        pattern: question.inputType === "phone" ? "[+][0-9 ]+" : ".*",
        title: question.inputType === "phone" ? "Please enter a valid phone number" : void 0
      })
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        backButtonLabel: question.backButtonLabel,
        onClick: () => {
          const updatedttc = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
          setTtc(updatedttc);
          onBack();
        }
      }), u$1("div", {}), u$1(SubmitButton, {
        buttonLabel: question.buttonLabel,
        isLastQuestion,
        onClick: () => {
        }
      })]
    })]
  });
}
function PictureSelectionQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  ttc,
  setTtc
}) {
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  const addItem = (item) => {
    let values = [];
    if (question.allowMulti) {
      if (Array.isArray(value)) {
        values = [...value, item];
      } else {
        values = [item];
      }
    } else {
      values = [item];
    }
    return onChange({
      [question.id]: values
    });
  };
  const removeItem = (item) => {
    let values = [];
    if (question.allowMulti) {
      if (Array.isArray(value)) {
        values = value.filter((i2) => i2 !== item);
      } else {
        values = [];
      }
    } else {
      values = [];
    }
    return onChange({
      [question.id]: values
    });
  };
  const handleChange = (id) => {
    if (Array.isArray(value) && value.includes(id)) {
      removeItem(id);
    } else {
      addItem(id);
    }
  };
  p(() => {
    if (!question.allowMulti && Array.isArray(value) && value.length > 1) {
      onChange({
        [question.id]: []
      });
    }
  }, [question.allowMulti]);
  const questionChoices = question.choices;
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedTtcObj);
      onSubmit({
        [question.id]: value
      }, updatedTtcObj);
    },
    className: "w-full",
    children: [question.imageUrl && u$1(QuestionImage, {
      imgUrl: question.imageUrl
    }), u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1("div", {
      className: "mt-4",
      children: u$1("fieldset", {
        children: [u$1("legend", {
          className: "sr-only",
          children: "Options"
        }), u$1("div", {
          className: "rounded-m bg-survey-bg relative grid max-h-[42vh] grid-cols-2 gap-x-5 gap-y-4 overflow-y-auto pr-2.5",
          children: questionChoices.map((choice, idx) => u$1("label", {
            tabIndex: idx + 1,
            htmlFor: choice.id,
            onKeyDown: (e2) => {
              if (e2.key == "Enter") {
                handleChange(choice.id);
              }
            },
            onClick: () => handleChange(choice.id),
            className: cn(Array.isArray(value) && value.includes(choice.id) ? `border-brand text-brand z-10 border-4 shadow-xl focus:border-4` : "", "border-border focus:border-border-highlight focus:bg-accent-selected-bg relative box-border inline-block h-28 w-full overflow-hidden rounded-xl border focus:outline-none"),
            children: [u$1("img", {
              src: choice.imageUrl,
              id: choice.id,
              alt: choice.imageUrl.split("/").pop(),
              className: "h-full w-full object-cover"
            }), question.allowMulti ? u$1("input", {
              id: `${choice.id}-checked`,
              name: `${choice.id}-checkbox`,
              type: "checkbox",
              tabIndex: -1,
              checked: Array.isArray(value) && value.includes(choice.id),
              className: cn("border-border pointer-events-none absolute right-2 top-2 z-20 h-5 w-5 rounded border", Array.isArray(value) && value.includes(choice.id) ? "border-brand text-brand" : ""),
              required: question.required && Array.isArray(value) && value.length ? false : question.required
            }) : u$1("input", {
              id: `${choice.id}-radio`,
              name: `${choice.id}-radio`,
              type: "radio",
              tabIndex: -1,
              checked: Array.isArray(value) && value.includes(choice.id),
              className: cn("border-border pointer-events-none absolute right-2 top-2 z-20 h-5 w-5 rounded-full border", Array.isArray(value) && value.includes(choice.id) ? "border-brand text-brand" : ""),
              required: question.required && Array.isArray(value) && value.length ? false : question.required
            })]
          }, choice.id))
        })]
      })
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        tabIndex: questionChoices.length + 3,
        backButtonLabel: question.backButtonLabel,
        onClick: () => {
          const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
          setTtc(updatedTtcObj);
          onBack();
        }
      }), u$1("div", {}), u$1(SubmitButton, {
        tabIndex: questionChoices.length + 2,
        buttonLabel: question.buttonLabel,
        isLastQuestion,
        onClick: () => {
        }
      })]
    })]
  });
}
const TiredFace = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        ...props
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m21.88 23.92c5.102-0.06134 7.273-1.882 8.383-3.346"
      }), u$1("path", {
        stroke: "currentColor",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m46.24 47.56c0-2.592-2.867-7.121-10.25-6.93-6.974 0.1812-10.22 4.518-10.22 7.111s4.271-1.611 10.05-1.492c6.317 0.13 10.43 3.903 10.43 1.311z"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m23.16 28.47c5.215 1.438 5.603 0.9096 8.204 1.207 1.068 0.1221-2.03 2.67-7.282 4.397"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m50.12 23.92c-5.102-0.06134-7.273-1.882-8.383-3.346"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m48.84 28.47c-5.215 1.438-5.603 0.9096-8.204 1.207-1.068 0.1221 2.03 2.67 7.282 4.397"
      })]
    })
  });
};
const WearyFace = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        ...props
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m22.88 23.92c5.102-0.06134 7.273-1.882 8.383-3.346"
      }), u$1("path", {
        stroke: "currentColor",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m46.24 47.56c0-2.592-2.867-7.121-10.25-6.93-6.974 0.1812-10.22 4.518-10.22 7.111s4.271-1.611 10.05-1.492c6.317 0.13 10.43 3.903 10.43 1.311z"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m49.12 23.92c-5.102-0.06134-7.273-1.882-8.383-3.346"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m48.24 30.51c-6.199 1.47-7.079 1.059-8.868-1.961"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "m23.76 30.51c6.199 1.47 7.079 1.059 8.868-1.961"
      })]
    })
  });
};
const PerseveringFace = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        ...props
      }), u$1("line", {
        x1: "44.5361",
        x2: "50.9214",
        y1: "21.4389",
        y2: "24.7158",
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2"
      }), u$1("line", {
        x1: "26.9214",
        x2: "20.5361",
        y1: "21.4389",
        y2: "24.7158",
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "M24,28c2.3334,1.3333,4.6666,2.6667,7,4c-2.3334,1.3333-4.6666,2.6667-7,4"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "M48,28c-2.3334,1.3333-4.6666,2.6667-7,4c2.3334,1.3333,4.6666,2.6667,7,4"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "M28,51c0.2704-0.3562,1-8,8.4211-8.0038C43,42.9929,43.6499,50.5372,44,51C38.6667,51,33.3333,51,28,51z"
      })]
    })
  });
};
const FrowningFace = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        ...props
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "M26.5,48c1.8768-3.8326,5.8239-6.1965,10-6c3.8343,0.1804,7.2926,2.4926,9,6"
      }), u$1("path", {
        d: "M30,31c0,1.6568-1.3448,3-3,3c-1.6553,0-3-1.3433-3-3c0-1.6552,1.3447-3,3-3C28.6552,28,30,29.3448,30,31"
      }), u$1("path", {
        d: "M48,31c0,1.6568-1.3447,3-3,3s-3-1.3433-3-3c0-1.6552,1.3447-3,3-3S48,29.3448,48,31"
      })]
    })
  });
};
const ConfusedFace = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        ...props
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        d: "m44.7 43.92c-6.328-1.736-11.41-0.906-17.4 1.902"
      }), u$1("path", {
        d: "M30,31c0,1.6568-1.3448,3-3,3c-1.6553,0-3-1.3433-3-3c0-1.6552,1.3447-3,3-3C28.6552,28,30,29.3448,30,31"
      }), u$1("path", {
        d: "M48,31c0,1.6568-1.3447,3-3,3s-3-1.3433-3-3c0-1.6552,1.3447-3,3-3S48,29.3448,48,31"
      })]
    })
  });
};
const NeutralFace = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        ...props
      }), u$1("line", {
        x1: "27",
        x2: "45",
        y1: "43",
        y2: "43",
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2"
      }), u$1("path", {
        d: "M30,31c0,1.6568-1.3448,3-3,3c-1.6553,0-3-1.3433-3-3c0-1.6552,1.3447-3,3-3C28.6552,28,30,29.3448,30,31"
      }), u$1("path", {
        d: "M48,31c0,1.6568-1.3447,3-3,3s-3-1.3433-3-3c0-1.6552,1.3447-3,3-3S48,29.3448,48,31"
      })]
    })
  });
};
const SlightlySmilingFace = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        ...props
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        d: "M45.8149,44.9293 c-2.8995,1.6362-6.2482,2.5699-9.8149,2.5699s-6.9153-0.9336-9.8149-2.5699"
      }), u$1("path", {
        d: "M30,31c0,1.6568-1.3448,3-3,3c-1.6553,0-3-1.3433-3-3c0-1.6552,1.3447-3,3-3C28.6552,28,30,29.3448,30,31"
      }), u$1("path", {
        d: "M48,31c0,1.6568-1.3447,3-3,3s-3-1.3433-3-3c0-1.6552,1.3447-3,3-3S48,29.3448,48,31"
      })]
    })
  });
};
const SmilingFaceWithSmilingEyes = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        ...props
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        d: "M45.8147,45.2268a15.4294,15.4294,0,0,1-19.6294,0"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "M31.6941,33.4036a4.7262,4.7262,0,0,0-8.6382,0"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "M48.9441,33.4036a4.7262,4.7262,0,0,0-8.6382,0"
      })]
    })
  });
};
const GrinningFaceWithSmilingEyes = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        ...props
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        d: "M50.595,41.64a11.5554,11.5554,0,0,1-.87,4.49c-12.49,3.03-25.43.34-27.49-.13a11.4347,11.4347,0,0,1-.83-4.36h.11s14.8,3.59,28.89.07Z"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        d: "M49.7251,46.13c-1.79,4.27-6.35,7.23-13.69,7.23-7.41,0-12.03-3.03-13.8-7.36C24.2951,46.47,37.235,49.16,49.7251,46.13Z"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "M31.6941,32.4036a4.7262,4.7262,0,0,0-8.6382,0"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeMiterlimit: "10",
        strokeWidth: "2",
        d: "M48.9441,32.4036a4.7262,4.7262,0,0,0-8.6382,0"
      })]
    })
  });
};
const GrinningSquintingFace = (props) => {
  return u$1("svg", {
    viewBox: "0 0 72 72",
    xmlns: "http://www.w3.org/2000/svg",
    width: 48,
    height: 48,
    children: u$1("g", {
      id: "line",
      children: [u$1("circle", {
        cx: "36",
        cy: "36",
        r: "23",
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        ...props
      }), u$1("polyline", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        points: "25.168 27.413 31.755 31.427 25.168 35.165"
      }), u$1("polyline", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        points: "46.832 27.413 40.245 31.427 46.832 35.165"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        d: "M50.595,41.64a11.5554,11.5554,0,0,1-.87,4.49c-12.49,3.03-25.43.34-27.49-.13a11.4347,11.4347,0,0,1-.83-4.36h.11s14.8,3.59,28.89.07Z"
      }), u$1("path", {
        fill: "none",
        stroke: "currentColor",
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "2",
        d: "M49.7251,46.13c-1.79,4.27-6.35,7.23-13.69,7.23-7.41,0-12.03-3.03-13.8-7.36C24.2951,46.47,37.235,49.16,49.7251,46.13Z"
      })]
    })
  });
};
[u$1("svg", {
  viewBox: "0 0 72 72",
  xmlns: "http://www.w3.org/2000/svg",
  width: 48,
  height: 48
})];
function RatingQuestion({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  ttc,
  setTtc
}) {
  const [hoveredNumber, setHoveredNumber] = h(0);
  const [startTime, setStartTime] = h(performance.now());
  useTtc(question.id, ttc, setTtc, startTime, setStartTime);
  const handleSelect = (number) => {
    onChange({
      [question.id]: number
    });
    if (question.required) {
      const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedTtcObj);
      onSubmit({
        [question.id]: number
      }, updatedTtcObj);
    }
  };
  const HiddenRadioInput = ({
    number
  }) => u$1("input", {
    type: "radio",
    name: "rating",
    value: number,
    className: "absolute left-0 h-full w-full cursor-pointer opacity-0",
    onChange: () => handleSelect(number),
    required: question.required,
    checked: value === number
  });
  return u$1("form", {
    onSubmit: (e2) => {
      e2.preventDefault();
      const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
      setTtc(updatedTtcObj);
      onSubmit({
        [question.id]: value
      }, updatedTtcObj);
    },
    className: "w-full",
    children: [question.imageUrl && u$1(QuestionImage, {
      imgUrl: question.imageUrl
    }), u$1(Headline, {
      headline: question.headline,
      questionId: question.id,
      required: question.required
    }), u$1(Subheader, {
      subheader: question.subheader,
      questionId: question.id
    }), u$1("div", {
      className: "mb-4 mt-6 flex items-center justify-center",
      children: u$1("fieldset", {
        className: "w-full ",
        children: [u$1("legend", {
          className: "sr-only",
          children: "Choices"
        }), u$1("div", {
          className: "flex pb-2",
          children: Array.from({
            length: question.range
          }, (_2, i2) => i2 + 1).map((number, i2, a2) => u$1("span", {
            onMouseOver: () => setHoveredNumber(number),
            onMouseLeave: () => setHoveredNumber(0),
            className: "bg-survey-bg relative flex-1 cursor-pointer text-center text-sm leading-[2.8rem]",
            children: question.scale === "number" ? u$1("label", {
              tabIndex: i2 + 1,
              onKeyDown: (e2) => {
                if (e2.key == "Enter") {
                  handleSelect(number);
                }
              },
              className: cn(value === number ? "bg-accent-selected-bg border-border-highlight z-10" : "", a2.length === number ? "rounded-r-md" : "", number === 1 ? "rounded-l-md" : "", "text-heading hover:bg-accent-bg focus:bg-accent-bg block h-full w-full border focus:outline-none"),
              children: [u$1(HiddenRadioInput, {
                number
              }), number]
            }) : question.scale === "star" ? u$1("label", {
              tabIndex: i2 + 1,
              onKeyDown: (e2) => {
                if (e2.key == "Enter") {
                  handleSelect(number);
                }
              },
              className: cn("flex h-full w-full justify-center focus:outline-none", number <= hoveredNumber ? "text-amber-400" : "text-slate-300", "hover:text-amber-400"),
              onFocus: () => setHoveredNumber(number),
              onBlur: () => setHoveredNumber(0),
              children: [u$1(HiddenRadioInput, {
                number
              }), u$1("svg", {
                xmlns: "http://www.w3.org/2000/svg",
                viewBox: "0 0 24 24",
                fill: "currentColor",
                className: "h-14 max-h-full w-14",
                children: u$1("path", {
                  fillRule: "evenodd",
                  d: "M11.48 3.499a.562.562 0 011.04 0l2.125 5.111a.563.563 0 00.475.345l5.518.442c.499.04.701.663.321.988l-4.204 3.602a.563.563 0 00-.182.557l1.285 5.385a.562.562 0 01-.84.61l-4.725-2.885a.563.563 0 00-.586 0L6.982 20.54a.562.562 0 01-.84-.61l1.285-5.386a.562.562 0 00-.182-.557l-4.204-3.602a.563.563 0 01.321-.988l5.518-.442a.563.563 0 00.475-.345L11.48 3.5z"
                })
              })]
            }) : u$1("label", {
              className: cn("flex h-full w-full justify-center", value === number || hoveredNumber === number ? "stroke-rating-selected text-rating-selected" : "stroke-heading text-heading"),
              tabIndex: i2 + 1,
              onKeyDown: (e2) => {
                if (e2.key == "Enter") {
                  handleSelect(number);
                }
              },
              onFocus: () => setHoveredNumber(number),
              onBlur: () => setHoveredNumber(0),
              children: [u$1(HiddenRadioInput, {
                number
              }), u$1(RatingSmiley, {
                active: value === number || hoveredNumber === number,
                idx: i2,
                range: question.range
              })]
            })
          }, number))
        }), u$1("div", {
          className: "text-subheading flex justify-between px-1.5 text-xs leading-6",
          children: [u$1("p", {
            className: "w-1/2 text-left",
            children: question.lowerLabel
          }), u$1("p", {
            className: "w-1/2 text-right",
            children: question.upperLabel
          })]
        })]
      })
    }), u$1("div", {
      className: "mt-4 flex w-full justify-between",
      children: [!isFirstQuestion && u$1(BackButton, {
        tabIndex: !question.required || value ? question.range + 2 : question.range + 1,
        backButtonLabel: question.backButtonLabel,
        onClick: () => {
          const updatedTtcObj = getUpdatedTtc(ttc, question.id, performance.now() - startTime);
          setTtc(updatedTtcObj);
          onBack();
        }
      }), u$1("div", {}), (!question.required || value) && u$1(SubmitButton, {
        tabIndex: question.range + 1,
        buttonLabel: question.buttonLabel,
        isLastQuestion,
        onClick: () => {
        }
      })]
    })]
  });
}
function RatingSmiley({
  active,
  idx,
  range
}) {
  const activeColor = "fill-rating-fill";
  const inactiveColor = "fill-none";
  let icons = [u$1(TiredFace, {
    className: active ? activeColor : inactiveColor
  }), u$1(WearyFace, {
    className: active ? activeColor : inactiveColor
  }), u$1(PerseveringFace, {
    className: active ? activeColor : inactiveColor
  }), u$1(FrowningFace, {
    className: active ? activeColor : inactiveColor
  }), u$1(ConfusedFace, {
    className: active ? activeColor : inactiveColor
  }), u$1(NeutralFace, {
    className: active ? activeColor : inactiveColor
  }), u$1(SlightlySmilingFace, {
    className: active ? activeColor : inactiveColor
  }), u$1(SmilingFaceWithSmilingEyes, {
    className: active ? activeColor : inactiveColor
  }), u$1(GrinningFaceWithSmilingEyes, {
    className: active ? activeColor : inactiveColor
  }), u$1(GrinningSquintingFace, {
    className: active ? activeColor : inactiveColor
  })];
  if (range == 7)
    icons = [icons[1], icons[3], icons[4], icons[5], icons[6], icons[8], icons[9]];
  else if (range == 5)
    icons = [icons[3], icons[4], icons[5], icons[6], icons[7]];
  else if (range == 4)
    icons = [icons[4], icons[5], icons[6], icons[7]];
  else if (range == 3)
    icons = [icons[4], icons[5], icons[7]];
  return icons[idx];
}
var util;
(function(util2) {
  util2.assertEqual = (val) => val;
  function assertIs(_arg) {
  }
  util2.assertIs = assertIs;
  function assertNever(_x) {
    throw new Error();
  }
  util2.assertNever = assertNever;
  util2.arrayToEnum = (items) => {
    const obj = {};
    for (const item of items) {
      obj[item] = item;
    }
    return obj;
  };
  util2.getValidEnumValues = (obj) => {
    const validKeys = util2.objectKeys(obj).filter((k2) => typeof obj[obj[k2]] !== "number");
    const filtered = {};
    for (const k2 of validKeys) {
      filtered[k2] = obj[k2];
    }
    return util2.objectValues(filtered);
  };
  util2.objectValues = (obj) => {
    return util2.objectKeys(obj).map(function(e2) {
      return obj[e2];
    });
  };
  util2.objectKeys = typeof Object.keys === "function" ? (obj) => Object.keys(obj) : (object) => {
    const keys = [];
    for (const key in object) {
      if (Object.prototype.hasOwnProperty.call(object, key)) {
        keys.push(key);
      }
    }
    return keys;
  };
  util2.find = (arr, checker) => {
    for (const item of arr) {
      if (checker(item))
        return item;
    }
    return void 0;
  };
  util2.isInteger = typeof Number.isInteger === "function" ? (val) => Number.isInteger(val) : (val) => typeof val === "number" && isFinite(val) && Math.floor(val) === val;
  function joinValues(array, separator = " | ") {
    return array.map((val) => typeof val === "string" ? `'${val}'` : val).join(separator);
  }
  util2.joinValues = joinValues;
  util2.jsonStringifyReplacer = (_2, value) => {
    if (typeof value === "bigint") {
      return value.toString();
    }
    return value;
  };
})(util || (util = {}));
var objectUtil;
(function(objectUtil2) {
  objectUtil2.mergeShapes = (first, second) => {
    return {
      ...first,
      ...second
      // second overwrites first
    };
  };
})(objectUtil || (objectUtil = {}));
const ZodParsedType = util.arrayToEnum([
  "string",
  "nan",
  "number",
  "integer",
  "float",
  "boolean",
  "date",
  "bigint",
  "symbol",
  "function",
  "undefined",
  "null",
  "array",
  "object",
  "unknown",
  "promise",
  "void",
  "never",
  "map",
  "set"
]);
const getParsedType = (data) => {
  const t2 = typeof data;
  switch (t2) {
    case "undefined":
      return ZodParsedType.undefined;
    case "string":
      return ZodParsedType.string;
    case "number":
      return isNaN(data) ? ZodParsedType.nan : ZodParsedType.number;
    case "boolean":
      return ZodParsedType.boolean;
    case "function":
      return ZodParsedType.function;
    case "bigint":
      return ZodParsedType.bigint;
    case "symbol":
      return ZodParsedType.symbol;
    case "object":
      if (Array.isArray(data)) {
        return ZodParsedType.array;
      }
      if (data === null) {
        return ZodParsedType.null;
      }
      if (data.then && typeof data.then === "function" && data.catch && typeof data.catch === "function") {
        return ZodParsedType.promise;
      }
      if (typeof Map !== "undefined" && data instanceof Map) {
        return ZodParsedType.map;
      }
      if (typeof Set !== "undefined" && data instanceof Set) {
        return ZodParsedType.set;
      }
      if (typeof Date !== "undefined" && data instanceof Date) {
        return ZodParsedType.date;
      }
      return ZodParsedType.object;
    default:
      return ZodParsedType.unknown;
  }
};
const ZodIssueCode = util.arrayToEnum([
  "invalid_type",
  "invalid_literal",
  "custom",
  "invalid_union",
  "invalid_union_discriminator",
  "invalid_enum_value",
  "unrecognized_keys",
  "invalid_arguments",
  "invalid_return_type",
  "invalid_date",
  "invalid_string",
  "too_small",
  "too_big",
  "invalid_intersection_types",
  "not_multiple_of",
  "not_finite"
]);
const quotelessJson = (obj) => {
  const json = JSON.stringify(obj, null, 2);
  return json.replace(/"([^"]+)":/g, "$1:");
};
class ZodError extends Error {
  constructor(issues) {
    super();
    this.issues = [];
    this.addIssue = (sub) => {
      this.issues = [...this.issues, sub];
    };
    this.addIssues = (subs = []) => {
      this.issues = [...this.issues, ...subs];
    };
    const actualProto = new.target.prototype;
    if (Object.setPrototypeOf) {
      Object.setPrototypeOf(this, actualProto);
    } else {
      this.__proto__ = actualProto;
    }
    this.name = "ZodError";
    this.issues = issues;
  }
  get errors() {
    return this.issues;
  }
  format(_mapper) {
    const mapper = _mapper || function(issue) {
      return issue.message;
    };
    const fieldErrors = { _errors: [] };
    const processError = (error) => {
      for (const issue of error.issues) {
        if (issue.code === "invalid_union") {
          issue.unionErrors.map(processError);
        } else if (issue.code === "invalid_return_type") {
          processError(issue.returnTypeError);
        } else if (issue.code === "invalid_arguments") {
          processError(issue.argumentsError);
        } else if (issue.path.length === 0) {
          fieldErrors._errors.push(mapper(issue));
        } else {
          let curr = fieldErrors;
          let i2 = 0;
          while (i2 < issue.path.length) {
            const el = issue.path[i2];
            const terminal = i2 === issue.path.length - 1;
            if (!terminal) {
              curr[el] = curr[el] || { _errors: [] };
            } else {
              curr[el] = curr[el] || { _errors: [] };
              curr[el]._errors.push(mapper(issue));
            }
            curr = curr[el];
            i2++;
          }
        }
      }
    };
    processError(this);
    return fieldErrors;
  }
  toString() {
    return this.message;
  }
  get message() {
    return JSON.stringify(this.issues, util.jsonStringifyReplacer, 2);
  }
  get isEmpty() {
    return this.issues.length === 0;
  }
  flatten(mapper = (issue) => issue.message) {
    const fieldErrors = {};
    const formErrors = [];
    for (const sub of this.issues) {
      if (sub.path.length > 0) {
        fieldErrors[sub.path[0]] = fieldErrors[sub.path[0]] || [];
        fieldErrors[sub.path[0]].push(mapper(sub));
      } else {
        formErrors.push(mapper(sub));
      }
    }
    return { formErrors, fieldErrors };
  }
  get formErrors() {
    return this.flatten();
  }
}
ZodError.create = (issues) => {
  const error = new ZodError(issues);
  return error;
};
const errorMap = (issue, _ctx) => {
  let message;
  switch (issue.code) {
    case ZodIssueCode.invalid_type:
      if (issue.received === ZodParsedType.undefined) {
        message = "Required";
      } else {
        message = `Expected ${issue.expected}, received ${issue.received}`;
      }
      break;
    case ZodIssueCode.invalid_literal:
      message = `Invalid literal value, expected ${JSON.stringify(issue.expected, util.jsonStringifyReplacer)}`;
      break;
    case ZodIssueCode.unrecognized_keys:
      message = `Unrecognized key(s) in object: ${util.joinValues(issue.keys, ", ")}`;
      break;
    case ZodIssueCode.invalid_union:
      message = `Invalid input`;
      break;
    case ZodIssueCode.invalid_union_discriminator:
      message = `Invalid discriminator value. Expected ${util.joinValues(issue.options)}`;
      break;
    case ZodIssueCode.invalid_enum_value:
      message = `Invalid enum value. Expected ${util.joinValues(issue.options)}, received '${issue.received}'`;
      break;
    case ZodIssueCode.invalid_arguments:
      message = `Invalid function arguments`;
      break;
    case ZodIssueCode.invalid_return_type:
      message = `Invalid function return type`;
      break;
    case ZodIssueCode.invalid_date:
      message = `Invalid date`;
      break;
    case ZodIssueCode.invalid_string:
      if (typeof issue.validation === "object") {
        if ("includes" in issue.validation) {
          message = `Invalid input: must include "${issue.validation.includes}"`;
          if (typeof issue.validation.position === "number") {
            message = `${message} at one or more positions greater than or equal to ${issue.validation.position}`;
          }
        } else if ("startsWith" in issue.validation) {
          message = `Invalid input: must start with "${issue.validation.startsWith}"`;
        } else if ("endsWith" in issue.validation) {
          message = `Invalid input: must end with "${issue.validation.endsWith}"`;
        } else {
          util.assertNever(issue.validation);
        }
      } else if (issue.validation !== "regex") {
        message = `Invalid ${issue.validation}`;
      } else {
        message = "Invalid";
      }
      break;
    case ZodIssueCode.too_small:
      if (issue.type === "array")
        message = `Array must contain ${issue.exact ? "exactly" : issue.inclusive ? `at least` : `more than`} ${issue.minimum} element(s)`;
      else if (issue.type === "string")
        message = `String must contain ${issue.exact ? "exactly" : issue.inclusive ? `at least` : `over`} ${issue.minimum} character(s)`;
      else if (issue.type === "number")
        message = `Number must be ${issue.exact ? `exactly equal to ` : issue.inclusive ? `greater than or equal to ` : `greater than `}${issue.minimum}`;
      else if (issue.type === "date")
        message = `Date must be ${issue.exact ? `exactly equal to ` : issue.inclusive ? `greater than or equal to ` : `greater than `}${new Date(Number(issue.minimum))}`;
      else
        message = "Invalid input";
      break;
    case ZodIssueCode.too_big:
      if (issue.type === "array")
        message = `Array must contain ${issue.exact ? `exactly` : issue.inclusive ? `at most` : `less than`} ${issue.maximum} element(s)`;
      else if (issue.type === "string")
        message = `String must contain ${issue.exact ? `exactly` : issue.inclusive ? `at most` : `under`} ${issue.maximum} character(s)`;
      else if (issue.type === "number")
        message = `Number must be ${issue.exact ? `exactly` : issue.inclusive ? `less than or equal to` : `less than`} ${issue.maximum}`;
      else if (issue.type === "bigint")
        message = `BigInt must be ${issue.exact ? `exactly` : issue.inclusive ? `less than or equal to` : `less than`} ${issue.maximum}`;
      else if (issue.type === "date")
        message = `Date must be ${issue.exact ? `exactly` : issue.inclusive ? `smaller than or equal to` : `smaller than`} ${new Date(Number(issue.maximum))}`;
      else
        message = "Invalid input";
      break;
    case ZodIssueCode.custom:
      message = `Invalid input`;
      break;
    case ZodIssueCode.invalid_intersection_types:
      message = `Intersection results could not be merged`;
      break;
    case ZodIssueCode.not_multiple_of:
      message = `Number must be a multiple of ${issue.multipleOf}`;
      break;
    case ZodIssueCode.not_finite:
      message = "Number must be finite";
      break;
    default:
      message = _ctx.defaultError;
      util.assertNever(issue);
  }
  return { message };
};
let overrideErrorMap = errorMap;
function setErrorMap(map) {
  overrideErrorMap = map;
}
function getErrorMap() {
  return overrideErrorMap;
}
const makeIssue = (params) => {
  const { data, path, errorMaps, issueData } = params;
  const fullPath = [...path, ...issueData.path || []];
  const fullIssue = {
    ...issueData,
    path: fullPath
  };
  let errorMessage = "";
  const maps = errorMaps.filter((m2) => !!m2).slice().reverse();
  for (const map of maps) {
    errorMessage = map(fullIssue, { data, defaultError: errorMessage }).message;
  }
  return {
    ...issueData,
    path: fullPath,
    message: issueData.message || errorMessage
  };
};
const EMPTY_PATH = [];
function addIssueToContext(ctx, issueData) {
  const issue = makeIssue({
    issueData,
    data: ctx.data,
    path: ctx.path,
    errorMaps: [
      ctx.common.contextualErrorMap,
      ctx.schemaErrorMap,
      getErrorMap(),
      errorMap
      // then global default map
    ].filter((x2) => !!x2)
  });
  ctx.common.issues.push(issue);
}
class ParseStatus {
  constructor() {
    this.value = "valid";
  }
  dirty() {
    if (this.value === "valid")
      this.value = "dirty";
  }
  abort() {
    if (this.value !== "aborted")
      this.value = "aborted";
  }
  static mergeArray(status, results) {
    const arrayValue = [];
    for (const s2 of results) {
      if (s2.status === "aborted")
        return INVALID;
      if (s2.status === "dirty")
        status.dirty();
      arrayValue.push(s2.value);
    }
    return { status: status.value, value: arrayValue };
  }
  static async mergeObjectAsync(status, pairs) {
    const syncPairs = [];
    for (const pair of pairs) {
      syncPairs.push({
        key: await pair.key,
        value: await pair.value
      });
    }
    return ParseStatus.mergeObjectSync(status, syncPairs);
  }
  static mergeObjectSync(status, pairs) {
    const finalObject = {};
    for (const pair of pairs) {
      const { key, value } = pair;
      if (key.status === "aborted")
        return INVALID;
      if (value.status === "aborted")
        return INVALID;
      if (key.status === "dirty")
        status.dirty();
      if (value.status === "dirty")
        status.dirty();
      if (key.value !== "__proto__" && (typeof value.value !== "undefined" || pair.alwaysSet)) {
        finalObject[key.value] = value.value;
      }
    }
    return { status: status.value, value: finalObject };
  }
}
const INVALID = Object.freeze({
  status: "aborted"
});
const DIRTY = (value) => ({ status: "dirty", value });
const OK = (value) => ({ status: "valid", value });
const isAborted = (x2) => x2.status === "aborted";
const isDirty = (x2) => x2.status === "dirty";
const isValid = (x2) => x2.status === "valid";
const isAsync = (x2) => typeof Promise !== "undefined" && x2 instanceof Promise;
var errorUtil;
(function(errorUtil2) {
  errorUtil2.errToObj = (message) => typeof message === "string" ? { message } : message || {};
  errorUtil2.toString = (message) => typeof message === "string" ? message : message === null || message === void 0 ? void 0 : message.message;
})(errorUtil || (errorUtil = {}));
class ParseInputLazyPath {
  constructor(parent, value, path, key) {
    this._cachedPath = [];
    this.parent = parent;
    this.data = value;
    this._path = path;
    this._key = key;
  }
  get path() {
    if (!this._cachedPath.length) {
      if (this._key instanceof Array) {
        this._cachedPath.push(...this._path, ...this._key);
      } else {
        this._cachedPath.push(...this._path, this._key);
      }
    }
    return this._cachedPath;
  }
}
const handleResult = (ctx, result) => {
  if (isValid(result)) {
    return { success: true, data: result.value };
  } else {
    if (!ctx.common.issues.length) {
      throw new Error("Validation failed but no issues detected.");
    }
    return {
      success: false,
      get error() {
        if (this._error)
          return this._error;
        const error = new ZodError(ctx.common.issues);
        this._error = error;
        return this._error;
      }
    };
  }
};
function processCreateParams(params) {
  if (!params)
    return {};
  const { errorMap: errorMap2, invalid_type_error, required_error, description } = params;
  if (errorMap2 && (invalid_type_error || required_error)) {
    throw new Error(`Can't use "invalid_type_error" or "required_error" in conjunction with custom error map.`);
  }
  if (errorMap2)
    return { errorMap: errorMap2, description };
  const customMap = (iss, ctx) => {
    if (iss.code !== "invalid_type")
      return { message: ctx.defaultError };
    if (typeof ctx.data === "undefined") {
      return { message: required_error !== null && required_error !== void 0 ? required_error : ctx.defaultError };
    }
    return { message: invalid_type_error !== null && invalid_type_error !== void 0 ? invalid_type_error : ctx.defaultError };
  };
  return { errorMap: customMap, description };
}
class ZodType {
  constructor(def) {
    this.spa = this.safeParseAsync;
    this._def = def;
    this.parse = this.parse.bind(this);
    this.safeParse = this.safeParse.bind(this);
    this.parseAsync = this.parseAsync.bind(this);
    this.safeParseAsync = this.safeParseAsync.bind(this);
    this.spa = this.spa.bind(this);
    this.refine = this.refine.bind(this);
    this.refinement = this.refinement.bind(this);
    this.superRefine = this.superRefine.bind(this);
    this.optional = this.optional.bind(this);
    this.nullable = this.nullable.bind(this);
    this.nullish = this.nullish.bind(this);
    this.array = this.array.bind(this);
    this.promise = this.promise.bind(this);
    this.or = this.or.bind(this);
    this.and = this.and.bind(this);
    this.transform = this.transform.bind(this);
    this.brand = this.brand.bind(this);
    this.default = this.default.bind(this);
    this.catch = this.catch.bind(this);
    this.describe = this.describe.bind(this);
    this.pipe = this.pipe.bind(this);
    this.readonly = this.readonly.bind(this);
    this.isNullable = this.isNullable.bind(this);
    this.isOptional = this.isOptional.bind(this);
  }
  get description() {
    return this._def.description;
  }
  _getType(input) {
    return getParsedType(input.data);
  }
  _getOrReturnCtx(input, ctx) {
    return ctx || {
      common: input.parent.common,
      data: input.data,
      parsedType: getParsedType(input.data),
      schemaErrorMap: this._def.errorMap,
      path: input.path,
      parent: input.parent
    };
  }
  _processInputParams(input) {
    return {
      status: new ParseStatus(),
      ctx: {
        common: input.parent.common,
        data: input.data,
        parsedType: getParsedType(input.data),
        schemaErrorMap: this._def.errorMap,
        path: input.path,
        parent: input.parent
      }
    };
  }
  _parseSync(input) {
    const result = this._parse(input);
    if (isAsync(result)) {
      throw new Error("Synchronous parse encountered promise.");
    }
    return result;
  }
  _parseAsync(input) {
    const result = this._parse(input);
    return Promise.resolve(result);
  }
  parse(data, params) {
    const result = this.safeParse(data, params);
    if (result.success)
      return result.data;
    throw result.error;
  }
  safeParse(data, params) {
    var _a;
    const ctx = {
      common: {
        issues: [],
        async: (_a = params === null || params === void 0 ? void 0 : params.async) !== null && _a !== void 0 ? _a : false,
        contextualErrorMap: params === null || params === void 0 ? void 0 : params.errorMap
      },
      path: (params === null || params === void 0 ? void 0 : params.path) || [],
      schemaErrorMap: this._def.errorMap,
      parent: null,
      data,
      parsedType: getParsedType(data)
    };
    const result = this._parseSync({ data, path: ctx.path, parent: ctx });
    return handleResult(ctx, result);
  }
  async parseAsync(data, params) {
    const result = await this.safeParseAsync(data, params);
    if (result.success)
      return result.data;
    throw result.error;
  }
  async safeParseAsync(data, params) {
    const ctx = {
      common: {
        issues: [],
        contextualErrorMap: params === null || params === void 0 ? void 0 : params.errorMap,
        async: true
      },
      path: (params === null || params === void 0 ? void 0 : params.path) || [],
      schemaErrorMap: this._def.errorMap,
      parent: null,
      data,
      parsedType: getParsedType(data)
    };
    const maybeAsyncResult = this._parse({ data, path: ctx.path, parent: ctx });
    const result = await (isAsync(maybeAsyncResult) ? maybeAsyncResult : Promise.resolve(maybeAsyncResult));
    return handleResult(ctx, result);
  }
  refine(check, message) {
    const getIssueProperties = (val) => {
      if (typeof message === "string" || typeof message === "undefined") {
        return { message };
      } else if (typeof message === "function") {
        return message(val);
      } else {
        return message;
      }
    };
    return this._refinement((val, ctx) => {
      const result = check(val);
      const setError = () => ctx.addIssue({
        code: ZodIssueCode.custom,
        ...getIssueProperties(val)
      });
      if (typeof Promise !== "undefined" && result instanceof Promise) {
        return result.then((data) => {
          if (!data) {
            setError();
            return false;
          } else {
            return true;
          }
        });
      }
      if (!result) {
        setError();
        return false;
      } else {
        return true;
      }
    });
  }
  refinement(check, refinementData) {
    return this._refinement((val, ctx) => {
      if (!check(val)) {
        ctx.addIssue(typeof refinementData === "function" ? refinementData(val, ctx) : refinementData);
        return false;
      } else {
        return true;
      }
    });
  }
  _refinement(refinement) {
    return new ZodEffects({
      schema: this,
      typeName: ZodFirstPartyTypeKind.ZodEffects,
      effect: { type: "refinement", refinement }
    });
  }
  superRefine(refinement) {
    return this._refinement(refinement);
  }
  optional() {
    return ZodOptional.create(this, this._def);
  }
  nullable() {
    return ZodNullable.create(this, this._def);
  }
  nullish() {
    return this.nullable().optional();
  }
  array() {
    return ZodArray.create(this, this._def);
  }
  promise() {
    return ZodPromise.create(this, this._def);
  }
  or(option) {
    return ZodUnion.create([this, option], this._def);
  }
  and(incoming) {
    return ZodIntersection.create(this, incoming, this._def);
  }
  transform(transform) {
    return new ZodEffects({
      ...processCreateParams(this._def),
      schema: this,
      typeName: ZodFirstPartyTypeKind.ZodEffects,
      effect: { type: "transform", transform }
    });
  }
  default(def) {
    const defaultValueFunc = typeof def === "function" ? def : () => def;
    return new ZodDefault({
      ...processCreateParams(this._def),
      innerType: this,
      defaultValue: defaultValueFunc,
      typeName: ZodFirstPartyTypeKind.ZodDefault
    });
  }
  brand() {
    return new ZodBranded({
      typeName: ZodFirstPartyTypeKind.ZodBranded,
      type: this,
      ...processCreateParams(this._def)
    });
  }
  catch(def) {
    const catchValueFunc = typeof def === "function" ? def : () => def;
    return new ZodCatch({
      ...processCreateParams(this._def),
      innerType: this,
      catchValue: catchValueFunc,
      typeName: ZodFirstPartyTypeKind.ZodCatch
    });
  }
  describe(description) {
    const This = this.constructor;
    return new This({
      ...this._def,
      description
    });
  }
  pipe(target) {
    return ZodPipeline.create(this, target);
  }
  readonly() {
    return ZodReadonly.create(this);
  }
  isOptional() {
    return this.safeParse(void 0).success;
  }
  isNullable() {
    return this.safeParse(null).success;
  }
}
const cuidRegex = /^c[^\s-]{8,}$/i;
const cuid2Regex = /^[a-z][a-z0-9]*$/;
const ulidRegex = /^[0-9A-HJKMNP-TV-Z]{26}$/;
const uuidRegex = /^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/i;
const emailRegex = /^(?!\.)(?!.*\.\.)([A-Z0-9_+-\.]*)[A-Z0-9_+-]@([A-Z0-9][A-Z0-9\-]*\.)+[A-Z]{2,}$/i;
const _emojiRegex = `^(\\p{Extended_Pictographic}|\\p{Emoji_Component})+$`;
let emojiRegex;
const ipv4Regex = /^(((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2}))\.){3}((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2}))$/;
const ipv6Regex = /^(([a-f0-9]{1,4}:){7}|::([a-f0-9]{1,4}:){0,6}|([a-f0-9]{1,4}:){1}:([a-f0-9]{1,4}:){0,5}|([a-f0-9]{1,4}:){2}:([a-f0-9]{1,4}:){0,4}|([a-f0-9]{1,4}:){3}:([a-f0-9]{1,4}:){0,3}|([a-f0-9]{1,4}:){4}:([a-f0-9]{1,4}:){0,2}|([a-f0-9]{1,4}:){5}:([a-f0-9]{1,4}:){0,1})([a-f0-9]{1,4}|(((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2}))\.){3}((25[0-5])|(2[0-4][0-9])|(1[0-9]{2})|([0-9]{1,2})))$/;
const datetimeRegex = (args) => {
  if (args.precision) {
    if (args.offset) {
      return new RegExp(`^\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}\\.\\d{${args.precision}}(([+-]\\d{2}(:?\\d{2})?)|Z)$`);
    } else {
      return new RegExp(`^\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}\\.\\d{${args.precision}}Z$`);
    }
  } else if (args.precision === 0) {
    if (args.offset) {
      return new RegExp(`^\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}(([+-]\\d{2}(:?\\d{2})?)|Z)$`);
    } else {
      return new RegExp(`^\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}Z$`);
    }
  } else {
    if (args.offset) {
      return new RegExp(`^\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}(\\.\\d+)?(([+-]\\d{2}(:?\\d{2})?)|Z)$`);
    } else {
      return new RegExp(`^\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}(\\.\\d+)?Z$`);
    }
  }
};
function isValidIP(ip, version) {
  if ((version === "v4" || !version) && ipv4Regex.test(ip)) {
    return true;
  }
  if ((version === "v6" || !version) && ipv6Regex.test(ip)) {
    return true;
  }
  return false;
}
class ZodString extends ZodType {
  _parse(input) {
    if (this._def.coerce) {
      input.data = String(input.data);
    }
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.string) {
      const ctx2 = this._getOrReturnCtx(input);
      addIssueToContext(
        ctx2,
        {
          code: ZodIssueCode.invalid_type,
          expected: ZodParsedType.string,
          received: ctx2.parsedType
        }
        //
      );
      return INVALID;
    }
    const status = new ParseStatus();
    let ctx = void 0;
    for (const check of this._def.checks) {
      if (check.kind === "min") {
        if (input.data.length < check.value) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.too_small,
            minimum: check.value,
            type: "string",
            inclusive: true,
            exact: false,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "max") {
        if (input.data.length > check.value) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.too_big,
            maximum: check.value,
            type: "string",
            inclusive: true,
            exact: false,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "length") {
        const tooBig = input.data.length > check.value;
        const tooSmall = input.data.length < check.value;
        if (tooBig || tooSmall) {
          ctx = this._getOrReturnCtx(input, ctx);
          if (tooBig) {
            addIssueToContext(ctx, {
              code: ZodIssueCode.too_big,
              maximum: check.value,
              type: "string",
              inclusive: true,
              exact: true,
              message: check.message
            });
          } else if (tooSmall) {
            addIssueToContext(ctx, {
              code: ZodIssueCode.too_small,
              minimum: check.value,
              type: "string",
              inclusive: true,
              exact: true,
              message: check.message
            });
          }
          status.dirty();
        }
      } else if (check.kind === "email") {
        if (!emailRegex.test(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "email",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "emoji") {
        if (!emojiRegex) {
          emojiRegex = new RegExp(_emojiRegex, "u");
        }
        if (!emojiRegex.test(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "emoji",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "uuid") {
        if (!uuidRegex.test(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "uuid",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "cuid") {
        if (!cuidRegex.test(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "cuid",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "cuid2") {
        if (!cuid2Regex.test(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "cuid2",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "ulid") {
        if (!ulidRegex.test(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "ulid",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "url") {
        try {
          new URL(input.data);
        } catch (_a) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "url",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "regex") {
        check.regex.lastIndex = 0;
        const testResult = check.regex.test(input.data);
        if (!testResult) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "regex",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "trim") {
        input.data = input.data.trim();
      } else if (check.kind === "includes") {
        if (!input.data.includes(check.value, check.position)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.invalid_string,
            validation: { includes: check.value, position: check.position },
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "toLowerCase") {
        input.data = input.data.toLowerCase();
      } else if (check.kind === "toUpperCase") {
        input.data = input.data.toUpperCase();
      } else if (check.kind === "startsWith") {
        if (!input.data.startsWith(check.value)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.invalid_string,
            validation: { startsWith: check.value },
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "endsWith") {
        if (!input.data.endsWith(check.value)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.invalid_string,
            validation: { endsWith: check.value },
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "datetime") {
        const regex = datetimeRegex(check);
        if (!regex.test(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.invalid_string,
            validation: "datetime",
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "ip") {
        if (!isValidIP(input.data, check.version)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            validation: "ip",
            code: ZodIssueCode.invalid_string,
            message: check.message
          });
          status.dirty();
        }
      } else {
        util.assertNever(check);
      }
    }
    return { status: status.value, value: input.data };
  }
  _regex(regex, validation, message) {
    return this.refinement((data) => regex.test(data), {
      validation,
      code: ZodIssueCode.invalid_string,
      ...errorUtil.errToObj(message)
    });
  }
  _addCheck(check) {
    return new ZodString({
      ...this._def,
      checks: [...this._def.checks, check]
    });
  }
  email(message) {
    return this._addCheck({ kind: "email", ...errorUtil.errToObj(message) });
  }
  url(message) {
    return this._addCheck({ kind: "url", ...errorUtil.errToObj(message) });
  }
  emoji(message) {
    return this._addCheck({ kind: "emoji", ...errorUtil.errToObj(message) });
  }
  uuid(message) {
    return this._addCheck({ kind: "uuid", ...errorUtil.errToObj(message) });
  }
  cuid(message) {
    return this._addCheck({ kind: "cuid", ...errorUtil.errToObj(message) });
  }
  cuid2(message) {
    return this._addCheck({ kind: "cuid2", ...errorUtil.errToObj(message) });
  }
  ulid(message) {
    return this._addCheck({ kind: "ulid", ...errorUtil.errToObj(message) });
  }
  ip(options) {
    return this._addCheck({ kind: "ip", ...errorUtil.errToObj(options) });
  }
  datetime(options) {
    var _a;
    if (typeof options === "string") {
      return this._addCheck({
        kind: "datetime",
        precision: null,
        offset: false,
        message: options
      });
    }
    return this._addCheck({
      kind: "datetime",
      precision: typeof (options === null || options === void 0 ? void 0 : options.precision) === "undefined" ? null : options === null || options === void 0 ? void 0 : options.precision,
      offset: (_a = options === null || options === void 0 ? void 0 : options.offset) !== null && _a !== void 0 ? _a : false,
      ...errorUtil.errToObj(options === null || options === void 0 ? void 0 : options.message)
    });
  }
  regex(regex, message) {
    return this._addCheck({
      kind: "regex",
      regex,
      ...errorUtil.errToObj(message)
    });
  }
  includes(value, options) {
    return this._addCheck({
      kind: "includes",
      value,
      position: options === null || options === void 0 ? void 0 : options.position,
      ...errorUtil.errToObj(options === null || options === void 0 ? void 0 : options.message)
    });
  }
  startsWith(value, message) {
    return this._addCheck({
      kind: "startsWith",
      value,
      ...errorUtil.errToObj(message)
    });
  }
  endsWith(value, message) {
    return this._addCheck({
      kind: "endsWith",
      value,
      ...errorUtil.errToObj(message)
    });
  }
  min(minLength, message) {
    return this._addCheck({
      kind: "min",
      value: minLength,
      ...errorUtil.errToObj(message)
    });
  }
  max(maxLength, message) {
    return this._addCheck({
      kind: "max",
      value: maxLength,
      ...errorUtil.errToObj(message)
    });
  }
  length(len, message) {
    return this._addCheck({
      kind: "length",
      value: len,
      ...errorUtil.errToObj(message)
    });
  }
  /**
   * @deprecated Use z.string().min(1) instead.
   * @see {@link ZodString.min}
   */
  nonempty(message) {
    return this.min(1, errorUtil.errToObj(message));
  }
  trim() {
    return new ZodString({
      ...this._def,
      checks: [...this._def.checks, { kind: "trim" }]
    });
  }
  toLowerCase() {
    return new ZodString({
      ...this._def,
      checks: [...this._def.checks, { kind: "toLowerCase" }]
    });
  }
  toUpperCase() {
    return new ZodString({
      ...this._def,
      checks: [...this._def.checks, { kind: "toUpperCase" }]
    });
  }
  get isDatetime() {
    return !!this._def.checks.find((ch) => ch.kind === "datetime");
  }
  get isEmail() {
    return !!this._def.checks.find((ch) => ch.kind === "email");
  }
  get isURL() {
    return !!this._def.checks.find((ch) => ch.kind === "url");
  }
  get isEmoji() {
    return !!this._def.checks.find((ch) => ch.kind === "emoji");
  }
  get isUUID() {
    return !!this._def.checks.find((ch) => ch.kind === "uuid");
  }
  get isCUID() {
    return !!this._def.checks.find((ch) => ch.kind === "cuid");
  }
  get isCUID2() {
    return !!this._def.checks.find((ch) => ch.kind === "cuid2");
  }
  get isULID() {
    return !!this._def.checks.find((ch) => ch.kind === "ulid");
  }
  get isIP() {
    return !!this._def.checks.find((ch) => ch.kind === "ip");
  }
  get minLength() {
    let min = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "min") {
        if (min === null || ch.value > min)
          min = ch.value;
      }
    }
    return min;
  }
  get maxLength() {
    let max = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "max") {
        if (max === null || ch.value < max)
          max = ch.value;
      }
    }
    return max;
  }
}
ZodString.create = (params) => {
  var _a;
  return new ZodString({
    checks: [],
    typeName: ZodFirstPartyTypeKind.ZodString,
    coerce: (_a = params === null || params === void 0 ? void 0 : params.coerce) !== null && _a !== void 0 ? _a : false,
    ...processCreateParams(params)
  });
};
function floatSafeRemainder(val, step) {
  const valDecCount = (val.toString().split(".")[1] || "").length;
  const stepDecCount = (step.toString().split(".")[1] || "").length;
  const decCount = valDecCount > stepDecCount ? valDecCount : stepDecCount;
  const valInt = parseInt(val.toFixed(decCount).replace(".", ""));
  const stepInt = parseInt(step.toFixed(decCount).replace(".", ""));
  return valInt % stepInt / Math.pow(10, decCount);
}
class ZodNumber extends ZodType {
  constructor() {
    super(...arguments);
    this.min = this.gte;
    this.max = this.lte;
    this.step = this.multipleOf;
  }
  _parse(input) {
    if (this._def.coerce) {
      input.data = Number(input.data);
    }
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.number) {
      const ctx2 = this._getOrReturnCtx(input);
      addIssueToContext(ctx2, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.number,
        received: ctx2.parsedType
      });
      return INVALID;
    }
    let ctx = void 0;
    const status = new ParseStatus();
    for (const check of this._def.checks) {
      if (check.kind === "int") {
        if (!util.isInteger(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.invalid_type,
            expected: "integer",
            received: "float",
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "min") {
        const tooSmall = check.inclusive ? input.data < check.value : input.data <= check.value;
        if (tooSmall) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.too_small,
            minimum: check.value,
            type: "number",
            inclusive: check.inclusive,
            exact: false,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "max") {
        const tooBig = check.inclusive ? input.data > check.value : input.data >= check.value;
        if (tooBig) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.too_big,
            maximum: check.value,
            type: "number",
            inclusive: check.inclusive,
            exact: false,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "multipleOf") {
        if (floatSafeRemainder(input.data, check.value) !== 0) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.not_multiple_of,
            multipleOf: check.value,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "finite") {
        if (!Number.isFinite(input.data)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.not_finite,
            message: check.message
          });
          status.dirty();
        }
      } else {
        util.assertNever(check);
      }
    }
    return { status: status.value, value: input.data };
  }
  gte(value, message) {
    return this.setLimit("min", value, true, errorUtil.toString(message));
  }
  gt(value, message) {
    return this.setLimit("min", value, false, errorUtil.toString(message));
  }
  lte(value, message) {
    return this.setLimit("max", value, true, errorUtil.toString(message));
  }
  lt(value, message) {
    return this.setLimit("max", value, false, errorUtil.toString(message));
  }
  setLimit(kind, value, inclusive, message) {
    return new ZodNumber({
      ...this._def,
      checks: [
        ...this._def.checks,
        {
          kind,
          value,
          inclusive,
          message: errorUtil.toString(message)
        }
      ]
    });
  }
  _addCheck(check) {
    return new ZodNumber({
      ...this._def,
      checks: [...this._def.checks, check]
    });
  }
  int(message) {
    return this._addCheck({
      kind: "int",
      message: errorUtil.toString(message)
    });
  }
  positive(message) {
    return this._addCheck({
      kind: "min",
      value: 0,
      inclusive: false,
      message: errorUtil.toString(message)
    });
  }
  negative(message) {
    return this._addCheck({
      kind: "max",
      value: 0,
      inclusive: false,
      message: errorUtil.toString(message)
    });
  }
  nonpositive(message) {
    return this._addCheck({
      kind: "max",
      value: 0,
      inclusive: true,
      message: errorUtil.toString(message)
    });
  }
  nonnegative(message) {
    return this._addCheck({
      kind: "min",
      value: 0,
      inclusive: true,
      message: errorUtil.toString(message)
    });
  }
  multipleOf(value, message) {
    return this._addCheck({
      kind: "multipleOf",
      value,
      message: errorUtil.toString(message)
    });
  }
  finite(message) {
    return this._addCheck({
      kind: "finite",
      message: errorUtil.toString(message)
    });
  }
  safe(message) {
    return this._addCheck({
      kind: "min",
      inclusive: true,
      value: Number.MIN_SAFE_INTEGER,
      message: errorUtil.toString(message)
    })._addCheck({
      kind: "max",
      inclusive: true,
      value: Number.MAX_SAFE_INTEGER,
      message: errorUtil.toString(message)
    });
  }
  get minValue() {
    let min = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "min") {
        if (min === null || ch.value > min)
          min = ch.value;
      }
    }
    return min;
  }
  get maxValue() {
    let max = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "max") {
        if (max === null || ch.value < max)
          max = ch.value;
      }
    }
    return max;
  }
  get isInt() {
    return !!this._def.checks.find((ch) => ch.kind === "int" || ch.kind === "multipleOf" && util.isInteger(ch.value));
  }
  get isFinite() {
    let max = null, min = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "finite" || ch.kind === "int" || ch.kind === "multipleOf") {
        return true;
      } else if (ch.kind === "min") {
        if (min === null || ch.value > min)
          min = ch.value;
      } else if (ch.kind === "max") {
        if (max === null || ch.value < max)
          max = ch.value;
      }
    }
    return Number.isFinite(min) && Number.isFinite(max);
  }
}
ZodNumber.create = (params) => {
  return new ZodNumber({
    checks: [],
    typeName: ZodFirstPartyTypeKind.ZodNumber,
    coerce: (params === null || params === void 0 ? void 0 : params.coerce) || false,
    ...processCreateParams(params)
  });
};
class ZodBigInt extends ZodType {
  constructor() {
    super(...arguments);
    this.min = this.gte;
    this.max = this.lte;
  }
  _parse(input) {
    if (this._def.coerce) {
      input.data = BigInt(input.data);
    }
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.bigint) {
      const ctx2 = this._getOrReturnCtx(input);
      addIssueToContext(ctx2, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.bigint,
        received: ctx2.parsedType
      });
      return INVALID;
    }
    let ctx = void 0;
    const status = new ParseStatus();
    for (const check of this._def.checks) {
      if (check.kind === "min") {
        const tooSmall = check.inclusive ? input.data < check.value : input.data <= check.value;
        if (tooSmall) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.too_small,
            type: "bigint",
            minimum: check.value,
            inclusive: check.inclusive,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "max") {
        const tooBig = check.inclusive ? input.data > check.value : input.data >= check.value;
        if (tooBig) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.too_big,
            type: "bigint",
            maximum: check.value,
            inclusive: check.inclusive,
            message: check.message
          });
          status.dirty();
        }
      } else if (check.kind === "multipleOf") {
        if (input.data % check.value !== BigInt(0)) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.not_multiple_of,
            multipleOf: check.value,
            message: check.message
          });
          status.dirty();
        }
      } else {
        util.assertNever(check);
      }
    }
    return { status: status.value, value: input.data };
  }
  gte(value, message) {
    return this.setLimit("min", value, true, errorUtil.toString(message));
  }
  gt(value, message) {
    return this.setLimit("min", value, false, errorUtil.toString(message));
  }
  lte(value, message) {
    return this.setLimit("max", value, true, errorUtil.toString(message));
  }
  lt(value, message) {
    return this.setLimit("max", value, false, errorUtil.toString(message));
  }
  setLimit(kind, value, inclusive, message) {
    return new ZodBigInt({
      ...this._def,
      checks: [
        ...this._def.checks,
        {
          kind,
          value,
          inclusive,
          message: errorUtil.toString(message)
        }
      ]
    });
  }
  _addCheck(check) {
    return new ZodBigInt({
      ...this._def,
      checks: [...this._def.checks, check]
    });
  }
  positive(message) {
    return this._addCheck({
      kind: "min",
      value: BigInt(0),
      inclusive: false,
      message: errorUtil.toString(message)
    });
  }
  negative(message) {
    return this._addCheck({
      kind: "max",
      value: BigInt(0),
      inclusive: false,
      message: errorUtil.toString(message)
    });
  }
  nonpositive(message) {
    return this._addCheck({
      kind: "max",
      value: BigInt(0),
      inclusive: true,
      message: errorUtil.toString(message)
    });
  }
  nonnegative(message) {
    return this._addCheck({
      kind: "min",
      value: BigInt(0),
      inclusive: true,
      message: errorUtil.toString(message)
    });
  }
  multipleOf(value, message) {
    return this._addCheck({
      kind: "multipleOf",
      value,
      message: errorUtil.toString(message)
    });
  }
  get minValue() {
    let min = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "min") {
        if (min === null || ch.value > min)
          min = ch.value;
      }
    }
    return min;
  }
  get maxValue() {
    let max = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "max") {
        if (max === null || ch.value < max)
          max = ch.value;
      }
    }
    return max;
  }
}
ZodBigInt.create = (params) => {
  var _a;
  return new ZodBigInt({
    checks: [],
    typeName: ZodFirstPartyTypeKind.ZodBigInt,
    coerce: (_a = params === null || params === void 0 ? void 0 : params.coerce) !== null && _a !== void 0 ? _a : false,
    ...processCreateParams(params)
  });
};
class ZodBoolean extends ZodType {
  _parse(input) {
    if (this._def.coerce) {
      input.data = Boolean(input.data);
    }
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.boolean) {
      const ctx = this._getOrReturnCtx(input);
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.boolean,
        received: ctx.parsedType
      });
      return INVALID;
    }
    return OK(input.data);
  }
}
ZodBoolean.create = (params) => {
  return new ZodBoolean({
    typeName: ZodFirstPartyTypeKind.ZodBoolean,
    coerce: (params === null || params === void 0 ? void 0 : params.coerce) || false,
    ...processCreateParams(params)
  });
};
class ZodDate extends ZodType {
  _parse(input) {
    if (this._def.coerce) {
      input.data = new Date(input.data);
    }
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.date) {
      const ctx2 = this._getOrReturnCtx(input);
      addIssueToContext(ctx2, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.date,
        received: ctx2.parsedType
      });
      return INVALID;
    }
    if (isNaN(input.data.getTime())) {
      const ctx2 = this._getOrReturnCtx(input);
      addIssueToContext(ctx2, {
        code: ZodIssueCode.invalid_date
      });
      return INVALID;
    }
    const status = new ParseStatus();
    let ctx = void 0;
    for (const check of this._def.checks) {
      if (check.kind === "min") {
        if (input.data.getTime() < check.value) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.too_small,
            message: check.message,
            inclusive: true,
            exact: false,
            minimum: check.value,
            type: "date"
          });
          status.dirty();
        }
      } else if (check.kind === "max") {
        if (input.data.getTime() > check.value) {
          ctx = this._getOrReturnCtx(input, ctx);
          addIssueToContext(ctx, {
            code: ZodIssueCode.too_big,
            message: check.message,
            inclusive: true,
            exact: false,
            maximum: check.value,
            type: "date"
          });
          status.dirty();
        }
      } else {
        util.assertNever(check);
      }
    }
    return {
      status: status.value,
      value: new Date(input.data.getTime())
    };
  }
  _addCheck(check) {
    return new ZodDate({
      ...this._def,
      checks: [...this._def.checks, check]
    });
  }
  min(minDate, message) {
    return this._addCheck({
      kind: "min",
      value: minDate.getTime(),
      message: errorUtil.toString(message)
    });
  }
  max(maxDate, message) {
    return this._addCheck({
      kind: "max",
      value: maxDate.getTime(),
      message: errorUtil.toString(message)
    });
  }
  get minDate() {
    let min = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "min") {
        if (min === null || ch.value > min)
          min = ch.value;
      }
    }
    return min != null ? new Date(min) : null;
  }
  get maxDate() {
    let max = null;
    for (const ch of this._def.checks) {
      if (ch.kind === "max") {
        if (max === null || ch.value < max)
          max = ch.value;
      }
    }
    return max != null ? new Date(max) : null;
  }
}
ZodDate.create = (params) => {
  return new ZodDate({
    checks: [],
    coerce: (params === null || params === void 0 ? void 0 : params.coerce) || false,
    typeName: ZodFirstPartyTypeKind.ZodDate,
    ...processCreateParams(params)
  });
};
class ZodSymbol extends ZodType {
  _parse(input) {
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.symbol) {
      const ctx = this._getOrReturnCtx(input);
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.symbol,
        received: ctx.parsedType
      });
      return INVALID;
    }
    return OK(input.data);
  }
}
ZodSymbol.create = (params) => {
  return new ZodSymbol({
    typeName: ZodFirstPartyTypeKind.ZodSymbol,
    ...processCreateParams(params)
  });
};
class ZodUndefined extends ZodType {
  _parse(input) {
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.undefined) {
      const ctx = this._getOrReturnCtx(input);
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.undefined,
        received: ctx.parsedType
      });
      return INVALID;
    }
    return OK(input.data);
  }
}
ZodUndefined.create = (params) => {
  return new ZodUndefined({
    typeName: ZodFirstPartyTypeKind.ZodUndefined,
    ...processCreateParams(params)
  });
};
class ZodNull extends ZodType {
  _parse(input) {
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.null) {
      const ctx = this._getOrReturnCtx(input);
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.null,
        received: ctx.parsedType
      });
      return INVALID;
    }
    return OK(input.data);
  }
}
ZodNull.create = (params) => {
  return new ZodNull({
    typeName: ZodFirstPartyTypeKind.ZodNull,
    ...processCreateParams(params)
  });
};
class ZodAny extends ZodType {
  constructor() {
    super(...arguments);
    this._any = true;
  }
  _parse(input) {
    return OK(input.data);
  }
}
ZodAny.create = (params) => {
  return new ZodAny({
    typeName: ZodFirstPartyTypeKind.ZodAny,
    ...processCreateParams(params)
  });
};
class ZodUnknown extends ZodType {
  constructor() {
    super(...arguments);
    this._unknown = true;
  }
  _parse(input) {
    return OK(input.data);
  }
}
ZodUnknown.create = (params) => {
  return new ZodUnknown({
    typeName: ZodFirstPartyTypeKind.ZodUnknown,
    ...processCreateParams(params)
  });
};
class ZodNever extends ZodType {
  _parse(input) {
    const ctx = this._getOrReturnCtx(input);
    addIssueToContext(ctx, {
      code: ZodIssueCode.invalid_type,
      expected: ZodParsedType.never,
      received: ctx.parsedType
    });
    return INVALID;
  }
}
ZodNever.create = (params) => {
  return new ZodNever({
    typeName: ZodFirstPartyTypeKind.ZodNever,
    ...processCreateParams(params)
  });
};
class ZodVoid extends ZodType {
  _parse(input) {
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.undefined) {
      const ctx = this._getOrReturnCtx(input);
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.void,
        received: ctx.parsedType
      });
      return INVALID;
    }
    return OK(input.data);
  }
}
ZodVoid.create = (params) => {
  return new ZodVoid({
    typeName: ZodFirstPartyTypeKind.ZodVoid,
    ...processCreateParams(params)
  });
};
class ZodArray extends ZodType {
  _parse(input) {
    const { ctx, status } = this._processInputParams(input);
    const def = this._def;
    if (ctx.parsedType !== ZodParsedType.array) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.array,
        received: ctx.parsedType
      });
      return INVALID;
    }
    if (def.exactLength !== null) {
      const tooBig = ctx.data.length > def.exactLength.value;
      const tooSmall = ctx.data.length < def.exactLength.value;
      if (tooBig || tooSmall) {
        addIssueToContext(ctx, {
          code: tooBig ? ZodIssueCode.too_big : ZodIssueCode.too_small,
          minimum: tooSmall ? def.exactLength.value : void 0,
          maximum: tooBig ? def.exactLength.value : void 0,
          type: "array",
          inclusive: true,
          exact: true,
          message: def.exactLength.message
        });
        status.dirty();
      }
    }
    if (def.minLength !== null) {
      if (ctx.data.length < def.minLength.value) {
        addIssueToContext(ctx, {
          code: ZodIssueCode.too_small,
          minimum: def.minLength.value,
          type: "array",
          inclusive: true,
          exact: false,
          message: def.minLength.message
        });
        status.dirty();
      }
    }
    if (def.maxLength !== null) {
      if (ctx.data.length > def.maxLength.value) {
        addIssueToContext(ctx, {
          code: ZodIssueCode.too_big,
          maximum: def.maxLength.value,
          type: "array",
          inclusive: true,
          exact: false,
          message: def.maxLength.message
        });
        status.dirty();
      }
    }
    if (ctx.common.async) {
      return Promise.all([...ctx.data].map((item, i2) => {
        return def.type._parseAsync(new ParseInputLazyPath(ctx, item, ctx.path, i2));
      })).then((result2) => {
        return ParseStatus.mergeArray(status, result2);
      });
    }
    const result = [...ctx.data].map((item, i2) => {
      return def.type._parseSync(new ParseInputLazyPath(ctx, item, ctx.path, i2));
    });
    return ParseStatus.mergeArray(status, result);
  }
  get element() {
    return this._def.type;
  }
  min(minLength, message) {
    return new ZodArray({
      ...this._def,
      minLength: { value: minLength, message: errorUtil.toString(message) }
    });
  }
  max(maxLength, message) {
    return new ZodArray({
      ...this._def,
      maxLength: { value: maxLength, message: errorUtil.toString(message) }
    });
  }
  length(len, message) {
    return new ZodArray({
      ...this._def,
      exactLength: { value: len, message: errorUtil.toString(message) }
    });
  }
  nonempty(message) {
    return this.min(1, message);
  }
}
ZodArray.create = (schema, params) => {
  return new ZodArray({
    type: schema,
    minLength: null,
    maxLength: null,
    exactLength: null,
    typeName: ZodFirstPartyTypeKind.ZodArray,
    ...processCreateParams(params)
  });
};
function deepPartialify(schema) {
  if (schema instanceof ZodObject) {
    const newShape = {};
    for (const key in schema.shape) {
      const fieldSchema = schema.shape[key];
      newShape[key] = ZodOptional.create(deepPartialify(fieldSchema));
    }
    return new ZodObject({
      ...schema._def,
      shape: () => newShape
    });
  } else if (schema instanceof ZodArray) {
    return new ZodArray({
      ...schema._def,
      type: deepPartialify(schema.element)
    });
  } else if (schema instanceof ZodOptional) {
    return ZodOptional.create(deepPartialify(schema.unwrap()));
  } else if (schema instanceof ZodNullable) {
    return ZodNullable.create(deepPartialify(schema.unwrap()));
  } else if (schema instanceof ZodTuple) {
    return ZodTuple.create(schema.items.map((item) => deepPartialify(item)));
  } else {
    return schema;
  }
}
class ZodObject extends ZodType {
  constructor() {
    super(...arguments);
    this._cached = null;
    this.nonstrict = this.passthrough;
    this.augment = this.extend;
  }
  _getCached() {
    if (this._cached !== null)
      return this._cached;
    const shape = this._def.shape();
    const keys = util.objectKeys(shape);
    return this._cached = { shape, keys };
  }
  _parse(input) {
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.object) {
      const ctx2 = this._getOrReturnCtx(input);
      addIssueToContext(ctx2, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.object,
        received: ctx2.parsedType
      });
      return INVALID;
    }
    const { status, ctx } = this._processInputParams(input);
    const { shape, keys: shapeKeys } = this._getCached();
    const extraKeys = [];
    if (!(this._def.catchall instanceof ZodNever && this._def.unknownKeys === "strip")) {
      for (const key in ctx.data) {
        if (!shapeKeys.includes(key)) {
          extraKeys.push(key);
        }
      }
    }
    const pairs = [];
    for (const key of shapeKeys) {
      const keyValidator = shape[key];
      const value = ctx.data[key];
      pairs.push({
        key: { status: "valid", value: key },
        value: keyValidator._parse(new ParseInputLazyPath(ctx, value, ctx.path, key)),
        alwaysSet: key in ctx.data
      });
    }
    if (this._def.catchall instanceof ZodNever) {
      const unknownKeys = this._def.unknownKeys;
      if (unknownKeys === "passthrough") {
        for (const key of extraKeys) {
          pairs.push({
            key: { status: "valid", value: key },
            value: { status: "valid", value: ctx.data[key] }
          });
        }
      } else if (unknownKeys === "strict") {
        if (extraKeys.length > 0) {
          addIssueToContext(ctx, {
            code: ZodIssueCode.unrecognized_keys,
            keys: extraKeys
          });
          status.dirty();
        }
      } else if (unknownKeys === "strip")
        ;
      else {
        throw new Error(`Internal ZodObject error: invalid unknownKeys value.`);
      }
    } else {
      const catchall = this._def.catchall;
      for (const key of extraKeys) {
        const value = ctx.data[key];
        pairs.push({
          key: { status: "valid", value: key },
          value: catchall._parse(
            new ParseInputLazyPath(ctx, value, ctx.path, key)
            //, ctx.child(key), value, getParsedType(value)
          ),
          alwaysSet: key in ctx.data
        });
      }
    }
    if (ctx.common.async) {
      return Promise.resolve().then(async () => {
        const syncPairs = [];
        for (const pair of pairs) {
          const key = await pair.key;
          syncPairs.push({
            key,
            value: await pair.value,
            alwaysSet: pair.alwaysSet
          });
        }
        return syncPairs;
      }).then((syncPairs) => {
        return ParseStatus.mergeObjectSync(status, syncPairs);
      });
    } else {
      return ParseStatus.mergeObjectSync(status, pairs);
    }
  }
  get shape() {
    return this._def.shape();
  }
  strict(message) {
    errorUtil.errToObj;
    return new ZodObject({
      ...this._def,
      unknownKeys: "strict",
      ...message !== void 0 ? {
        errorMap: (issue, ctx) => {
          var _a, _b, _c, _d;
          const defaultError = (_c = (_b = (_a = this._def).errorMap) === null || _b === void 0 ? void 0 : _b.call(_a, issue, ctx).message) !== null && _c !== void 0 ? _c : ctx.defaultError;
          if (issue.code === "unrecognized_keys")
            return {
              message: (_d = errorUtil.errToObj(message).message) !== null && _d !== void 0 ? _d : defaultError
            };
          return {
            message: defaultError
          };
        }
      } : {}
    });
  }
  strip() {
    return new ZodObject({
      ...this._def,
      unknownKeys: "strip"
    });
  }
  passthrough() {
    return new ZodObject({
      ...this._def,
      unknownKeys: "passthrough"
    });
  }
  // const AugmentFactory =
  //   <Def extends ZodObjectDef>(def: Def) =>
  //   <Augmentation extends ZodRawShape>(
  //     augmentation: Augmentation
  //   ): ZodObject<
  //     extendShape<ReturnType<Def["shape"]>, Augmentation>,
  //     Def["unknownKeys"],
  //     Def["catchall"]
  //   > => {
  //     return new ZodObject({
  //       ...def,
  //       shape: () => ({
  //         ...def.shape(),
  //         ...augmentation,
  //       }),
  //     }) as any;
  //   };
  extend(augmentation) {
    return new ZodObject({
      ...this._def,
      shape: () => ({
        ...this._def.shape(),
        ...augmentation
      })
    });
  }
  /**
   * Prior to zod@1.0.12 there was a bug in the
   * inferred type of merged objects. Please
   * upgrade if you are experiencing issues.
   */
  merge(merging) {
    const merged = new ZodObject({
      unknownKeys: merging._def.unknownKeys,
      catchall: merging._def.catchall,
      shape: () => ({
        ...this._def.shape(),
        ...merging._def.shape()
      }),
      typeName: ZodFirstPartyTypeKind.ZodObject
    });
    return merged;
  }
  // merge<
  //   Incoming extends AnyZodObject,
  //   Augmentation extends Incoming["shape"],
  //   NewOutput extends {
  //     [k in keyof Augmentation | keyof Output]: k extends keyof Augmentation
  //       ? Augmentation[k]["_output"]
  //       : k extends keyof Output
  //       ? Output[k]
  //       : never;
  //   },
  //   NewInput extends {
  //     [k in keyof Augmentation | keyof Input]: k extends keyof Augmentation
  //       ? Augmentation[k]["_input"]
  //       : k extends keyof Input
  //       ? Input[k]
  //       : never;
  //   }
  // >(
  //   merging: Incoming
  // ): ZodObject<
  //   extendShape<T, ReturnType<Incoming["_def"]["shape"]>>,
  //   Incoming["_def"]["unknownKeys"],
  //   Incoming["_def"]["catchall"],
  //   NewOutput,
  //   NewInput
  // > {
  //   const merged: any = new ZodObject({
  //     unknownKeys: merging._def.unknownKeys,
  //     catchall: merging._def.catchall,
  //     shape: () =>
  //       objectUtil.mergeShapes(this._def.shape(), merging._def.shape()),
  //     typeName: ZodFirstPartyTypeKind.ZodObject,
  //   }) as any;
  //   return merged;
  // }
  setKey(key, schema) {
    return this.augment({ [key]: schema });
  }
  // merge<Incoming extends AnyZodObject>(
  //   merging: Incoming
  // ): //ZodObject<T & Incoming["_shape"], UnknownKeys, Catchall> = (merging) => {
  // ZodObject<
  //   extendShape<T, ReturnType<Incoming["_def"]["shape"]>>,
  //   Incoming["_def"]["unknownKeys"],
  //   Incoming["_def"]["catchall"]
  // > {
  //   // const mergedShape = objectUtil.mergeShapes(
  //   //   this._def.shape(),
  //   //   merging._def.shape()
  //   // );
  //   const merged: any = new ZodObject({
  //     unknownKeys: merging._def.unknownKeys,
  //     catchall: merging._def.catchall,
  //     shape: () =>
  //       objectUtil.mergeShapes(this._def.shape(), merging._def.shape()),
  //     typeName: ZodFirstPartyTypeKind.ZodObject,
  //   }) as any;
  //   return merged;
  // }
  catchall(index) {
    return new ZodObject({
      ...this._def,
      catchall: index
    });
  }
  pick(mask) {
    const shape = {};
    util.objectKeys(mask).forEach((key) => {
      if (mask[key] && this.shape[key]) {
        shape[key] = this.shape[key];
      }
    });
    return new ZodObject({
      ...this._def,
      shape: () => shape
    });
  }
  omit(mask) {
    const shape = {};
    util.objectKeys(this.shape).forEach((key) => {
      if (!mask[key]) {
        shape[key] = this.shape[key];
      }
    });
    return new ZodObject({
      ...this._def,
      shape: () => shape
    });
  }
  /**
   * @deprecated
   */
  deepPartial() {
    return deepPartialify(this);
  }
  partial(mask) {
    const newShape = {};
    util.objectKeys(this.shape).forEach((key) => {
      const fieldSchema = this.shape[key];
      if (mask && !mask[key]) {
        newShape[key] = fieldSchema;
      } else {
        newShape[key] = fieldSchema.optional();
      }
    });
    return new ZodObject({
      ...this._def,
      shape: () => newShape
    });
  }
  required(mask) {
    const newShape = {};
    util.objectKeys(this.shape).forEach((key) => {
      if (mask && !mask[key]) {
        newShape[key] = this.shape[key];
      } else {
        const fieldSchema = this.shape[key];
        let newField = fieldSchema;
        while (newField instanceof ZodOptional) {
          newField = newField._def.innerType;
        }
        newShape[key] = newField;
      }
    });
    return new ZodObject({
      ...this._def,
      shape: () => newShape
    });
  }
  keyof() {
    return createZodEnum(util.objectKeys(this.shape));
  }
}
ZodObject.create = (shape, params) => {
  return new ZodObject({
    shape: () => shape,
    unknownKeys: "strip",
    catchall: ZodNever.create(),
    typeName: ZodFirstPartyTypeKind.ZodObject,
    ...processCreateParams(params)
  });
};
ZodObject.strictCreate = (shape, params) => {
  return new ZodObject({
    shape: () => shape,
    unknownKeys: "strict",
    catchall: ZodNever.create(),
    typeName: ZodFirstPartyTypeKind.ZodObject,
    ...processCreateParams(params)
  });
};
ZodObject.lazycreate = (shape, params) => {
  return new ZodObject({
    shape,
    unknownKeys: "strip",
    catchall: ZodNever.create(),
    typeName: ZodFirstPartyTypeKind.ZodObject,
    ...processCreateParams(params)
  });
};
class ZodUnion extends ZodType {
  _parse(input) {
    const { ctx } = this._processInputParams(input);
    const options = this._def.options;
    function handleResults(results) {
      for (const result of results) {
        if (result.result.status === "valid") {
          return result.result;
        }
      }
      for (const result of results) {
        if (result.result.status === "dirty") {
          ctx.common.issues.push(...result.ctx.common.issues);
          return result.result;
        }
      }
      const unionErrors = results.map((result) => new ZodError(result.ctx.common.issues));
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_union,
        unionErrors
      });
      return INVALID;
    }
    if (ctx.common.async) {
      return Promise.all(options.map(async (option) => {
        const childCtx = {
          ...ctx,
          common: {
            ...ctx.common,
            issues: []
          },
          parent: null
        };
        return {
          result: await option._parseAsync({
            data: ctx.data,
            path: ctx.path,
            parent: childCtx
          }),
          ctx: childCtx
        };
      })).then(handleResults);
    } else {
      let dirty = void 0;
      const issues = [];
      for (const option of options) {
        const childCtx = {
          ...ctx,
          common: {
            ...ctx.common,
            issues: []
          },
          parent: null
        };
        const result = option._parseSync({
          data: ctx.data,
          path: ctx.path,
          parent: childCtx
        });
        if (result.status === "valid") {
          return result;
        } else if (result.status === "dirty" && !dirty) {
          dirty = { result, ctx: childCtx };
        }
        if (childCtx.common.issues.length) {
          issues.push(childCtx.common.issues);
        }
      }
      if (dirty) {
        ctx.common.issues.push(...dirty.ctx.common.issues);
        return dirty.result;
      }
      const unionErrors = issues.map((issues2) => new ZodError(issues2));
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_union,
        unionErrors
      });
      return INVALID;
    }
  }
  get options() {
    return this._def.options;
  }
}
ZodUnion.create = (types, params) => {
  return new ZodUnion({
    options: types,
    typeName: ZodFirstPartyTypeKind.ZodUnion,
    ...processCreateParams(params)
  });
};
const getDiscriminator = (type) => {
  if (type instanceof ZodLazy) {
    return getDiscriminator(type.schema);
  } else if (type instanceof ZodEffects) {
    return getDiscriminator(type.innerType());
  } else if (type instanceof ZodLiteral) {
    return [type.value];
  } else if (type instanceof ZodEnum) {
    return type.options;
  } else if (type instanceof ZodNativeEnum) {
    return Object.keys(type.enum);
  } else if (type instanceof ZodDefault) {
    return getDiscriminator(type._def.innerType);
  } else if (type instanceof ZodUndefined) {
    return [void 0];
  } else if (type instanceof ZodNull) {
    return [null];
  } else {
    return null;
  }
};
class ZodDiscriminatedUnion extends ZodType {
  _parse(input) {
    const { ctx } = this._processInputParams(input);
    if (ctx.parsedType !== ZodParsedType.object) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.object,
        received: ctx.parsedType
      });
      return INVALID;
    }
    const discriminator = this.discriminator;
    const discriminatorValue = ctx.data[discriminator];
    const option = this.optionsMap.get(discriminatorValue);
    if (!option) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_union_discriminator,
        options: Array.from(this.optionsMap.keys()),
        path: [discriminator]
      });
      return INVALID;
    }
    if (ctx.common.async) {
      return option._parseAsync({
        data: ctx.data,
        path: ctx.path,
        parent: ctx
      });
    } else {
      return option._parseSync({
        data: ctx.data,
        path: ctx.path,
        parent: ctx
      });
    }
  }
  get discriminator() {
    return this._def.discriminator;
  }
  get options() {
    return this._def.options;
  }
  get optionsMap() {
    return this._def.optionsMap;
  }
  /**
   * The constructor of the discriminated union schema. Its behaviour is very similar to that of the normal z.union() constructor.
   * However, it only allows a union of objects, all of which need to share a discriminator property. This property must
   * have a different value for each object in the union.
   * @param discriminator the name of the discriminator property
   * @param types an array of object schemas
   * @param params
   */
  static create(discriminator, options, params) {
    const optionsMap = /* @__PURE__ */ new Map();
    for (const type of options) {
      const discriminatorValues = getDiscriminator(type.shape[discriminator]);
      if (!discriminatorValues) {
        throw new Error(`A discriminator value for key \`${discriminator}\` could not be extracted from all schema options`);
      }
      for (const value of discriminatorValues) {
        if (optionsMap.has(value)) {
          throw new Error(`Discriminator property ${String(discriminator)} has duplicate value ${String(value)}`);
        }
        optionsMap.set(value, type);
      }
    }
    return new ZodDiscriminatedUnion({
      typeName: ZodFirstPartyTypeKind.ZodDiscriminatedUnion,
      discriminator,
      options,
      optionsMap,
      ...processCreateParams(params)
    });
  }
}
function mergeValues(a2, b2) {
  const aType = getParsedType(a2);
  const bType = getParsedType(b2);
  if (a2 === b2) {
    return { valid: true, data: a2 };
  } else if (aType === ZodParsedType.object && bType === ZodParsedType.object) {
    const bKeys = util.objectKeys(b2);
    const sharedKeys = util.objectKeys(a2).filter((key) => bKeys.indexOf(key) !== -1);
    const newObj = { ...a2, ...b2 };
    for (const key of sharedKeys) {
      const sharedValue = mergeValues(a2[key], b2[key]);
      if (!sharedValue.valid) {
        return { valid: false };
      }
      newObj[key] = sharedValue.data;
    }
    return { valid: true, data: newObj };
  } else if (aType === ZodParsedType.array && bType === ZodParsedType.array) {
    if (a2.length !== b2.length) {
      return { valid: false };
    }
    const newArray = [];
    for (let index = 0; index < a2.length; index++) {
      const itemA = a2[index];
      const itemB = b2[index];
      const sharedValue = mergeValues(itemA, itemB);
      if (!sharedValue.valid) {
        return { valid: false };
      }
      newArray.push(sharedValue.data);
    }
    return { valid: true, data: newArray };
  } else if (aType === ZodParsedType.date && bType === ZodParsedType.date && +a2 === +b2) {
    return { valid: true, data: a2 };
  } else {
    return { valid: false };
  }
}
class ZodIntersection extends ZodType {
  _parse(input) {
    const { status, ctx } = this._processInputParams(input);
    const handleParsed = (parsedLeft, parsedRight) => {
      if (isAborted(parsedLeft) || isAborted(parsedRight)) {
        return INVALID;
      }
      const merged = mergeValues(parsedLeft.value, parsedRight.value);
      if (!merged.valid) {
        addIssueToContext(ctx, {
          code: ZodIssueCode.invalid_intersection_types
        });
        return INVALID;
      }
      if (isDirty(parsedLeft) || isDirty(parsedRight)) {
        status.dirty();
      }
      return { status: status.value, value: merged.data };
    };
    if (ctx.common.async) {
      return Promise.all([
        this._def.left._parseAsync({
          data: ctx.data,
          path: ctx.path,
          parent: ctx
        }),
        this._def.right._parseAsync({
          data: ctx.data,
          path: ctx.path,
          parent: ctx
        })
      ]).then(([left, right]) => handleParsed(left, right));
    } else {
      return handleParsed(this._def.left._parseSync({
        data: ctx.data,
        path: ctx.path,
        parent: ctx
      }), this._def.right._parseSync({
        data: ctx.data,
        path: ctx.path,
        parent: ctx
      }));
    }
  }
}
ZodIntersection.create = (left, right, params) => {
  return new ZodIntersection({
    left,
    right,
    typeName: ZodFirstPartyTypeKind.ZodIntersection,
    ...processCreateParams(params)
  });
};
class ZodTuple extends ZodType {
  _parse(input) {
    const { status, ctx } = this._processInputParams(input);
    if (ctx.parsedType !== ZodParsedType.array) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.array,
        received: ctx.parsedType
      });
      return INVALID;
    }
    if (ctx.data.length < this._def.items.length) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.too_small,
        minimum: this._def.items.length,
        inclusive: true,
        exact: false,
        type: "array"
      });
      return INVALID;
    }
    const rest = this._def.rest;
    if (!rest && ctx.data.length > this._def.items.length) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.too_big,
        maximum: this._def.items.length,
        inclusive: true,
        exact: false,
        type: "array"
      });
      status.dirty();
    }
    const items = [...ctx.data].map((item, itemIndex) => {
      const schema = this._def.items[itemIndex] || this._def.rest;
      if (!schema)
        return null;
      return schema._parse(new ParseInputLazyPath(ctx, item, ctx.path, itemIndex));
    }).filter((x2) => !!x2);
    if (ctx.common.async) {
      return Promise.all(items).then((results) => {
        return ParseStatus.mergeArray(status, results);
      });
    } else {
      return ParseStatus.mergeArray(status, items);
    }
  }
  get items() {
    return this._def.items;
  }
  rest(rest) {
    return new ZodTuple({
      ...this._def,
      rest
    });
  }
}
ZodTuple.create = (schemas, params) => {
  if (!Array.isArray(schemas)) {
    throw new Error("You must pass an array of schemas to z.tuple([ ... ])");
  }
  return new ZodTuple({
    items: schemas,
    typeName: ZodFirstPartyTypeKind.ZodTuple,
    rest: null,
    ...processCreateParams(params)
  });
};
class ZodRecord extends ZodType {
  get keySchema() {
    return this._def.keyType;
  }
  get valueSchema() {
    return this._def.valueType;
  }
  _parse(input) {
    const { status, ctx } = this._processInputParams(input);
    if (ctx.parsedType !== ZodParsedType.object) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.object,
        received: ctx.parsedType
      });
      return INVALID;
    }
    const pairs = [];
    const keyType = this._def.keyType;
    const valueType = this._def.valueType;
    for (const key in ctx.data) {
      pairs.push({
        key: keyType._parse(new ParseInputLazyPath(ctx, key, ctx.path, key)),
        value: valueType._parse(new ParseInputLazyPath(ctx, ctx.data[key], ctx.path, key))
      });
    }
    if (ctx.common.async) {
      return ParseStatus.mergeObjectAsync(status, pairs);
    } else {
      return ParseStatus.mergeObjectSync(status, pairs);
    }
  }
  get element() {
    return this._def.valueType;
  }
  static create(first, second, third) {
    if (second instanceof ZodType) {
      return new ZodRecord({
        keyType: first,
        valueType: second,
        typeName: ZodFirstPartyTypeKind.ZodRecord,
        ...processCreateParams(third)
      });
    }
    return new ZodRecord({
      keyType: ZodString.create(),
      valueType: first,
      typeName: ZodFirstPartyTypeKind.ZodRecord,
      ...processCreateParams(second)
    });
  }
}
class ZodMap extends ZodType {
  get keySchema() {
    return this._def.keyType;
  }
  get valueSchema() {
    return this._def.valueType;
  }
  _parse(input) {
    const { status, ctx } = this._processInputParams(input);
    if (ctx.parsedType !== ZodParsedType.map) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.map,
        received: ctx.parsedType
      });
      return INVALID;
    }
    const keyType = this._def.keyType;
    const valueType = this._def.valueType;
    const pairs = [...ctx.data.entries()].map(([key, value], index) => {
      return {
        key: keyType._parse(new ParseInputLazyPath(ctx, key, ctx.path, [index, "key"])),
        value: valueType._parse(new ParseInputLazyPath(ctx, value, ctx.path, [index, "value"]))
      };
    });
    if (ctx.common.async) {
      const finalMap = /* @__PURE__ */ new Map();
      return Promise.resolve().then(async () => {
        for (const pair of pairs) {
          const key = await pair.key;
          const value = await pair.value;
          if (key.status === "aborted" || value.status === "aborted") {
            return INVALID;
          }
          if (key.status === "dirty" || value.status === "dirty") {
            status.dirty();
          }
          finalMap.set(key.value, value.value);
        }
        return { status: status.value, value: finalMap };
      });
    } else {
      const finalMap = /* @__PURE__ */ new Map();
      for (const pair of pairs) {
        const key = pair.key;
        const value = pair.value;
        if (key.status === "aborted" || value.status === "aborted") {
          return INVALID;
        }
        if (key.status === "dirty" || value.status === "dirty") {
          status.dirty();
        }
        finalMap.set(key.value, value.value);
      }
      return { status: status.value, value: finalMap };
    }
  }
}
ZodMap.create = (keyType, valueType, params) => {
  return new ZodMap({
    valueType,
    keyType,
    typeName: ZodFirstPartyTypeKind.ZodMap,
    ...processCreateParams(params)
  });
};
class ZodSet extends ZodType {
  _parse(input) {
    const { status, ctx } = this._processInputParams(input);
    if (ctx.parsedType !== ZodParsedType.set) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.set,
        received: ctx.parsedType
      });
      return INVALID;
    }
    const def = this._def;
    if (def.minSize !== null) {
      if (ctx.data.size < def.minSize.value) {
        addIssueToContext(ctx, {
          code: ZodIssueCode.too_small,
          minimum: def.minSize.value,
          type: "set",
          inclusive: true,
          exact: false,
          message: def.minSize.message
        });
        status.dirty();
      }
    }
    if (def.maxSize !== null) {
      if (ctx.data.size > def.maxSize.value) {
        addIssueToContext(ctx, {
          code: ZodIssueCode.too_big,
          maximum: def.maxSize.value,
          type: "set",
          inclusive: true,
          exact: false,
          message: def.maxSize.message
        });
        status.dirty();
      }
    }
    const valueType = this._def.valueType;
    function finalizeSet(elements2) {
      const parsedSet = /* @__PURE__ */ new Set();
      for (const element of elements2) {
        if (element.status === "aborted")
          return INVALID;
        if (element.status === "dirty")
          status.dirty();
        parsedSet.add(element.value);
      }
      return { status: status.value, value: parsedSet };
    }
    const elements = [...ctx.data.values()].map((item, i2) => valueType._parse(new ParseInputLazyPath(ctx, item, ctx.path, i2)));
    if (ctx.common.async) {
      return Promise.all(elements).then((elements2) => finalizeSet(elements2));
    } else {
      return finalizeSet(elements);
    }
  }
  min(minSize, message) {
    return new ZodSet({
      ...this._def,
      minSize: { value: minSize, message: errorUtil.toString(message) }
    });
  }
  max(maxSize, message) {
    return new ZodSet({
      ...this._def,
      maxSize: { value: maxSize, message: errorUtil.toString(message) }
    });
  }
  size(size, message) {
    return this.min(size, message).max(size, message);
  }
  nonempty(message) {
    return this.min(1, message);
  }
}
ZodSet.create = (valueType, params) => {
  return new ZodSet({
    valueType,
    minSize: null,
    maxSize: null,
    typeName: ZodFirstPartyTypeKind.ZodSet,
    ...processCreateParams(params)
  });
};
class ZodFunction extends ZodType {
  constructor() {
    super(...arguments);
    this.validate = this.implement;
  }
  _parse(input) {
    const { ctx } = this._processInputParams(input);
    if (ctx.parsedType !== ZodParsedType.function) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.function,
        received: ctx.parsedType
      });
      return INVALID;
    }
    function makeArgsIssue(args, error) {
      return makeIssue({
        data: args,
        path: ctx.path,
        errorMaps: [
          ctx.common.contextualErrorMap,
          ctx.schemaErrorMap,
          getErrorMap(),
          errorMap
        ].filter((x2) => !!x2),
        issueData: {
          code: ZodIssueCode.invalid_arguments,
          argumentsError: error
        }
      });
    }
    function makeReturnsIssue(returns, error) {
      return makeIssue({
        data: returns,
        path: ctx.path,
        errorMaps: [
          ctx.common.contextualErrorMap,
          ctx.schemaErrorMap,
          getErrorMap(),
          errorMap
        ].filter((x2) => !!x2),
        issueData: {
          code: ZodIssueCode.invalid_return_type,
          returnTypeError: error
        }
      });
    }
    const params = { errorMap: ctx.common.contextualErrorMap };
    const fn = ctx.data;
    if (this._def.returns instanceof ZodPromise) {
      const me = this;
      return OK(async function(...args) {
        const error = new ZodError([]);
        const parsedArgs = await me._def.args.parseAsync(args, params).catch((e2) => {
          error.addIssue(makeArgsIssue(args, e2));
          throw error;
        });
        const result = await Reflect.apply(fn, this, parsedArgs);
        const parsedReturns = await me._def.returns._def.type.parseAsync(result, params).catch((e2) => {
          error.addIssue(makeReturnsIssue(result, e2));
          throw error;
        });
        return parsedReturns;
      });
    } else {
      const me = this;
      return OK(function(...args) {
        const parsedArgs = me._def.args.safeParse(args, params);
        if (!parsedArgs.success) {
          throw new ZodError([makeArgsIssue(args, parsedArgs.error)]);
        }
        const result = Reflect.apply(fn, this, parsedArgs.data);
        const parsedReturns = me._def.returns.safeParse(result, params);
        if (!parsedReturns.success) {
          throw new ZodError([makeReturnsIssue(result, parsedReturns.error)]);
        }
        return parsedReturns.data;
      });
    }
  }
  parameters() {
    return this._def.args;
  }
  returnType() {
    return this._def.returns;
  }
  args(...items) {
    return new ZodFunction({
      ...this._def,
      args: ZodTuple.create(items).rest(ZodUnknown.create())
    });
  }
  returns(returnType) {
    return new ZodFunction({
      ...this._def,
      returns: returnType
    });
  }
  implement(func) {
    const validatedFunc = this.parse(func);
    return validatedFunc;
  }
  strictImplement(func) {
    const validatedFunc = this.parse(func);
    return validatedFunc;
  }
  static create(args, returns, params) {
    return new ZodFunction({
      args: args ? args : ZodTuple.create([]).rest(ZodUnknown.create()),
      returns: returns || ZodUnknown.create(),
      typeName: ZodFirstPartyTypeKind.ZodFunction,
      ...processCreateParams(params)
    });
  }
}
class ZodLazy extends ZodType {
  get schema() {
    return this._def.getter();
  }
  _parse(input) {
    const { ctx } = this._processInputParams(input);
    const lazySchema = this._def.getter();
    return lazySchema._parse({ data: ctx.data, path: ctx.path, parent: ctx });
  }
}
ZodLazy.create = (getter, params) => {
  return new ZodLazy({
    getter,
    typeName: ZodFirstPartyTypeKind.ZodLazy,
    ...processCreateParams(params)
  });
};
class ZodLiteral extends ZodType {
  _parse(input) {
    if (input.data !== this._def.value) {
      const ctx = this._getOrReturnCtx(input);
      addIssueToContext(ctx, {
        received: ctx.data,
        code: ZodIssueCode.invalid_literal,
        expected: this._def.value
      });
      return INVALID;
    }
    return { status: "valid", value: input.data };
  }
  get value() {
    return this._def.value;
  }
}
ZodLiteral.create = (value, params) => {
  return new ZodLiteral({
    value,
    typeName: ZodFirstPartyTypeKind.ZodLiteral,
    ...processCreateParams(params)
  });
};
function createZodEnum(values, params) {
  return new ZodEnum({
    values,
    typeName: ZodFirstPartyTypeKind.ZodEnum,
    ...processCreateParams(params)
  });
}
class ZodEnum extends ZodType {
  _parse(input) {
    if (typeof input.data !== "string") {
      const ctx = this._getOrReturnCtx(input);
      const expectedValues = this._def.values;
      addIssueToContext(ctx, {
        expected: util.joinValues(expectedValues),
        received: ctx.parsedType,
        code: ZodIssueCode.invalid_type
      });
      return INVALID;
    }
    if (this._def.values.indexOf(input.data) === -1) {
      const ctx = this._getOrReturnCtx(input);
      const expectedValues = this._def.values;
      addIssueToContext(ctx, {
        received: ctx.data,
        code: ZodIssueCode.invalid_enum_value,
        options: expectedValues
      });
      return INVALID;
    }
    return OK(input.data);
  }
  get options() {
    return this._def.values;
  }
  get enum() {
    const enumValues = {};
    for (const val of this._def.values) {
      enumValues[val] = val;
    }
    return enumValues;
  }
  get Values() {
    const enumValues = {};
    for (const val of this._def.values) {
      enumValues[val] = val;
    }
    return enumValues;
  }
  get Enum() {
    const enumValues = {};
    for (const val of this._def.values) {
      enumValues[val] = val;
    }
    return enumValues;
  }
  extract(values) {
    return ZodEnum.create(values);
  }
  exclude(values) {
    return ZodEnum.create(this.options.filter((opt) => !values.includes(opt)));
  }
}
ZodEnum.create = createZodEnum;
class ZodNativeEnum extends ZodType {
  _parse(input) {
    const nativeEnumValues = util.getValidEnumValues(this._def.values);
    const ctx = this._getOrReturnCtx(input);
    if (ctx.parsedType !== ZodParsedType.string && ctx.parsedType !== ZodParsedType.number) {
      const expectedValues = util.objectValues(nativeEnumValues);
      addIssueToContext(ctx, {
        expected: util.joinValues(expectedValues),
        received: ctx.parsedType,
        code: ZodIssueCode.invalid_type
      });
      return INVALID;
    }
    if (nativeEnumValues.indexOf(input.data) === -1) {
      const expectedValues = util.objectValues(nativeEnumValues);
      addIssueToContext(ctx, {
        received: ctx.data,
        code: ZodIssueCode.invalid_enum_value,
        options: expectedValues
      });
      return INVALID;
    }
    return OK(input.data);
  }
  get enum() {
    return this._def.values;
  }
}
ZodNativeEnum.create = (values, params) => {
  return new ZodNativeEnum({
    values,
    typeName: ZodFirstPartyTypeKind.ZodNativeEnum,
    ...processCreateParams(params)
  });
};
class ZodPromise extends ZodType {
  unwrap() {
    return this._def.type;
  }
  _parse(input) {
    const { ctx } = this._processInputParams(input);
    if (ctx.parsedType !== ZodParsedType.promise && ctx.common.async === false) {
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.promise,
        received: ctx.parsedType
      });
      return INVALID;
    }
    const promisified = ctx.parsedType === ZodParsedType.promise ? ctx.data : Promise.resolve(ctx.data);
    return OK(promisified.then((data) => {
      return this._def.type.parseAsync(data, {
        path: ctx.path,
        errorMap: ctx.common.contextualErrorMap
      });
    }));
  }
}
ZodPromise.create = (schema, params) => {
  return new ZodPromise({
    type: schema,
    typeName: ZodFirstPartyTypeKind.ZodPromise,
    ...processCreateParams(params)
  });
};
class ZodEffects extends ZodType {
  innerType() {
    return this._def.schema;
  }
  sourceType() {
    return this._def.schema._def.typeName === ZodFirstPartyTypeKind.ZodEffects ? this._def.schema.sourceType() : this._def.schema;
  }
  _parse(input) {
    const { status, ctx } = this._processInputParams(input);
    const effect = this._def.effect || null;
    const checkCtx = {
      addIssue: (arg) => {
        addIssueToContext(ctx, arg);
        if (arg.fatal) {
          status.abort();
        } else {
          status.dirty();
        }
      },
      get path() {
        return ctx.path;
      }
    };
    checkCtx.addIssue = checkCtx.addIssue.bind(checkCtx);
    if (effect.type === "preprocess") {
      const processed = effect.transform(ctx.data, checkCtx);
      if (ctx.common.issues.length) {
        return {
          status: "dirty",
          value: ctx.data
        };
      }
      if (ctx.common.async) {
        return Promise.resolve(processed).then((processed2) => {
          return this._def.schema._parseAsync({
            data: processed2,
            path: ctx.path,
            parent: ctx
          });
        });
      } else {
        return this._def.schema._parseSync({
          data: processed,
          path: ctx.path,
          parent: ctx
        });
      }
    }
    if (effect.type === "refinement") {
      const executeRefinement = (acc) => {
        const result = effect.refinement(acc, checkCtx);
        if (ctx.common.async) {
          return Promise.resolve(result);
        }
        if (result instanceof Promise) {
          throw new Error("Async refinement encountered during synchronous parse operation. Use .parseAsync instead.");
        }
        return acc;
      };
      if (ctx.common.async === false) {
        const inner = this._def.schema._parseSync({
          data: ctx.data,
          path: ctx.path,
          parent: ctx
        });
        if (inner.status === "aborted")
          return INVALID;
        if (inner.status === "dirty")
          status.dirty();
        executeRefinement(inner.value);
        return { status: status.value, value: inner.value };
      } else {
        return this._def.schema._parseAsync({ data: ctx.data, path: ctx.path, parent: ctx }).then((inner) => {
          if (inner.status === "aborted")
            return INVALID;
          if (inner.status === "dirty")
            status.dirty();
          return executeRefinement(inner.value).then(() => {
            return { status: status.value, value: inner.value };
          });
        });
      }
    }
    if (effect.type === "transform") {
      if (ctx.common.async === false) {
        const base = this._def.schema._parseSync({
          data: ctx.data,
          path: ctx.path,
          parent: ctx
        });
        if (!isValid(base))
          return base;
        const result = effect.transform(base.value, checkCtx);
        if (result instanceof Promise) {
          throw new Error(`Asynchronous transform encountered during synchronous parse operation. Use .parseAsync instead.`);
        }
        return { status: status.value, value: result };
      } else {
        return this._def.schema._parseAsync({ data: ctx.data, path: ctx.path, parent: ctx }).then((base) => {
          if (!isValid(base))
            return base;
          return Promise.resolve(effect.transform(base.value, checkCtx)).then((result) => ({ status: status.value, value: result }));
        });
      }
    }
    util.assertNever(effect);
  }
}
ZodEffects.create = (schema, effect, params) => {
  return new ZodEffects({
    schema,
    typeName: ZodFirstPartyTypeKind.ZodEffects,
    effect,
    ...processCreateParams(params)
  });
};
ZodEffects.createWithPreprocess = (preprocess, schema, params) => {
  return new ZodEffects({
    schema,
    effect: { type: "preprocess", transform: preprocess },
    typeName: ZodFirstPartyTypeKind.ZodEffects,
    ...processCreateParams(params)
  });
};
class ZodOptional extends ZodType {
  _parse(input) {
    const parsedType = this._getType(input);
    if (parsedType === ZodParsedType.undefined) {
      return OK(void 0);
    }
    return this._def.innerType._parse(input);
  }
  unwrap() {
    return this._def.innerType;
  }
}
ZodOptional.create = (type, params) => {
  return new ZodOptional({
    innerType: type,
    typeName: ZodFirstPartyTypeKind.ZodOptional,
    ...processCreateParams(params)
  });
};
class ZodNullable extends ZodType {
  _parse(input) {
    const parsedType = this._getType(input);
    if (parsedType === ZodParsedType.null) {
      return OK(null);
    }
    return this._def.innerType._parse(input);
  }
  unwrap() {
    return this._def.innerType;
  }
}
ZodNullable.create = (type, params) => {
  return new ZodNullable({
    innerType: type,
    typeName: ZodFirstPartyTypeKind.ZodNullable,
    ...processCreateParams(params)
  });
};
class ZodDefault extends ZodType {
  _parse(input) {
    const { ctx } = this._processInputParams(input);
    let data = ctx.data;
    if (ctx.parsedType === ZodParsedType.undefined) {
      data = this._def.defaultValue();
    }
    return this._def.innerType._parse({
      data,
      path: ctx.path,
      parent: ctx
    });
  }
  removeDefault() {
    return this._def.innerType;
  }
}
ZodDefault.create = (type, params) => {
  return new ZodDefault({
    innerType: type,
    typeName: ZodFirstPartyTypeKind.ZodDefault,
    defaultValue: typeof params.default === "function" ? params.default : () => params.default,
    ...processCreateParams(params)
  });
};
class ZodCatch extends ZodType {
  _parse(input) {
    const { ctx } = this._processInputParams(input);
    const newCtx = {
      ...ctx,
      common: {
        ...ctx.common,
        issues: []
      }
    };
    const result = this._def.innerType._parse({
      data: newCtx.data,
      path: newCtx.path,
      parent: {
        ...newCtx
      }
    });
    if (isAsync(result)) {
      return result.then((result2) => {
        return {
          status: "valid",
          value: result2.status === "valid" ? result2.value : this._def.catchValue({
            get error() {
              return new ZodError(newCtx.common.issues);
            },
            input: newCtx.data
          })
        };
      });
    } else {
      return {
        status: "valid",
        value: result.status === "valid" ? result.value : this._def.catchValue({
          get error() {
            return new ZodError(newCtx.common.issues);
          },
          input: newCtx.data
        })
      };
    }
  }
  removeCatch() {
    return this._def.innerType;
  }
}
ZodCatch.create = (type, params) => {
  return new ZodCatch({
    innerType: type,
    typeName: ZodFirstPartyTypeKind.ZodCatch,
    catchValue: typeof params.catch === "function" ? params.catch : () => params.catch,
    ...processCreateParams(params)
  });
};
class ZodNaN extends ZodType {
  _parse(input) {
    const parsedType = this._getType(input);
    if (parsedType !== ZodParsedType.nan) {
      const ctx = this._getOrReturnCtx(input);
      addIssueToContext(ctx, {
        code: ZodIssueCode.invalid_type,
        expected: ZodParsedType.nan,
        received: ctx.parsedType
      });
      return INVALID;
    }
    return { status: "valid", value: input.data };
  }
}
ZodNaN.create = (params) => {
  return new ZodNaN({
    typeName: ZodFirstPartyTypeKind.ZodNaN,
    ...processCreateParams(params)
  });
};
const BRAND = Symbol("zod_brand");
class ZodBranded extends ZodType {
  _parse(input) {
    const { ctx } = this._processInputParams(input);
    const data = ctx.data;
    return this._def.type._parse({
      data,
      path: ctx.path,
      parent: ctx
    });
  }
  unwrap() {
    return this._def.type;
  }
}
class ZodPipeline extends ZodType {
  _parse(input) {
    const { status, ctx } = this._processInputParams(input);
    if (ctx.common.async) {
      const handleAsync = async () => {
        const inResult = await this._def.in._parseAsync({
          data: ctx.data,
          path: ctx.path,
          parent: ctx
        });
        if (inResult.status === "aborted")
          return INVALID;
        if (inResult.status === "dirty") {
          status.dirty();
          return DIRTY(inResult.value);
        } else {
          return this._def.out._parseAsync({
            data: inResult.value,
            path: ctx.path,
            parent: ctx
          });
        }
      };
      return handleAsync();
    } else {
      const inResult = this._def.in._parseSync({
        data: ctx.data,
        path: ctx.path,
        parent: ctx
      });
      if (inResult.status === "aborted")
        return INVALID;
      if (inResult.status === "dirty") {
        status.dirty();
        return {
          status: "dirty",
          value: inResult.value
        };
      } else {
        return this._def.out._parseSync({
          data: inResult.value,
          path: ctx.path,
          parent: ctx
        });
      }
    }
  }
  static create(a2, b2) {
    return new ZodPipeline({
      in: a2,
      out: b2,
      typeName: ZodFirstPartyTypeKind.ZodPipeline
    });
  }
}
class ZodReadonly extends ZodType {
  _parse(input) {
    const result = this._def.innerType._parse(input);
    if (isValid(result)) {
      result.value = Object.freeze(result.value);
    }
    return result;
  }
}
ZodReadonly.create = (type, params) => {
  return new ZodReadonly({
    innerType: type,
    typeName: ZodFirstPartyTypeKind.ZodReadonly,
    ...processCreateParams(params)
  });
};
const custom = (check, params = {}, fatal) => {
  if (check)
    return ZodAny.create().superRefine((data, ctx) => {
      var _a, _b;
      if (!check(data)) {
        const p2 = typeof params === "function" ? params(data) : typeof params === "string" ? { message: params } : params;
        const _fatal = (_b = (_a = p2.fatal) !== null && _a !== void 0 ? _a : fatal) !== null && _b !== void 0 ? _b : true;
        const p22 = typeof p2 === "string" ? { message: p2 } : p2;
        ctx.addIssue({ code: "custom", ...p22, fatal: _fatal });
      }
    });
  return ZodAny.create();
};
const late = {
  object: ZodObject.lazycreate
};
var ZodFirstPartyTypeKind;
(function(ZodFirstPartyTypeKind2) {
  ZodFirstPartyTypeKind2["ZodString"] = "ZodString";
  ZodFirstPartyTypeKind2["ZodNumber"] = "ZodNumber";
  ZodFirstPartyTypeKind2["ZodNaN"] = "ZodNaN";
  ZodFirstPartyTypeKind2["ZodBigInt"] = "ZodBigInt";
  ZodFirstPartyTypeKind2["ZodBoolean"] = "ZodBoolean";
  ZodFirstPartyTypeKind2["ZodDate"] = "ZodDate";
  ZodFirstPartyTypeKind2["ZodSymbol"] = "ZodSymbol";
  ZodFirstPartyTypeKind2["ZodUndefined"] = "ZodUndefined";
  ZodFirstPartyTypeKind2["ZodNull"] = "ZodNull";
  ZodFirstPartyTypeKind2["ZodAny"] = "ZodAny";
  ZodFirstPartyTypeKind2["ZodUnknown"] = "ZodUnknown";
  ZodFirstPartyTypeKind2["ZodNever"] = "ZodNever";
  ZodFirstPartyTypeKind2["ZodVoid"] = "ZodVoid";
  ZodFirstPartyTypeKind2["ZodArray"] = "ZodArray";
  ZodFirstPartyTypeKind2["ZodObject"] = "ZodObject";
  ZodFirstPartyTypeKind2["ZodUnion"] = "ZodUnion";
  ZodFirstPartyTypeKind2["ZodDiscriminatedUnion"] = "ZodDiscriminatedUnion";
  ZodFirstPartyTypeKind2["ZodIntersection"] = "ZodIntersection";
  ZodFirstPartyTypeKind2["ZodTuple"] = "ZodTuple";
  ZodFirstPartyTypeKind2["ZodRecord"] = "ZodRecord";
  ZodFirstPartyTypeKind2["ZodMap"] = "ZodMap";
  ZodFirstPartyTypeKind2["ZodSet"] = "ZodSet";
  ZodFirstPartyTypeKind2["ZodFunction"] = "ZodFunction";
  ZodFirstPartyTypeKind2["ZodLazy"] = "ZodLazy";
  ZodFirstPartyTypeKind2["ZodLiteral"] = "ZodLiteral";
  ZodFirstPartyTypeKind2["ZodEnum"] = "ZodEnum";
  ZodFirstPartyTypeKind2["ZodEffects"] = "ZodEffects";
  ZodFirstPartyTypeKind2["ZodNativeEnum"] = "ZodNativeEnum";
  ZodFirstPartyTypeKind2["ZodOptional"] = "ZodOptional";
  ZodFirstPartyTypeKind2["ZodNullable"] = "ZodNullable";
  ZodFirstPartyTypeKind2["ZodDefault"] = "ZodDefault";
  ZodFirstPartyTypeKind2["ZodCatch"] = "ZodCatch";
  ZodFirstPartyTypeKind2["ZodPromise"] = "ZodPromise";
  ZodFirstPartyTypeKind2["ZodBranded"] = "ZodBranded";
  ZodFirstPartyTypeKind2["ZodPipeline"] = "ZodPipeline";
  ZodFirstPartyTypeKind2["ZodReadonly"] = "ZodReadonly";
})(ZodFirstPartyTypeKind || (ZodFirstPartyTypeKind = {}));
const instanceOfType = (cls, params = {
  message: `Input not instance of ${cls.name}`
}) => custom((data) => data instanceof cls, params);
const stringType = ZodString.create;
const numberType = ZodNumber.create;
const nanType = ZodNaN.create;
const bigIntType = ZodBigInt.create;
const booleanType = ZodBoolean.create;
const dateType = ZodDate.create;
const symbolType = ZodSymbol.create;
const undefinedType = ZodUndefined.create;
const nullType = ZodNull.create;
const anyType = ZodAny.create;
const unknownType = ZodUnknown.create;
const neverType = ZodNever.create;
const voidType = ZodVoid.create;
const arrayType = ZodArray.create;
const objectType = ZodObject.create;
const strictObjectType = ZodObject.strictCreate;
const unionType = ZodUnion.create;
const discriminatedUnionType = ZodDiscriminatedUnion.create;
const intersectionType = ZodIntersection.create;
const tupleType = ZodTuple.create;
const recordType = ZodRecord.create;
const mapType = ZodMap.create;
const setType = ZodSet.create;
const functionType = ZodFunction.create;
const lazyType = ZodLazy.create;
const literalType = ZodLiteral.create;
const enumType = ZodEnum.create;
const nativeEnumType = ZodNativeEnum.create;
const promiseType = ZodPromise.create;
const effectsType = ZodEffects.create;
const optionalType = ZodOptional.create;
const nullableType = ZodNullable.create;
const preprocessType = ZodEffects.createWithPreprocess;
const pipelineType = ZodPipeline.create;
const ostring = () => stringType().optional();
const onumber = () => numberType().optional();
const oboolean = () => booleanType().optional();
const coerce = {
  string: (arg) => ZodString.create({ ...arg, coerce: true }),
  number: (arg) => ZodNumber.create({ ...arg, coerce: true }),
  boolean: (arg) => ZodBoolean.create({
    ...arg,
    coerce: true
  }),
  bigint: (arg) => ZodBigInt.create({ ...arg, coerce: true }),
  date: (arg) => ZodDate.create({ ...arg, coerce: true })
};
const NEVER = INVALID;
var z = /* @__PURE__ */ Object.freeze({
  __proto__: null,
  defaultErrorMap: errorMap,
  setErrorMap,
  getErrorMap,
  makeIssue,
  EMPTY_PATH,
  addIssueToContext,
  ParseStatus,
  INVALID,
  DIRTY,
  OK,
  isAborted,
  isDirty,
  isValid,
  isAsync,
  get util() {
    return util;
  },
  get objectUtil() {
    return objectUtil;
  },
  ZodParsedType,
  getParsedType,
  ZodType,
  ZodString,
  ZodNumber,
  ZodBigInt,
  ZodBoolean,
  ZodDate,
  ZodSymbol,
  ZodUndefined,
  ZodNull,
  ZodAny,
  ZodUnknown,
  ZodNever,
  ZodVoid,
  ZodArray,
  ZodObject,
  ZodUnion,
  ZodDiscriminatedUnion,
  ZodIntersection,
  ZodTuple,
  ZodRecord,
  ZodMap,
  ZodSet,
  ZodFunction,
  ZodLazy,
  ZodLiteral,
  ZodEnum,
  ZodNativeEnum,
  ZodPromise,
  ZodEffects,
  ZodTransformer: ZodEffects,
  ZodOptional,
  ZodNullable,
  ZodDefault,
  ZodCatch,
  ZodNaN,
  BRAND,
  ZodBranded,
  ZodPipeline,
  ZodReadonly,
  custom,
  Schema: ZodType,
  ZodSchema: ZodType,
  late,
  get ZodFirstPartyTypeKind() {
    return ZodFirstPartyTypeKind;
  },
  coerce,
  any: anyType,
  array: arrayType,
  bigint: bigIntType,
  boolean: booleanType,
  date: dateType,
  discriminatedUnion: discriminatedUnionType,
  effect: effectsType,
  "enum": enumType,
  "function": functionType,
  "instanceof": instanceOfType,
  intersection: intersectionType,
  lazy: lazyType,
  literal: literalType,
  map: mapType,
  nan: nanType,
  nativeEnum: nativeEnumType,
  never: neverType,
  "null": nullType,
  nullable: nullableType,
  number: numberType,
  object: objectType,
  oboolean,
  onumber,
  optional: optionalType,
  ostring,
  pipeline: pipelineType,
  preprocess: preprocessType,
  promise: promiseType,
  record: recordType,
  set: setType,
  strictObject: strictObjectType,
  string: stringType,
  symbol: symbolType,
  transformer: effectsType,
  tuple: tupleType,
  "undefined": undefinedType,
  union: unionType,
  unknown: unknownType,
  "void": voidType,
  NEVER,
  ZodIssueCode,
  quotelessJson,
  ZodError
});
z.string();
z.number();
z.number().optional();
const ZColor = z.string().regex(/^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/);
const ZPlacement = z.enum(["bottomLeft", "bottomRight", "topLeft", "topRight", "center"]);
const ZAllowedFileExtension = z.enum(["png", "jpeg", "jpg", "pdf", "doc", "docx", "xls", "xlsx", "ppt", "pptx", "plain", "csv", "mp4", "mov", "avi", "mkv", "webm", "zip", "rar", "7z", "tar"]);
const ZSurveyThankYouCard = z.object({
  enabled: z.boolean(),
  headline: z.optional(z.string()),
  subheader: z.optional(z.string())
});
var TSurveyQuestionType = /* @__PURE__ */ ((TSurveyQuestionType2) => {
  TSurveyQuestionType2["FileUpload"] = "fileUpload";
  TSurveyQuestionType2["OpenText"] = "openText";
  TSurveyQuestionType2["MultipleChoiceSingle"] = "multipleChoiceSingle";
  TSurveyQuestionType2["MultipleChoiceMulti"] = "multipleChoiceMulti";
  TSurveyQuestionType2["NPS"] = "nps";
  TSurveyQuestionType2["CTA"] = "cta";
  TSurveyQuestionType2["Rating"] = "rating";
  TSurveyQuestionType2["Consent"] = "consent";
  TSurveyQuestionType2["PictureSelection"] = "pictureSelection";
  TSurveyQuestionType2["Cal"] = "cal";
  TSurveyQuestionType2["Date"] = "date";
  return TSurveyQuestionType2;
})(TSurveyQuestionType || {});
const ZSurveyWelcomeCard = z.object({
  enabled: z.boolean(),
  headline: z.optional(z.string()),
  html: z.string().optional(),
  fileUrl: z.string().optional(),
  buttonLabel: z.string().optional(),
  timeToFinish: z.boolean().default(true),
  showResponseCount: z.boolean().default(false)
});
const ZSurveyHiddenFields = z.object({
  enabled: z.boolean(),
  fieldIds: z.optional(z.array(z.string()))
});
const ZSurveyProductOverwrites = z.object({
  brandColor: ZColor.nullish(),
  highlightBorderColor: ZColor.nullish(),
  placement: ZPlacement.nullish(),
  clickOutsideClose: z.boolean().nullish(),
  darkOverlay: z.boolean().nullish()
});
z.enum(["animation", "color", "image"]);
const ZSurveyStylingBackground = z.object({
  bg: z.string().nullish(),
  bgType: z.enum(["animation", "color", "image"]).nullish(),
  brightness: z.number().nullish()
});
const ZSurveyStyling = z.object({
  background: ZSurveyStylingBackground.nullish()
});
const ZSurveyClosedMessage = z.object({
  enabled: z.boolean().optional(),
  heading: z.string().optional(),
  subheading: z.string().optional()
}).nullable().optional();
const ZSurveySingleUse = z.object({
  enabled: z.boolean(),
  heading: z.optional(z.string()),
  subheading: z.optional(z.string()),
  isEncrypted: z.boolean()
}).nullable();
const ZSurveyVerifyEmail = z.object({
  name: z.optional(z.string()),
  subheading: z.optional(z.string())
}).optional();
const ZSurveyChoice = z.object({
  id: z.string(),
  label: z.string()
});
const ZSurveyPictureChoice = z.object({
  id: z.string(),
  imageUrl: z.string()
});
const ZSurveyLogicCondition = z.enum(["accepted", "clicked", "submitted", "skipped", "equals", "notEquals", "lessThan", "lessEqual", "greaterThan", "greaterEqual", "includesAll", "includesOne", "uploaded", "notUploaded", "booked"]);
const ZSurveyLogicBase = z.object({
  condition: ZSurveyLogicCondition.optional(),
  value: z.union([z.string(), z.array(z.string())]).optional(),
  destination: z.union([z.string(), z.literal("end")]).optional()
});
const ZSurveyFileUploadLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["uploaded", "notUploaded"]).optional(),
  value: z.undefined()
});
const ZSurveyOpenTextLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["submitted", "skipped"]).optional(),
  value: z.undefined()
});
const ZSurveyConsentLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["skipped", "accepted"]).optional(),
  value: z.undefined()
});
const ZSurveyMultipleChoiceSingleLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["submitted", "skipped", "equals", "notEquals"]).optional(),
  value: z.string().optional()
});
const ZSurveyMultipleChoiceMultiLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["submitted", "skipped", "includesAll", "includesOne", "equals"]).optional(),
  value: z.union([z.array(z.string()), z.string()]).optional()
});
const ZSurveyNPSLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["equals", "notEquals", "lessThan", "lessEqual", "greaterThan", "greaterEqual", "submitted", "skipped"]).optional(),
  value: z.union([z.string(), z.number()]).optional()
});
const ZSurveyCTALogic = ZSurveyLogicBase.extend({
  // "submitted" condition is legacy and should be removed later
  condition: z.enum(["clicked", "submitted", "skipped"]).optional(),
  value: z.undefined()
});
const ZSurveyRatingLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["equals", "notEquals", "lessThan", "lessEqual", "greaterThan", "greaterEqual", "submitted", "skipped"]).optional(),
  value: z.union([z.string(), z.number()]).optional()
});
const ZSurveyPictureSelectionLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["submitted", "skipped"]).optional(),
  value: z.undefined()
});
const ZSurveyCalLogic = ZSurveyLogicBase.extend({
  condition: z.enum(["booked", "skipped"]).optional(),
  value: z.undefined()
});
const ZSurveyLogic = z.union([ZSurveyOpenTextLogic, ZSurveyConsentLogic, ZSurveyMultipleChoiceSingleLogic, ZSurveyMultipleChoiceMultiLogic, ZSurveyNPSLogic, ZSurveyCTALogic, ZSurveyRatingLogic, ZSurveyPictureSelectionLogic, ZSurveyFileUploadLogic, ZSurveyCalLogic]);
const ZSurveyQuestionBase = z.object({
  id: z.string(),
  type: z.string(),
  headline: z.string(),
  subheader: z.string().optional(),
  imageUrl: z.string().optional(),
  required: z.boolean(),
  buttonLabel: z.string().optional(),
  backButtonLabel: z.string().optional(),
  scale: z.enum(["number", "smiley", "star"]).optional(),
  range: z.union([z.literal(5), z.literal(3), z.literal(4), z.literal(7), z.literal(10)]).optional(),
  logic: z.array(ZSurveyLogic).optional(),
  isDraft: z.boolean().optional()
});
const ZSurveyOpenTextQuestionInputType = z.enum(["text", "email", "url", "number", "phone"]);
const ZSurveyOpenTextQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "openText"
    /* OpenText */
  ),
  placeholder: z.string().optional(),
  longAnswer: z.boolean().optional(),
  logic: z.array(ZSurveyOpenTextLogic).optional(),
  inputType: ZSurveyOpenTextQuestionInputType.optional().default("text")
});
const ZSurveyConsentQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "consent"
    /* Consent */
  ),
  html: z.string().optional(),
  label: z.string(),
  dismissButtonLabel: z.string().optional(),
  placeholder: z.string().optional(),
  logic: z.array(ZSurveyConsentLogic).optional()
});
const ZSurveyMultipleChoiceSingleQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "multipleChoiceSingle"
    /* MultipleChoiceSingle */
  ),
  choices: z.array(ZSurveyChoice),
  logic: z.array(ZSurveyMultipleChoiceSingleLogic).optional(),
  shuffleOption: z.enum(["none", "all", "exceptLast"]).optional()
});
const ZSurveyMultipleChoiceMultiQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "multipleChoiceMulti"
    /* MultipleChoiceMulti */
  ),
  choices: z.array(ZSurveyChoice),
  logic: z.array(ZSurveyMultipleChoiceMultiLogic).optional(),
  shuffleOption: z.enum(["none", "all", "exceptLast"]).optional()
});
const ZSurveyNPSQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "nps"
    /* NPS */
  ),
  lowerLabel: z.string(),
  upperLabel: z.string(),
  logic: z.array(ZSurveyNPSLogic).optional()
});
const ZSurveyCTAQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "cta"
    /* CTA */
  ),
  html: z.string().optional(),
  buttonUrl: z.string().optional(),
  buttonExternal: z.boolean(),
  dismissButtonLabel: z.string().optional(),
  logic: z.array(ZSurveyCTALogic).optional()
});
const ZSurveyRatingQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "rating"
    /* Rating */
  ),
  scale: z.enum(["number", "smiley", "star"]),
  range: z.union([z.literal(5), z.literal(3), z.literal(4), z.literal(7), z.literal(10)]),
  lowerLabel: z.string(),
  upperLabel: z.string(),
  logic: z.array(ZSurveyRatingLogic).optional()
});
const ZSurveyDateQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "date"
    /* Date */
  ),
  html: z.string().optional(),
  format: z.enum(["M-d-y", "d-M-y", "y-M-d"])
});
const ZSurveyPictureSelectionQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "pictureSelection"
    /* PictureSelection */
  ),
  allowMulti: z.boolean().optional().default(false),
  choices: z.array(ZSurveyPictureChoice),
  logic: z.array(ZSurveyPictureSelectionLogic).optional()
});
const ZSurveyFileUploadQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "fileUpload"
    /* FileUpload */
  ),
  allowMultipleFiles: z.boolean(),
  maxSizeInMB: z.number().optional(),
  allowedFileExtensions: z.array(ZAllowedFileExtension).optional(),
  logic: z.array(ZSurveyFileUploadLogic).optional()
});
const ZSurveyCalQuestion = ZSurveyQuestionBase.extend({
  type: z.literal(
    "cal"
    /* Cal */
  ),
  calUserName: z.string(),
  logic: z.array(ZSurveyCalLogic).optional()
});
const ZSurveyQuestion = z.union([ZSurveyOpenTextQuestion, ZSurveyConsentQuestion, ZSurveyMultipleChoiceSingleQuestion, ZSurveyMultipleChoiceMultiQuestion, ZSurveyNPSQuestion, ZSurveyCTAQuestion, ZSurveyRatingQuestion, ZSurveyPictureSelectionQuestion, ZSurveyDateQuestion, ZSurveyFileUploadQuestion, ZSurveyCalQuestion]);
const ZSurveyQuestions = z.array(ZSurveyQuestion);
const ZSurveyAttributeFilter = z.object({
  attributeClassId: z.string().cuid2(),
  condition: z.enum(["equals", "notEquals"]),
  value: z.string()
});
const ZSurveyDisplayOption = z.enum(["displayOnce", "displayMultiple", "respondMultiple"]);
const ZSurveyType = z.enum(["web", "email", "link", "mobile"]);
const ZSurveyStatus = z.enum(["draft", "inProgress", "paused", "completed"]);
z.object({
  id: z.string().cuid2(),
  createdAt: z.date(),
  updatedAt: z.date(),
  name: z.string(),
  type: ZSurveyType,
  environmentId: z.string(),
  status: ZSurveyStatus,
  attributeFilters: z.array(ZSurveyAttributeFilter),
  displayOption: ZSurveyDisplayOption,
  autoClose: z.number().nullable(),
  triggers: z.array(z.string()),
  redirectUrl: z.string().url().nullable(),
  recontactDays: z.number().nullable(),
  welcomeCard: ZSurveyWelcomeCard,
  questions: ZSurveyQuestions,
  thankYouCard: ZSurveyThankYouCard,
  hiddenFields: ZSurveyHiddenFields,
  delay: z.number(),
  autoComplete: z.number().nullable(),
  closeOnDate: z.date().nullable(),
  productOverwrites: ZSurveyProductOverwrites.nullable(),
  styling: ZSurveyStyling.nullable(),
  surveyClosedMessage: ZSurveyClosedMessage.nullable(),
  singleUse: ZSurveySingleUse.nullable(),
  verifyEmail: ZSurveyVerifyEmail.nullable(),
  pin: z.string().nullable().optional(),
  resultShareKey: z.string().nullable()
});
z.object({
  name: z.string(),
  type: ZSurveyType.optional(),
  status: ZSurveyStatus.optional(),
  displayOption: ZSurveyDisplayOption.optional(),
  autoClose: z.number().optional(),
  redirectUrl: z.string().url().optional(),
  recontactDays: z.number().optional(),
  welcomeCard: ZSurveyWelcomeCard.optional(),
  questions: ZSurveyQuestions.optional(),
  thankYouCard: ZSurveyThankYouCard.optional(),
  hiddenFields: ZSurveyHiddenFields,
  delay: z.number().optional(),
  autoComplete: z.number().optional(),
  closeOnDate: z.date().optional(),
  surveyClosedMessage: ZSurveyClosedMessage.optional(),
  verifyEmail: ZSurveyVerifyEmail.optional(),
  attributeFilters: z.array(ZSurveyAttributeFilter).optional(),
  triggers: z.array(z.string()).optional()
});
z.union([z.literal("fileUpload"), z.literal("openText"), z.literal("multipleChoiceSingle"), z.literal("multipleChoiceMulti"), z.literal("nps"), z.literal("cta"), z.literal("rating"), z.literal("consent"), z.literal("pictureSelection"), z.literal("cal"), z.literal("date")]);
function QuestionConditional({
  question,
  value,
  onChange,
  onSubmit,
  onBack,
  isFirstQuestion,
  isLastQuestion,
  autoFocus = true,
  ttc,
  setTtc,
  surveyId,
  onFileUpload
}) {
  return question.type === TSurveyQuestionType.OpenText ? u$1(OpenTextQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    autoFocus,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.MultipleChoiceSingle ? u$1(MultipleChoiceSingleQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.MultipleChoiceMulti ? u$1(MultipleChoiceMultiQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.NPS ? u$1(NPSQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.CTA ? u$1(CTAQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.Rating ? u$1(RatingQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.Consent ? u$1(ConsentQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.Date ? u$1(DateQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.PictureSelection ? u$1(PictureSelectionQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.FileUpload ? u$1(FileUploadQuestion, {
    surveyId,
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    onFileUpload,
    ttc,
    setTtc
  }) : question.type === TSurveyQuestionType.Cal ? u$1(CalQuestion, {
    question,
    value,
    onChange,
    onSubmit,
    onBack,
    isFirstQuestion,
    isLastQuestion,
    ttc,
    setTtc
  }) : null;
}
const REDIRECT_TIMEOUT = 5;
function RedirectCountDown({
  redirectUrl,
  isRedirectDisabled
}) {
  const [timeRemaining, setTimeRemaining] = h(REDIRECT_TIMEOUT);
  p(() => {
    let interval;
    if (redirectUrl) {
      const interval2 = setInterval(() => {
        setTimeRemaining((prevTime) => {
          if (prevTime <= 0) {
            clearInterval(interval2);
            if (!isRedirectDisabled) {
              window.location.href = redirectUrl;
            }
            return 0;
          }
          return prevTime - 1;
        });
      }, 1e3);
    }
    return () => clearInterval(interval);
  }, [redirectUrl, isRedirectDisabled]);
  if (!redirectUrl)
    return null;
  return u$1("div", {
    children: u$1("div", {
      className: "bg-accent-bg text-subheading mt-10 rounded-md p-2 text-sm",
      children: [u$1("span", {
        children: "You're redirected in "
      }), u$1("span", {
        children: timeRemaining
      })]
    })
  });
}
function ThankYouCard({
  headline,
  subheader,
  redirectUrl,
  isRedirectDisabled
}) {
  return u$1("div", {
    className: "text-center",
    children: [u$1("div", {
      className: "text-brand flex items-center justify-center",
      children: u$1("svg", {
        xmlns: "http://www.w3.org/2000/svg",
        fill: "none",
        viewBox: "0 0 24 24",
        "stroke-width": "1.5",
        stroke: "currentColor",
        class: "h-24 w-24",
        children: u$1("path", {
          "stroke-linecap": "round",
          "stroke-linejoin": "round",
          d: "M9 12.75L11.25 15 15 9.75M21 12a9 9 0 11-18 0 9 9 0 0118 0z"
        })
      })
    }), u$1("span", {
      className: "bg-shadow mb-[10px] inline-block h-1 w-16 rounded-[100%]"
    }), u$1("div", {
      children: [u$1(Headline, {
        alignTextCenter: true,
        headline,
        questionId: "thankYouCard"
      }), u$1(Subheader, {
        subheader,
        questionId: "thankYouCard"
      }), u$1(RedirectCountDown, {
        redirectUrl,
        isRedirectDisabled
      })]
    })]
  });
}
const TimerIcon = () => {
  return u$1("div", {
    className: "mr-1",
    children: u$1("svg", {
      xmlns: "http://www.w3.org/2000/svg",
      width: "16",
      height: "16",
      fill: "currentColor",
      class: "bi bi-stopwatch",
      viewBox: "0 0 16 16",
      children: [u$1("path", {
        d: "M8.5 5.6a.5.5 0 1 0-1 0v2.9h-3a.5.5 0 0 0 0 1H8a.5.5 0 0 0 .5-.5V5.6z"
      }), u$1("path", {
        d: "M6.5 1A.5.5 0 0 1 7 .5h2a.5.5 0 0 1 0 1v.57c1.36.196 2.594.78 3.584 1.64a.715.715 0 0 1 .012-.013l.354-.354-.354-.353a.5.5 0 0 1 .707-.708l1.414 1.415a.5.5 0 1 1-.707.707l-.353-.354-.354.354a.512.512 0 0 1-.013.012A7 7 0 1 1 7 2.071V1.5a.5.5 0 0 1-.5-.5zM8 3a6 6 0 1 0 .001 12A6 6 0 0 0 8 3z"
      })]
    })
  });
};
const UsersIcon = () => {
  return u$1("div", {
    className: "mr-1",
    children: u$1("svg", {
      xmlns: "http://www.w3.org/2000/svg",
      fill: "none",
      viewBox: "0 0 24 24",
      "stroke-width": "1.5",
      stroke: "currentColor",
      class: "h-4 w-4",
      children: u$1("path", {
        "stroke-linecap": "round",
        "stroke-linejoin": "round",
        d: "M15 19.128a9.38 9.38 0 002.625.372 9.337 9.337 0 004.121-.952 4.125 4.125 0 00-7.533-2.493M15 19.128v-.003c0-1.113-.285-2.16-.786-3.07M15 19.128v.106A12.318 12.318 0 018.624 21c-2.331 0-4.512-.645-6.374-1.766l-.001-.109a6.375 6.375 0 0111.964-3.07M12 6.375a3.375 3.375 0 11-6.75 0 3.375 3.375 0 016.75 0zm8.25 2.25a2.625 2.625 0 11-5.25 0 2.625 2.625 0 015.25 0z"
      })
    })
  });
};
function WelcomeCard({
  headline,
  html,
  fileUrl,
  buttonLabel,
  onSubmit,
  survey,
  responseCount
}) {
  const calculateTimeToComplete = () => {
    let idx = calculateElementIdx(survey, 0);
    if (idx === 0.5) {
      idx = 1;
    }
    const timeInSeconds = survey.questions.length / idx * 15;
    if (timeInSeconds > 360) {
      return "6+ minutes";
    }
    const minutes = Math.floor(timeInSeconds / 60);
    const remainingSeconds = timeInSeconds % 60;
    if (remainingSeconds > 0) {
      if (minutes === 0) {
        return "less than 1 minute";
      } else {
        return `less than ${minutes + 1} minutes`;
      }
    }
    return `${minutes} minutes`;
  };
  const timeToFinish = survey.welcomeCard.timeToFinish;
  const showResponseCount = survey.welcomeCard.showResponseCount;
  return u$1("div", {
    children: [fileUrl && /* eslint-disable-next-line @next/next/no-img-element */
    u$1("img", {
      src: fileUrl,
      className: "mb-8 max-h-96 w-1/3 rounded-lg object-contain",
      alt: "Company Logo"
    }), u$1(Headline, {
      headline,
      questionId: "welcomeCard"
    }), u$1(HtmlBody, {
      htmlString: html,
      questionId: "welcomeCard"
    }), u$1("div", {
      className: "mt-10 flex w-full justify-between",
      children: u$1("div", {
        className: "flex w-full justify-start gap-4",
        children: [u$1(SubmitButton, {
          buttonLabel,
          isLastQuestion: false,
          focus: true,
          onClick: () => {
            onSubmit({
              ["welcomeCard"]: "clicked"
            }, {});
          },
          type: "button"
        }), u$1("div", {
          className: "text-subheading flex items-center text-xs",
          children: "Press Enter ↵"
        })]
      })
    }), timeToFinish && !showResponseCount ? u$1("div", {
      className: "item-center mt-4 flex text-slate-500",
      children: [u$1(TimerIcon, {}), u$1("p", {
        className: "pt-1 text-xs",
        children: u$1("span", {
          children: [" Takes ", calculateTimeToComplete(), " "]
        })
      })]
    }) : showResponseCount && !timeToFinish && responseCount && responseCount > 3 ? u$1("div", {
      className: "item-center mt-4 flex text-slate-500",
      children: [u$1(UsersIcon, {}), u$1("p", {
        className: "pt-1 text-xs",
        children: u$1("span", {
          children: `${responseCount} people responded`
        })
      })]
    }) : timeToFinish && showResponseCount ? u$1("div", {
      className: "item-center mt-4 flex text-slate-500",
      children: [u$1(TimerIcon, {}), u$1("p", {
        className: "pt-1 text-xs",
        children: [u$1("span", {
          children: [" Takes ", calculateTimeToComplete(), " "]
        }), u$1("span", {
          children: responseCount && responseCount > 3 ? `⋅ ${responseCount} people responded` : ""
        })]
      })]
    }) : null]
  });
}
function Survey({
  survey,
  isBrandingEnabled,
  activeQuestionId,
  onDisplay = () => {
  },
  onActiveQuestionChange = () => {
  },
  onResponse = () => {
  },
  onClose = () => {
  },
  onFinished = () => {
  },
  isRedirectDisabled = false,
  prefillResponseData,
  onFileUpload,
  responseCount
}) {
  var _a;
  const [questionId, setQuestionId] = h(activeQuestionId || (survey.welcomeCard.enabled ? "start" : (_a = survey == null ? void 0 : survey.questions[0]) == null ? void 0 : _a.id));
  const [loadingElement, setLoadingElement] = h(false);
  const [history, setHistory] = h([]);
  const [responseData, setResponseData] = h({});
  const currentQuestionIndex = survey.questions.findIndex((q2) => q2.id === questionId);
  const currentQuestion = survey.questions[currentQuestionIndex];
  const contentRef = _(null);
  const [ttc, setTtc] = h({});
  p(() => {
    var _a2, _b;
    if (activeQuestionId === "hidden")
      return;
    if (activeQuestionId === "start" && !survey.welcomeCard.enabled) {
      setQuestionId((_a2 = survey == null ? void 0 : survey.questions[0]) == null ? void 0 : _a2.id);
      return;
    }
    setQuestionId(activeQuestionId || (survey.welcomeCard.enabled ? "start" : (_b = survey == null ? void 0 : survey.questions[0]) == null ? void 0 : _b.id));
  }, [activeQuestionId, survey.questions, survey.welcomeCard.enabled]);
  p(() => {
    if (contentRef.current) {
      contentRef.current.scrollTop = 0;
    }
  }, [questionId]);
  p(() => {
    onDisplay();
    if (prefillResponseData) {
      onSubmit(prefillResponseData, {}, true);
    }
  }, []);
  let currIdx = currentQuestionIndex;
  let currQues = currentQuestion;
  function getNextQuestionId(data, isFromPrefilling = false) {
    var _a2, _b;
    const questions = survey.questions;
    const responseValue = data[questionId];
    if (questionId === "start") {
      if (!isFromPrefilling) {
        return ((_a2 = questions[0]) == null ? void 0 : _a2.id) || "end";
      } else {
        currIdx = 0;
        currQues = questions[0];
      }
    }
    if (currIdx === -1)
      throw new Error("Question not found");
    if ((currQues == null ? void 0 : currQues.logic) && (currQues == null ? void 0 : currQues.logic.length) > 0) {
      for (let logic of currQues.logic) {
        if (!logic.destination)
          continue;
        if (evaluateCondition(logic, responseValue)) {
          return logic.destination;
        }
      }
    }
    return ((_b = questions[currIdx + 1]) == null ? void 0 : _b.id) || "end";
  }
  const onChange = (responseDataUpdate) => {
    const updatedResponseData = {
      ...responseData,
      ...responseDataUpdate
    };
    setResponseData(updatedResponseData);
  };
  const onSubmit = (responseData2, ttc2, isFromPrefilling = false) => {
    const questionId2 = Object.keys(responseData2)[0];
    setLoadingElement(true);
    const nextQuestionId = getNextQuestionId(responseData2, isFromPrefilling);
    const finished = nextQuestionId === "end";
    onResponse({
      data: responseData2,
      ttc: ttc2,
      finished
    });
    if (finished) {
      onFinished();
    }
    setQuestionId(nextQuestionId);
    setHistory([...history, questionId2]);
    setLoadingElement(false);
    onActiveQuestionChange(nextQuestionId);
  };
  const onBack = () => {
    var _a2;
    let prevQuestionId;
    if ((history == null ? void 0 : history.length) > 0) {
      const newHistory = [...history];
      prevQuestionId = newHistory.pop();
      if (prefillResponseData && prevQuestionId === survey.questions[0].id)
        return;
      setHistory(newHistory);
    } else {
      prevQuestionId = (_a2 = survey.questions[currIdx - 1]) == null ? void 0 : _a2.id;
    }
    if (!prevQuestionId)
      throw new Error("Question not found");
    setQuestionId(prevQuestionId);
    onActiveQuestionChange(prevQuestionId);
  };
  function getCardContent() {
    var _a2;
    if (questionId === "start" && survey.welcomeCard.enabled) {
      return u$1(WelcomeCard, {
        headline: survey.welcomeCard.headline,
        html: survey.welcomeCard.html,
        fileUrl: survey.welcomeCard.fileUrl,
        buttonLabel: survey.welcomeCard.buttonLabel,
        onSubmit,
        survey,
        responseCount
      });
    } else if (questionId === "end" && survey.thankYouCard.enabled) {
      return u$1(ThankYouCard, {
        headline: survey.thankYouCard.headline,
        subheader: survey.thankYouCard.subheader,
        redirectUrl: survey.redirectUrl,
        isRedirectDisabled
      });
    } else {
      const currQues2 = survey.questions.find((q2) => q2.id === questionId);
      return currQues2 && u$1(QuestionConditional, {
        surveyId: survey.id,
        question: currQues2,
        value: responseData[currQues2.id],
        onChange,
        onSubmit,
        onBack,
        ttc,
        setTtc,
        onFileUpload,
        isFirstQuestion: history && prefillResponseData ? history[history.length - 1] === survey.questions[0].id : currQues2.id === ((_a2 = survey == null ? void 0 : survey.questions[0]) == null ? void 0 : _a2.id),
        isLastQuestion: currQues2.id === survey.questions[survey.questions.length - 1].id
      });
    }
  }
  return u$1(g$2, {
    children: u$1(AutoCloseWrapper, {
      survey,
      onClose,
      children: u$1("div", {
        className: "no-scrollbar flex h-full w-full flex-col justify-between rounded-lg bg-[--fb-survey-background-color] px-6 pb-3 pt-6",
        children: [u$1("div", {
          ref: contentRef,
          className: cn(loadingElement ? "animate-pulse opacity-60" : "", "my-auto"),
          children: survey.questions.length === 0 && !survey.welcomeCard.enabled && !survey.thankYouCard.enabled ? (
            // Handle the case when there are no questions and both welcome and thank you cards are disabled
            u$1("div", {
              children: "No questions available."
            })
          ) : getCardContent()
        }), u$1("div", {
          className: "mt-8",
          children: [isBrandingEnabled && u$1(FormbricksBranding, {}), u$1(ProgressBar, {
            survey,
            questionId
          })]
        })]
      })
    })
  });
}
function Modal({
  children,
  isOpen,
  placement,
  clickOutside,
  darkOverlay,
  highlightBorderColor,
  onClose
}) {
  const [show, setShow] = h(false);
  const isCenter = placement === "center";
  const modalRef = _(null);
  p(() => {
    setShow(isOpen);
  }, [isOpen]);
  p(() => {
    if (!isCenter)
      return;
    function handleClickOutside(e2) {
      if (clickOutside && show && modalRef.current && !modalRef.current.contains(e2.target)) {
        onClose();
      }
    }
    document.addEventListener("mousedown", handleClickOutside);
    return () => {
      document.removeEventListener("mousedown", handleClickOutside);
    };
  }, [show, clickOutside, onClose, isCenter]);
  const getPlacementStyle = (placement2) => {
    switch (placement2) {
      case "bottomRight":
        return "sm:bottom-3 sm:right-3";
      case "topRight":
        return "sm:top-3 sm:right-3 sm:bottom-3";
      case "topLeft":
        return "sm:top-3 sm:left-3 sm:bottom-3";
      case "bottomLeft":
        return "sm:bottom-3 sm:left-3";
      case "center":
        return "sm:top-1/2 sm:left-1/2 sm:transform sm:-translate-x-1/2 sm:-translate-y-1/2";
      default:
        return "sm:bottom-3 sm:right-3";
    }
  };
  const highlightBorderColorStyle = F$1(() => {
    if (!highlightBorderColor)
      return {
        overflow: "visible"
      };
    return {
      borderRadius: "8px",
      border: "2px solid",
      borderColor: highlightBorderColor
    };
  }, [highlightBorderColor]);
  if (!show)
    return null;
  return u$1("div", {
    "aria-live": "assertive",
    className: cn(isCenter ? "pointer-events-auto" : "pointer-events-none", "z-999999 fixed inset-0 flex items-end"),
    children: u$1("div", {
      className: cn("relative h-full w-full", isCenter ? darkOverlay ? "bg-gray-700/80" : "bg-white/50" : "bg-none transition-all duration-500 ease-in-out"),
      children: u$1("div", {
        ref: modalRef,
        className: cn(getPlacementStyle(placement), show ? "opacity-100" : "opacity-0", "border-border pointer-events-auto absolute bottom-0 h-fit w-full overflow-visible rounded-lg border bg-white shadow-lg transition-all duration-500 ease-in-out sm:m-4 sm:max-w-sm"),
        children: [!isCenter && u$1("div", {
          class: "absolute right-0 top-0 block pr-2 pt-2",
          children: u$1("button", {
            type: "button",
            onClick: onClose,
            class: "text-close-button hover:text-close-button-focus focus:ring-close-button-focus relative h-5 w-5 rounded-md focus:outline-none focus:ring-2 focus:ring-offset-2",
            children: [u$1("span", {
              class: "sr-only",
              children: "Close survey"
            }), u$1("svg", {
              class: "h-4 w-4",
              fill: "none",
              viewBox: "0 0 24 24",
              "stroke-width": "2",
              stroke: "currentColor",
              "aria-hidden": "true",
              children: u$1("path", {
                "stroke-linecap": "round",
                "stroke-linejoin": "round",
                d: "M4 4L20 20M4 20L20 4"
              })
            })]
          })
        }), u$1("div", {
          style: highlightBorderColorStyle,
          children
        })]
      })
    })
  });
}
function SurveyModal({
  survey,
  isBrandingEnabled,
  activeQuestionId,
  placement,
  clickOutside,
  darkOverlay,
  highlightBorderColor,
  onDisplay = () => {
  },
  onActiveQuestionChange = () => {
  },
  onResponse = () => {
  },
  onClose = () => {
  },
  onFinished = () => {
  },
  onFileUpload,
  isRedirectDisabled = false,
  responseCount
}) {
  const [isOpen, setIsOpen] = h(true);
  const close = () => {
    setIsOpen(false);
    setTimeout(() => {
      onClose();
    }, 1e3);
  };
  return u$1("div", {
    id: "fbjs",
    className: "formbricks-form",
    children: u$1(Modal, {
      placement,
      clickOutside,
      darkOverlay,
      highlightBorderColor,
      isOpen,
      onClose: close,
      children: u$1(Survey, {
        survey,
        isBrandingEnabled,
        activeQuestionId,
        onDisplay,
        onActiveQuestionChange,
        onResponse,
        onClose: close,
        onFinished: () => {
          onFinished();
          setTimeout(() => {
            if (!survey.redirectUrl) {
              close();
            }
          }, 4e3);
        },
        onFileUpload,
        isRedirectDisabled,
        responseCount
      })
    })
  });
}
const global = "*,:before,:after{--tw-border-spacing-x: 0;--tw-border-spacing-y: 0;--tw-translate-x: 0;--tw-translate-y: 0;--tw-rotate: 0;--tw-skew-x: 0;--tw-skew-y: 0;--tw-scale-x: 1;--tw-scale-y: 1;--tw-pan-x: ;--tw-pan-y: ;--tw-pinch-zoom: ;--tw-scroll-snap-strictness: proximity;--tw-gradient-from-position: ;--tw-gradient-via-position: ;--tw-gradient-to-position: ;--tw-ordinal: ;--tw-slashed-zero: ;--tw-numeric-figure: ;--tw-numeric-spacing: ;--tw-numeric-fraction: ;--tw-ring-inset: ;--tw-ring-offset-width: 0px;--tw-ring-offset-color: #fff;--tw-ring-color: rgb(59 130 246 / .5);--tw-ring-offset-shadow: 0 0 #0000;--tw-ring-shadow: 0 0 #0000;--tw-shadow: 0 0 #0000;--tw-shadow-colored: 0 0 #0000;--tw-blur: ;--tw-brightness: ;--tw-contrast: ;--tw-grayscale: ;--tw-hue-rotate: ;--tw-invert: ;--tw-saturate: ;--tw-sepia: ;--tw-drop-shadow: ;--tw-backdrop-blur: ;--tw-backdrop-brightness: ;--tw-backdrop-contrast: ;--tw-backdrop-grayscale: ;--tw-backdrop-hue-rotate: ;--tw-backdrop-invert: ;--tw-backdrop-opacity: ;--tw-backdrop-saturate: ;--tw-backdrop-sepia: }::backdrop{--tw-border-spacing-x: 0;--tw-border-spacing-y: 0;--tw-translate-x: 0;--tw-translate-y: 0;--tw-rotate: 0;--tw-skew-x: 0;--tw-skew-y: 0;--tw-scale-x: 1;--tw-scale-y: 1;--tw-pan-x: ;--tw-pan-y: ;--tw-pinch-zoom: ;--tw-scroll-snap-strictness: proximity;--tw-gradient-from-position: ;--tw-gradient-via-position: ;--tw-gradient-to-position: ;--tw-ordinal: ;--tw-slashed-zero: ;--tw-numeric-figure: ;--tw-numeric-spacing: ;--tw-numeric-fraction: ;--tw-ring-inset: ;--tw-ring-offset-width: 0px;--tw-ring-offset-color: #fff;--tw-ring-color: rgb(59 130 246 / .5);--tw-ring-offset-shadow: 0 0 #0000;--tw-ring-shadow: 0 0 #0000;--tw-shadow: 0 0 #0000;--tw-shadow-colored: 0 0 #0000;--tw-blur: ;--tw-brightness: ;--tw-contrast: ;--tw-grayscale: ;--tw-hue-rotate: ;--tw-invert: ;--tw-saturate: ;--tw-sepia: ;--tw-drop-shadow: ;--tw-backdrop-blur: ;--tw-backdrop-brightness: ;--tw-backdrop-contrast: ;--tw-backdrop-grayscale: ;--tw-backdrop-hue-rotate: ;--tw-backdrop-invert: ;--tw-backdrop-opacity: ;--tw-backdrop-saturate: ;--tw-backdrop-sepia: }.container{width:100%}@media (min-width: 640px){.container{max-width:640px}}@media (min-width: 768px){.container{max-width:768px}}@media (min-width: 1024px){.container{max-width:1024px}}@media (min-width: 1280px){.container{max-width:1280px}}@media (min-width: 1536px){.container{max-width:1536px}}#fbjs :is(.sr-only){position:absolute;width:1px;height:1px;padding:0;margin:-1px;overflow:hidden;clip:rect(0,0,0,0);white-space:nowrap;border-width:0}#fbjs :is(.pointer-events-none){pointer-events:none}#fbjs :is(.pointer-events-auto){pointer-events:auto}#fbjs :is(.visible){visibility:visible}#fbjs :is(.fixed){position:fixed}#fbjs :is(.absolute){position:absolute}#fbjs :is(.relative){position:relative}#fbjs :is(.inset-0){top:0;right:0;bottom:0;left:0}#fbjs :is(.bottom-0){bottom:0}#fbjs :is(.left-0){left:0}#fbjs :is(.right-0){right:0}#fbjs :is(.right-2){right:.5em}#fbjs :is(.top-0){top:0}#fbjs :is(.top-2){top:.5em}#fbjs :is(.z-10){z-index:10}#fbjs :is(.z-20){z-index:20}#fbjs :is(.z-999999){z-index:999999}#fbjs :is(.m-2){margin:.5em}#fbjs :is(.my-4){margin-top:1em;margin-bottom:1em}#fbjs :is(.my-auto){margin-top:auto;margin-bottom:auto}#fbjs :is(.mb-1){margin-bottom:.25em}#fbjs :is(.mb-1\\.5){margin-bottom:.375em}#fbjs :is(.mb-4){margin-bottom:1em}#fbjs :is(.mb-5){margin-bottom:1.25em}#fbjs :is(.mb-8){margin-bottom:2em}#fbjs :is(.mb-\\[10px\\]){margin-bottom:10px}#fbjs :is(.ml-2){margin-left:.5em}#fbjs :is(.ml-3){margin-left:.75em}#fbjs :is(.mr-1){margin-right:.25em}#fbjs :is(.mr-4){margin-right:1em}#fbjs :is(.mt-1){margin-top:.25em}#fbjs :is(.mt-10){margin-top:2.5em}#fbjs :is(.mt-2){margin-top:.5em}#fbjs :is(.mt-3){margin-top:.75em}#fbjs :is(.mt-4){margin-top:1em}#fbjs :is(.mt-6){margin-top:1.5em}#fbjs :is(.mt-8){margin-top:2em}#fbjs :is(.box-border){box-sizing:border-box}#fbjs :is(.block){display:block}#fbjs :is(.inline-block){display:inline-block}#fbjs :is(.inline){display:inline}#fbjs :is(.flex){display:flex}#fbjs :is(.grid){display:grid}#fbjs :is(.hidden){display:none}#fbjs :is(.h-1){height:.25em}#fbjs :is(.h-10){height:2.5em}#fbjs :is(.h-12){height:3em}#fbjs :is(.h-14){height:3.5em}#fbjs :is(.h-2){height:.5em}#fbjs :is(.h-24){height:6em}#fbjs :is(.h-28){height:7em}#fbjs :is(.h-4){height:1em}#fbjs :is(.h-5){height:1.25em}#fbjs :is(.h-6){height:1.5em}#fbjs :is(.h-9){height:2.25em}#fbjs :is(.h-96){height:24em}#fbjs :is(.h-fit){height:-moz-fit-content;height:fit-content}#fbjs :is(.h-full){height:100%}#fbjs :is(.max-h-96){max-height:24em}#fbjs :is(.max-h-\\[42vh\\]){max-height:42vh}#fbjs :is(.max-h-full){max-height:100%}#fbjs :is(.w-1\\/2){width:50%}#fbjs :is(.w-1\\/3){width:33.333333%}#fbjs :is(.w-14){width:3.5em}#fbjs :is(.w-16){width:4em}#fbjs :is(.w-24){width:6em}#fbjs :is(.w-4){width:1em}#fbjs :is(.w-5){width:1.25em}#fbjs :is(.w-6){width:1.5em}#fbjs :is(.w-80){width:20em}#fbjs :is(.w-full){width:100%}#fbjs :is(.flex-1){flex:1 1 0%}@keyframes pulse{50%{opacity:.5}}#fbjs :is(.animate-pulse){animation:pulse 2s cubic-bezier(.4,0,.6,1) infinite}@keyframes spin{to{transform:rotate(360deg)}}#fbjs :is(.animate-spin){animation:spin 1s linear infinite}#fbjs :is(.cursor-pointer){cursor:pointer}#fbjs :is(.appearance-none){-webkit-appearance:none;-moz-appearance:none;appearance:none}#fbjs :is(.grid-cols-2){grid-template-columns:repeat(2,minmax(0,1fr))}#fbjs :is(.flex-col){flex-direction:column}#fbjs :is(.items-end){align-items:flex-end}#fbjs :is(.items-center){align-items:center}#fbjs :is(.justify-start){justify-content:flex-start}#fbjs :is(.justify-end){justify-content:flex-end}#fbjs :is(.justify-center){justify-content:center}#fbjs :is(.justify-between){justify-content:space-between}#fbjs :is(.gap-2){gap:.5em}#fbjs :is(.gap-4){gap:1em}#fbjs :is(.gap-x-5){-moz-column-gap:1.25em;column-gap:1.25em}#fbjs :is(.gap-y-4){row-gap:1em}#fbjs :is(.space-y-2>:not([hidden])~:not([hidden])){--tw-space-y-reverse: 0;margin-top:calc(.5em * calc(1 - var(--tw-space-y-reverse)));margin-bottom:calc(.5em * var(--tw-space-y-reverse))}#fbjs :is(.self-start){align-self:flex-start}#fbjs :is(.overflow-auto){overflow:auto}#fbjs :is(.overflow-hidden){overflow:hidden}#fbjs :is(.overflow-visible){overflow:visible}#fbjs :is(.overflow-y-auto){overflow-y:auto}#fbjs :is(.rounded){border-radius:.25em}#fbjs :is(.rounded-\\[100\\%\\]){border-radius:100%}#fbjs :is(.rounded-full){border-radius:9999px}#fbjs :is(.rounded-lg){border-radius:.5em}#fbjs :is(.rounded-md){border-radius:.375em}#fbjs :is(.rounded-xl){border-radius:.75em}#fbjs :is(.rounded-l-md){border-top-left-radius:.375em;border-bottom-left-radius:.375em}#fbjs :is(.rounded-r-md){border-top-right-radius:.375em;border-bottom-right-radius:.375em}#fbjs :is(.border){border-width:1px}#fbjs :is(.border-2){border-width:2px}#fbjs :is(.border-4){border-width:4px}#fbjs :is(.border-b-2){border-bottom-width:2px}#fbjs :is(.border-dashed){border-style:dashed}#fbjs :is(.border-\\[\\#e5e7eb\\]){--tw-border-opacity: 1;border-color:rgb(229 231 235 / var(--tw-border-opacity))}#fbjs :is(.border-back-button-border){border-color:var(--fb-back-btn-border)}#fbjs :is(.border-border){border-color:var(--fb-border-color)}#fbjs :is(.border-border-highlight){border-color:var(--fb-border-color-highlight)}#fbjs :is(.border-brand){border-color:var(--fb-brand-color)}#fbjs :is(.border-neutral-900){--tw-border-opacity: 1;border-color:rgb(23 23 23 / var(--tw-border-opacity))}#fbjs :is(.border-red-500){--tw-border-opacity: 1;border-color:rgb(239 68 68 / var(--tw-border-opacity))}#fbjs :is(.border-slate-200){--tw-border-opacity: 1;border-color:rgb(226 232 240 / var(--tw-border-opacity))}#fbjs :is(.border-slate-300){--tw-border-opacity: 1;border-color:rgb(203 213 225 / var(--tw-border-opacity))}#fbjs :is(.border-submit-button-border){border-color:var(--fb-submit-btn-border)}#fbjs :is(.\\!bg-slate-900){--tw-bg-opacity: 1 !important;background-color:rgb(15 23 42 / var(--tw-bg-opacity))!important}#fbjs :is(.bg-\\[--fb-survey-background-color\\]){background-color:var(--fb-survey-background-color)}#fbjs :is(.bg-accent-bg){background-color:var(--fb-accent-background-color)}#fbjs :is(.bg-accent-selected-bg){background-color:var(--fb-accent-background-color-selected)}#fbjs :is(.bg-brand){background-color:var(--fb-brand-color)}#fbjs :is(.bg-gray-700\\/80){background-color:#374151cc}#fbjs :is(.bg-shadow){background-color:var(--fb-shadow-color)}#fbjs :is(.bg-slate-100){--tw-bg-opacity: 1;background-color:rgb(241 245 249 / var(--tw-bg-opacity))}#fbjs :is(.bg-slate-200){--tw-bg-opacity: 1;background-color:rgb(226 232 240 / var(--tw-bg-opacity))}#fbjs :is(.bg-slate-50){--tw-bg-opacity: 1;background-color:rgb(248 250 252 / var(--tw-bg-opacity))}#fbjs :is(.bg-survey-bg){background-color:var(--fb-survey-background-color)}#fbjs :is(.bg-white){--tw-bg-opacity: 1;background-color:rgb(255 255 255 / var(--tw-bg-opacity))}#fbjs :is(.bg-white\\/50){background-color:#ffffff80}#fbjs :is(.bg-none){background-image:none}#fbjs :is(.fill-none){fill:none}#fbjs :is(.fill-rating-fill){fill:var(--fb-rating-fill)}#fbjs :is(.stroke-heading){stroke:var(--fb-heading-color)}#fbjs :is(.stroke-rating-selected){stroke:var(--fb-rating-selected)}#fbjs :is(.object-contain){-o-object-fit:contain;object-fit:contain}#fbjs :is(.object-cover){-o-object-fit:cover;object-fit:cover}#fbjs :is(.p-0){padding:0}#fbjs :is(.p-2){padding:.5em}#fbjs :is(.p-3){padding:.75em}#fbjs :is(.p-4){padding:1em}#fbjs :is(.px-1){padding-left:.25em;padding-right:.25em}#fbjs :is(.px-1\\.5){padding-left:.375em;padding-right:.375em}#fbjs :is(.px-3){padding-left:.75em;padding-right:.75em}#fbjs :is(.px-6){padding-left:1.5em;padding-right:1.5em}#fbjs :is(.py-0){padding-top:0;padding-bottom:0}#fbjs :is(.py-0\\.5){padding-top:.125em;padding-bottom:.125em}#fbjs :is(.py-2){padding-top:.5em;padding-bottom:.5em}#fbjs :is(.py-3){padding-top:.75em;padding-bottom:.75em}#fbjs :is(.py-4){padding-top:1em;padding-bottom:1em}#fbjs :is(.py-6){padding-top:1.5em;padding-bottom:1.5em}#fbjs :is(.pb-2){padding-bottom:.5em}#fbjs :is(.pb-3){padding-bottom:.75em}#fbjs :is(.pr-2){padding-right:.5em}#fbjs :is(.pr-2\\.5){padding-right:.625em}#fbjs :is(.pt-1){padding-top:.25em}#fbjs :is(.pt-2){padding-top:.5em}#fbjs :is(.pt-6){padding-top:1.5em}#fbjs :is(.text-left){text-align:left}#fbjs :is(.text-center){text-align:center}#fbjs :is(.text-right){text-align:right}#fbjs :is(.text-base){font-size:1em;line-height:1.5em}#fbjs :is(.text-sm){font-size:.875em;line-height:1.25em}#fbjs :is(.text-xs){font-size:.75em;line-height:1em}#fbjs :is(.font-medium){font-weight:500}#fbjs :is(.font-normal){font-weight:400}#fbjs :is(.font-semibold){font-weight:600}#fbjs :is(.leading-10){line-height:2.5em}#fbjs :is(.leading-4){line-height:1em}#fbjs :is(.leading-6){line-height:1.5em}#fbjs :is(.leading-7){line-height:1.75em}#fbjs :is(.leading-\\[2\\.8rem\\]){line-height:2.8em}#fbjs :is(.\\!text-slate-100){--tw-text-opacity: 1 !important;color:rgb(241 245 249 / var(--tw-text-opacity))!important}#fbjs :is(.text-amber-400){--tw-text-opacity: 1;color:rgb(251 191 36 / var(--tw-text-opacity))}#fbjs :is(.text-brand){color:var(--fb-brand-color)}#fbjs :is(.text-close-button){color:var(--fb-close-btn-color)}#fbjs :is(.text-heading){color:var(--fb-heading-color)}#fbjs :is(.text-info-text){color:var(--fb-info-text-color)}#fbjs :is(.text-on-brand){color:var(--fb-brand-text-color)}#fbjs :is(.text-rating-selected){color:var(--fb-rating-selected)}#fbjs :is(.text-red-500){--tw-text-opacity: 1;color:rgb(239 68 68 / var(--tw-text-opacity))}#fbjs :is(.text-red-600){--tw-text-opacity: 1;color:rgb(220 38 38 / var(--tw-text-opacity))}#fbjs :is(.text-signature){color:var(--fb-signature-text-color)}#fbjs :is(.text-slate-300){--tw-text-opacity: 1;color:rgb(203 213 225 / var(--tw-text-opacity))}#fbjs :is(.text-slate-500){--tw-text-opacity: 1;color:rgb(100 116 139 / var(--tw-text-opacity))}#fbjs :is(.text-slate-600){--tw-text-opacity: 1;color:rgb(71 85 105 / var(--tw-text-opacity))}#fbjs :is(.text-slate-700){--tw-text-opacity: 1;color:rgb(51 65 85 / var(--tw-text-opacity))}#fbjs :is(.text-slate-900){--tw-text-opacity: 1;color:rgb(15 23 42 / var(--tw-text-opacity))}#fbjs :is(.text-subheading){color:var(--fb-subheading-color)}#fbjs :is(.opacity-0){opacity:0}#fbjs :is(.opacity-100){opacity:1}#fbjs :is(.opacity-60){opacity:.6}#fbjs :is(.shadow-lg){--tw-shadow: 0 10px 15px -3px rgb(0 0 0 / .1), 0 4px 6px -4px rgb(0 0 0 / .1);--tw-shadow-colored: 0 10px 15px -3px var(--tw-shadow-color), 0 4px 6px -4px var(--tw-shadow-color);box-shadow:var(--tw-ring-offset-shadow, 0 0 #0000),var(--tw-ring-shadow, 0 0 #0000),var(--tw-shadow)}#fbjs :is(.shadow-md){--tw-shadow: 0 4px 6px -1px rgb(0 0 0 / .1), 0 2px 4px -2px rgb(0 0 0 / .1);--tw-shadow-colored: 0 4px 6px -1px var(--tw-shadow-color), 0 2px 4px -2px var(--tw-shadow-color);box-shadow:var(--tw-ring-offset-shadow, 0 0 #0000),var(--tw-ring-shadow, 0 0 #0000),var(--tw-shadow)}#fbjs :is(.shadow-sm){--tw-shadow: 0 1px 2px 0 rgb(0 0 0 / .05);--tw-shadow-colored: 0 1px 2px 0 var(--tw-shadow-color);box-shadow:var(--tw-ring-offset-shadow, 0 0 #0000),var(--tw-ring-shadow, 0 0 #0000),var(--tw-shadow)}#fbjs :is(.shadow-xl){--tw-shadow: 0 20px 25px -5px rgb(0 0 0 / .1), 0 8px 10px -6px rgb(0 0 0 / .1);--tw-shadow-colored: 0 20px 25px -5px var(--tw-shadow-color), 0 8px 10px -6px var(--tw-shadow-color);box-shadow:var(--tw-ring-offset-shadow, 0 0 #0000),var(--tw-ring-shadow, 0 0 #0000),var(--tw-shadow)}#fbjs :is(.filter){filter:var(--tw-blur) var(--tw-brightness) var(--tw-contrast) var(--tw-grayscale) var(--tw-hue-rotate) var(--tw-invert) var(--tw-saturate) var(--tw-sepia) var(--tw-drop-shadow)}#fbjs :is(.transition-all){transition-property:all;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-duration:.15s}#fbjs :is(.duration-500){transition-duration:.5s}#fbjs :is(.ease-in-out){transition-timing-function:cubic-bezier(.4,0,.2,1)}#fbjs{font-size:16px}#fbjs *{scrollbar-width:thin;scrollbar-color:#e2e8f0}#fbjs *::-webkit-scrollbar{width:8px;border-radius:99px}#fbjs *::-webkit-scrollbar-track{background:#e2e8f0;border-radius:99px}#fbjs *::-webkit-scrollbar-thumb{background-color:#cbd5e1;border:3px solid #cbd5e1;border-radius:99px}.fb-htmlbody{display:block;font-size:.875em;font-weight:400;line-height:1.5em;color:var(--fb-subheading-color)!important}p.fb-editor-paragraph{color:var(--fb-subheading-color)!important}:root{--slate-50: rgb(248 250 252);--slate-100: rgb(241 245 249);--slate-200: rgb(226 232 240);--slate-300: rgb(203 213 225);--slate-400: rgb(148 163 184);--slate-500: rgb(100 116 139);--slate-600: rgb(71 85 105);--slate-700: rgb(51 65 85);--slate-800: rgb(30 41 59);--slate-900: rgb(15 23 42);--gray-100: rgb(243 244 246);--gray-200: rgb(229 231 235);--yellow-300: rgb(253 224 71);--yellow-500: rgb(234 179 8);--fb-brand-color: rgb(255, 255, 255);--fb-brand-text-color: black;--fb-border-color: var(--slate-300);--fb-border-color-highlight: var(--slate-500);--fb-focus-color: var(--slate-500);--fb-heading-color: var(--slate-900);--fb-subheading-color: var(--slate-700);--fb-info-text-color: var(--slate-500);--fb-signature-text-color: var(--slate-400);--fb-survey-background-color: white;--fb-accent-background-color: var(--slate-200);--fb-accent-background-color-selected: var(--slate-100);--fb-placeholder-color: var(--slate-400);--fb-shadow-color: var(--slate-300);--fb-rating-fill: var(--yellow-300);--fb-rating-hover: var(--yellow-500);--fb-back-btn-border: transparent;--fb-submit-btn-border: transparent;--fb-rating-selected: black;--fb-close-btn-color: var(--slate-500);--fb-close-btn-color-hover: var(--slate-700)}#fbjs :is(.placeholder\\:text-placeholder)::-moz-placeholder{color:var(--fb-placeholder-color)}#fbjs :is(.placeholder\\:text-placeholder)::placeholder{color:var(--fb-placeholder-color)}#fbjs :is(.first\\:rounded-l-md:first-child){border-top-left-radius:.375em;border-bottom-left-radius:.375em}#fbjs :is(.last\\:rounded-r-md:last-child){border-top-right-radius:.375em;border-bottom-right-radius:.375em}#fbjs :is(.focus-within\\:border-border-highlight:focus-within){border-color:var(--fb-border-color-highlight)}#fbjs :is(.focus-within\\:bg-accent-bg:focus-within){background-color:var(--fb-accent-background-color)}#fbjs :is(.hover\\:bg-accent-bg:hover){background-color:var(--fb-accent-background-color)}#fbjs :is(.hover\\:bg-slate-100:hover){--tw-bg-opacity: 1;background-color:rgb(241 245 249 / var(--tw-bg-opacity))}#fbjs :is(.hover\\:bg-slate-200:hover){--tw-bg-opacity: 1;background-color:rgb(226 232 240 / var(--tw-bg-opacity))}#fbjs :is(.hover\\:bg-slate-50:hover){--tw-bg-opacity: 1;background-color:rgb(248 250 252 / var(--tw-bg-opacity))}#fbjs :is(.hover\\:text-amber-400:hover){--tw-text-opacity: 1;color:rgb(251 191 36 / var(--tw-text-opacity))}#fbjs :is(.hover\\:text-close-button-focus:hover){color:var(--fb-close-btn-hover-color)}#fbjs :is(.hover\\:text-heading:hover){color:var(--fb-heading-color)}#fbjs :is(.hover\\:text-slate-900:hover){--tw-text-opacity: 1;color:rgb(15 23 42 / var(--tw-text-opacity))}#fbjs :is(.hover\\:opacity-90:hover){opacity:.9}#fbjs :is(.focus\\:border-4:focus){border-width:4px}#fbjs :is(.focus\\:border-border-highlight:focus){border-color:var(--fb-border-color-highlight)}#fbjs :is(.focus\\:bg-accent-bg:focus){background-color:var(--fb-accent-background-color)}#fbjs :is(.focus\\:bg-accent-selected-bg:focus){background-color:var(--fb-accent-background-color-selected)}#fbjs :is(.focus\\:outline-none:focus){outline:2px solid transparent;outline-offset:2px}#fbjs :is(.focus\\:ring-0:focus){--tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);--tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(0px + var(--tw-ring-offset-width)) var(--tw-ring-color);box-shadow:var(--tw-ring-offset-shadow),var(--tw-ring-shadow),var(--tw-shadow, 0 0 #0000)}#fbjs :is(.focus\\:ring-2:focus){--tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);--tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(2px + var(--tw-ring-offset-width)) var(--tw-ring-color);box-shadow:var(--tw-ring-offset-shadow),var(--tw-ring-shadow),var(--tw-shadow, 0 0 #0000)}#fbjs :is(.focus\\:ring-border-highlight:focus){--tw-ring-color: var(--fb-border-color-highlight)}#fbjs :is(.focus\\:ring-close-button-focus:focus){--tw-ring-color: var(--fb-close-btn-hover-color)}#fbjs :is(.focus\\:ring-focus:focus){--tw-ring-color: var(--fb-focus-color)}#fbjs :is(.focus\\:ring-neutral-900:focus){--tw-ring-opacity: 1;--tw-ring-color: rgb(23 23 23 / var(--tw-ring-opacity))}#fbjs :is(.focus\\:ring-offset-0:focus){--tw-ring-offset-width: 0px}#fbjs :is(.focus\\:ring-offset-1:focus){--tw-ring-offset-width: 1px}#fbjs :is(.focus\\:ring-offset-2:focus){--tw-ring-offset-width: 2px}#fbjs :is(.disabled\\:cursor-not-allowed:disabled){cursor:not-allowed}#fbjs :is(.disabled\\:opacity-50:disabled){opacity:.5}#fbjs :is(.aria-selected\\:opacity-100[aria-selected=true]){opacity:1}@media (min-width: 640px){#fbjs :is(.sm\\:bottom-3){bottom:.75em}#fbjs :is(.sm\\:left-1\\/2){left:50%}#fbjs :is(.sm\\:left-3){left:.75em}#fbjs :is(.sm\\:right-3){right:.75em}#fbjs :is(.sm\\:top-1\\/2){top:50%}#fbjs :is(.sm\\:top-3){top:.75em}#fbjs :is(.sm\\:m-4){margin:1em}#fbjs :is(.sm\\:max-w-sm){max-width:24em}#fbjs :is(.sm\\:-translate-x-1\\/2){--tw-translate-x: -50%;transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skew(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}#fbjs :is(.sm\\:-translate-y-1\\/2){--tw-translate-y: -50%;transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skew(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}#fbjs :is(.sm\\:transform){transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skew(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y))}#fbjs :is(.sm\\:text-sm){font-size:.875em;line-height:1.25em}}#fbjs :is(:where(.dark) .dark\\:border-slate-600){--tw-border-opacity: 1;border-color:rgb(71 85 105 / var(--tw-border-opacity))}#fbjs :is(:where(.dark) .dark\\:bg-slate-700){--tw-bg-opacity: 1;background-color:rgb(51 65 85 / var(--tw-bg-opacity))}#fbjs :is(:where(.dark) .dark\\:text-slate-400){--tw-text-opacity: 1;color:rgb(148 163 184 / var(--tw-text-opacity))}#fbjs :is(:where(.dark) .dark\\:hover\\:border-slate-500:hover){--tw-border-opacity: 1;border-color:rgb(100 116 139 / var(--tw-border-opacity))}#fbjs :is(:where(.dark) .dark\\:hover\\:bg-slate-800:hover){--tw-bg-opacity: 1;background-color:rgb(30 41 59 / var(--tw-bg-opacity))}";
const preflight = '#fbjs *,#fbjs :before,#fbjs :after{box-sizing:border-box;border-width:0;border-style:solid;border-color:#e5e7eb}#fbjs :before,#fbjs :after{--tw-content: ""}#fbjs html{line-height:1.5;-webkit-text-size-adjust:100%;-moz-tab-size:4;-o-tab-size:4;tab-size:4;font-family:ui-sans-serif,system-ui,sans-serif,"Apple Color Emoji","Segoe UI Emoji",Segoe UI Symbol,"Noto Color Emoji";font-feature-settings:normal}#fbjs body{margin:0;line-height:inherit}#fbjs hr{height:0;color:inherit;border-top-width:1px}#fbjs abbr:where([title]){-webkit-text-decoration:underline dotted;text-decoration:underline dotted}#fbjs h1,#fbjs h2,#fbjs h3,#fbjs h4,#fbjs h5,#fbjs h6{font-size:inherit;font-weight:inherit}#fbjs a{color:inherit;text-decoration:inherit}#fbjs b,#fbjs strong{font-weight:bolder}#fbjs code,#fbjs kbd,#fbjs samp,#fbjs pre{font-family:ui-monospace,SFMono-Regular,Menlo,Monaco,Consolas,Liberation Mono,Courier New,monospace;font-size:1em}#fbjs small{font-size:80%}#fbjs sub,#fbjs sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}#fbjs sub{bottom:-.25em}#fbjs sup{top:-.5em}#fbjs table{text-indent:0;border-color:inherit;border-collapse:collapse}#fbjs button,#fbjs input,#fbjs optgroup,#fbjs select,#fbjs textarea{font-family:inherit;font-size:100%;font-weight:inherit;line-height:inherit;color:inherit;margin:0;padding:0}#fbjs button,#fbjs select{text-transform:none}#fbjs button,#fbjs [type=button],#fbjs [type=reset],#fbjs [type=submit]{-webkit-appearance:button;background-color:transparent;background-image:none}#fbjs :-moz-focusring{outline:auto}#fbjs :-moz-ui-invalid{box-shadow:none}#fbjs progress{vertical-align:baseline}#fbjs ::-webkit-inner-spin-button,#fbjs ::-webkit-outer-spin-button{height:auto}#fbjs [type=search]{-webkit-appearance:textfield;outline-offset:-2px}#fbjs ::-webkit-search-decoration{-webkit-appearance:none}#fbjs ::-webkit-file-upload-button{-webkit-appearance:button;font:inherit}#fbjs summary{display:list-item}#fbjs blockquote,#fbjs dl,#fbjs dd,#fbjs h1,#fbjs h2,#fbjs h3,#fbjs h4,#fbjs h5,#fbjs h6,#fbjs hr,#fbjs figure,#fbjs p,#fbjs pre{margin:0}#fbjs fieldset{margin:0;padding:0}#fbjs legend{padding:0}#fbjs ol,#fbjs ul,#fbjs menu{list-style:none;margin:0;padding:0}#fbjs textarea{resize:vertical}#fbjs input::-moz-placeholder,#fbjs textarea::-moz-placeholder{opacity:1;color:#9ca3af}#fbjs input::placeholder,#fbjs textarea::placeholder{opacity:1;color:#9ca3af}#fbjs button,#fbjs [role=button]{cursor:pointer}#fbjs :disabled{cursor:default}#fbjs img,#fbjs svg,#fbjs video,#fbjs canvas,#fbjs audio,#fbjs iframe,#fbjs embed,#fbjs object{display:block;vertical-align:middle}#fbjs img,#fbjs video{max-width:100%;height:auto}#fbjs [hidden]{display:none}';
const editorCss = ".fb-editor-text-bold{font-weight:700!important}.fb-editor-text-italic{font-style:italic!important}.fb-editor-link{color:#334155!important;text-decoration:underline!important}.editor-tokenFunction{color:#dd4a68!important}.fb-editor-paragraph{margin:0!important;position:relative!important;color:#334155!important}.fb-editor-paragraph:last-child{margin-bottom:0!important}.fb-editor-heading-h1{font-size:25px!important;font-weight:400!important;margin-bottom:20px!important;font-weight:700!important}.fb-editor-heading-h2{font-size:20px!important;font-weight:700!important;margin-bottom:20px!important}.fb-editor-list-ul,.fb-editor-list-ol{margin-bottom:12px!important}.fb-editor-listitem{margin:0 32px!important}.fb-editor-nested-listitem{list-style-type:none!important}";
const addStylesToDom = () => {
  if (document.getElementById("formbricks__css") === null) {
    const styleElement = document.createElement("style");
    styleElement.id = "formbricks__css";
    styleElement.innerHTML = preflight + global + editorCss;
    document.head.appendChild(styleElement);
  }
};
const addCustomThemeToDom = ({
  brandColor
}) => {
  if (document.getElementById("formbricks__css") === null)
    return;
  const styleElement = document.createElement("style");
  styleElement.id = "formbricks__css__custom";
  styleElement.innerHTML = `
    :root {
      --fb-brand-color: ${brandColor};
      ${isLight(brandColor) ? "--fb-brand-text-color: black;" : "--fb-brand-text-color: white;"}
    }
  `;
  document.head.appendChild(styleElement);
};
const renderSurveyModal = (props) => {
  addStylesToDom();
  addCustomThemeToDom({
    brandColor: props.brandColor
  });
  const element = document.createElement("div");
  element.id = "formbricks-modal-container";
  document.body.appendChild(element);
  q(y(SurveyModal, props), element);
};
const diffInDays = (date1, date2) => {
  const diffTime = Math.abs(date2.getTime() - date1.getTime());
  return Math.floor(diffTime / (1e3 * 60 * 60 * 24));
};
const config$5 = Config.getInstance();
const logger$6 = Logger.getInstance();
let syncIntervalId = null;
const syncWithBackend = async ({
  apiHost,
  environmentId,
  userId
}) => {
  const url = `${apiHost}/api/v1/client/${environmentId}/in-app/sync/${userId}`;
  const publicUrl = `${apiHost}/api/v1/client/${environmentId}/in-app/sync`;
  if (!userId) {
    const response2 = await fetch(publicUrl);
    if (!response2.ok) {
      const jsonRes = await response2.json();
      return err({
        code: "network_error",
        status: response2.status,
        message: "Error syncing with backend",
        url,
        responseMessage: jsonRes.message
      });
    }
    return ok((await response2.json()).data);
  }
  const response = await fetch(url);
  if (!response.ok) {
    const jsonRes = await response.json();
    return err({
      code: "network_error",
      status: response.status,
      message: "Error syncing with backend",
      url,
      responseMessage: jsonRes.message
    });
  }
  const data = await response.json();
  const { data: state } = data;
  return ok(state);
};
const sync = async (params) => {
  try {
    const syncResult = await syncWithBackend(params);
    if ((syncResult == null ? void 0 : syncResult.ok) !== true) {
      logger$6.error(`Sync failed: ${JSON.stringify(syncResult.error)}`);
      throw syncResult.error;
    }
    let oldState;
    try {
      oldState = config$5.get().state;
    } catch (e2) {
    }
    let state = {
      surveys: syncResult.value.surveys,
      noCodeActionClasses: syncResult.value.noCodeActionClasses,
      product: syncResult.value.product,
      attributes: (oldState == null ? void 0 : oldState.attributes) || {}
    };
    if (!params.userId) {
      state = {
        ...state,
        displays: (oldState == null ? void 0 : oldState.displays) || []
      };
      state = filterPublicSurveys(state);
      const surveyNames = state.surveys.map((s2) => s2.name);
      logger$6.debug("Fetched " + surveyNames.length + " surveys during sync: " + surveyNames.join(", "));
    } else {
      const surveyNames = state.surveys.map((s2) => s2.name);
      logger$6.debug("Fetched " + surveyNames.length + " surveys during sync: " + surveyNames.join(", "));
    }
    config$5.update({
      apiHost: params.apiHost,
      environmentId: params.environmentId,
      userId: params.userId,
      state
    });
  } catch (error) {
    logger$6.error(`Error during sync: ${error}`);
    throw error;
  }
};
const filterPublicSurveys = (state) => {
  const { displays, product } = state;
  let { surveys } = state;
  if (!displays) {
    return state;
  }
  let filteredSurveys = surveys.filter((survey) => {
    if (survey.displayOption === "respondMultiple") {
      return true;
    } else if (survey.displayOption === "displayOnce") {
      return displays.filter((display) => display.surveyId === survey.id).length === 0;
    } else if (survey.displayOption === "displayMultiple") {
      return displays.filter((display) => display.surveyId === survey.id && display.responded).length === 0;
    } else {
      throw Error("Invalid displayOption");
    }
  });
  const latestDisplay = displays.length > 0 ? displays[displays.length - 1] : void 0;
  filteredSurveys = filteredSurveys.filter((survey) => {
    if (!latestDisplay) {
      return true;
    } else if (survey.recontactDays !== null) {
      const lastDisplaySurvey = displays.filter((display) => display.surveyId === survey.id)[0];
      if (!lastDisplaySurvey) {
        return true;
      }
      return diffInDays(/* @__PURE__ */ new Date(), new Date(lastDisplaySurvey.createdAt)) >= survey.recontactDays;
    } else if (product.recontactDays !== null) {
      return diffInDays(/* @__PURE__ */ new Date(), new Date(latestDisplay.createdAt)) >= product.recontactDays;
    } else {
      return true;
    }
  });
  return {
    ...state,
    surveys: filteredSurveys
  };
};
const addExpiryCheckListener = () => {
  const updateInterval = 1e3 * 60;
  if (typeof window !== "undefined" && syncIntervalId === null) {
    syncIntervalId = window.setInterval(async () => {
      if (config$5.get().expiresAt && new Date(config$5.get().expiresAt) >= /* @__PURE__ */ new Date()) {
        return;
      }
      logger$6.debug("Config has expired. Starting sync.");
      await sync({
        apiHost: config$5.get().apiHost,
        environmentId: config$5.get().environmentId,
        userId: config$5.get().userId
        // personId: config.get().state?.person?.id,
      });
    }, updateInterval);
  }
};
const removeExpiryCheckListener = () => {
  if (typeof window !== "undefined" && syncIntervalId !== null) {
    window.clearInterval(syncIntervalId);
    syncIntervalId = null;
  }
};
const containerId = "formbricks-web-container";
const config$4 = Config.getInstance();
const logger$5 = Logger.getInstance();
const errorHandler$1 = ErrorHandler.getInstance();
let surveyRunning = false;
const renderWidget = (survey) => {
  if (surveyRunning) {
    logger$5.debug("A survey is already running. Skipping.");
    return;
  }
  surveyRunning = true;
  if (survey.delay) {
    logger$5.debug(`Delaying survey by ${survey.delay} seconds.`);
  }
  const product = config$4.get().state.product;
  const surveyState = new SurveyState$1(survey.id, null, null, config$4.get().userId);
  const responseQueue = new ResponseQueue(
    {
      apiHost: config$4.get().apiHost,
      environmentId: config$4.get().environmentId,
      retryAttempts: 2,
      onResponseSendingFailed: (response) => {
        alert(`Failed to send response: ${JSON.stringify(response, null, 2)}`);
      }
    },
    surveyState
  );
  const productOverwrites = survey.productOverwrites ?? {};
  const brandColor = productOverwrites.brandColor ?? product.brandColor;
  const highlightBorderColor = productOverwrites.highlightBorderColor ?? product.highlightBorderColor;
  const clickOutside = productOverwrites.clickOutsideClose ?? product.clickOutsideClose;
  const darkOverlay = productOverwrites.darkOverlay ?? product.darkOverlay;
  const placement = productOverwrites.placement ?? product.placement;
  const isBrandingEnabled = product.inAppSurveyBranding;
  setTimeout(() => {
    renderSurveyModal({
      survey,
      brandColor,
      isBrandingEnabled,
      clickOutside,
      darkOverlay,
      highlightBorderColor,
      placement,
      onDisplay: async () => {
        const { userId } = config$4.get();
        if (!userId) {
          const localDisplay = {
            createdAt: /* @__PURE__ */ new Date(),
            surveyId: survey.id,
            responded: false
          };
          const existingDisplays = config$4.get().state.displays;
          const displays = existingDisplays ? [...existingDisplays, localDisplay] : [localDisplay];
          const previousConfig = config$4.get();
          let state = filterPublicSurveys({
            ...previousConfig.state,
            displays
          });
          config$4.update({
            ...previousConfig,
            state
          });
        }
        const api = new FormbricksAPI({
          apiHost: config$4.get().apiHost,
          environmentId: config$4.get().environmentId
        });
        const res = await api.client.display.create({
          surveyId: survey.id,
          userId
        });
        if (!res.ok) {
          throw new Error("Could not create display");
        }
        const { id } = res.data;
        surveyState.updateDisplayId(id);
        responseQueue.updateSurveyState(surveyState);
      },
      onResponse: (responseUpdate) => {
        const { userId } = config$4.get();
        if (!userId) {
          const displays = config$4.get().state.displays;
          const lastDisplay = displays && displays[displays.length - 1];
          if (!lastDisplay) {
            throw new Error("No lastDisplay found");
          }
          if (!lastDisplay.responded) {
            lastDisplay.responded = true;
            const previousConfig = config$4.get();
            let state = filterPublicSurveys({
              ...previousConfig.state,
              displays
            });
            config$4.update({
              ...previousConfig,
              state
            });
          }
        }
        if (userId) {
          surveyState.updateUserId(userId);
        }
        responseQueue.updateSurveyState(surveyState);
        responseQueue.add({
          data: responseUpdate.data,
          ttc: responseUpdate.ttc,
          finished: responseUpdate.finished
        });
      },
      onClose: closeSurvey,
      onFileUpload: async (file, params) => {
        const api = new FormbricksAPI({
          apiHost: config$4.get().apiHost,
          environmentId: config$4.get().environmentId
        });
        return await api.client.storage.uploadFile(file, params);
      }
    });
  }, survey.delay * 1e3);
};
const closeSurvey = async () => {
  var _a;
  (_a = document.getElementById(containerId)) == null ? void 0 : _a.remove();
  addWidgetContainer();
  if (!config$4.get().userId) {
    const state = config$4.get().state;
    const updatedState = filterPublicSurveys(state);
    config$4.update({
      ...config$4.get(),
      state: updatedState
    });
    surveyRunning = false;
    return;
  }
  try {
    await sync({
      apiHost: config$4.get().apiHost,
      environmentId: config$4.get().environmentId,
      userId: config$4.get().userId
    });
    surveyRunning = false;
  } catch (e2) {
    errorHandler$1.handle(e2);
  }
};
const addWidgetContainer = () => {
  const containerElement = document.createElement("div");
  containerElement.id = containerId;
  document.body.appendChild(containerElement);
};
const logger$4 = Logger.getInstance();
const config$3 = Config.getInstance();
const intentsToNotCreateOnApp = ["Exit Intent (Desktop)", "50% Scroll"];
const trackAction = async (name, properties = {}) => {
  var _a;
  const { userId } = config$3.get();
  const input = {
    environmentId: config$3.get().environmentId,
    userId,
    name,
    properties: properties || {}
  };
  if (userId && !intentsToNotCreateOnApp.includes(name)) {
    logger$4.debug(`Sending action "${name}" to backend`);
    const api = new FormbricksAPI({
      apiHost: config$3.get().apiHost,
      environmentId: config$3.get().environmentId
    });
    const res = await api.client.action.create({
      ...input,
      userId
    });
    if (!res.ok) {
      return err({
        code: "network_error",
        message: `Error tracking action ${name}`,
        status: 500,
        url: `${config$3.get().apiHost}/api/v1/client/${config$3.get().environmentId}/actions`,
        responseMessage: res.error.message
      });
    }
  }
  logger$4.debug(`Formbricks: Action "${name}" tracked`);
  const activeSurveys = (_a = config$3.get().state) == null ? void 0 : _a.surveys;
  if (!!activeSurveys && activeSurveys.length > 0) {
    triggerSurvey(name, activeSurveys);
  } else {
    logger$4.debug("No active surveys to display");
  }
  return okVoid();
};
const triggerSurvey = (actionName, activeSurveys) => {
  for (const survey of activeSurveys) {
    for (const trigger of survey.triggers) {
      if (trigger === actionName) {
        logger$4.debug(`Formbricks: survey ${survey.id} triggered by action "${actionName}"`);
        renderWidget(survey);
        return;
      }
    }
  }
};
const getApi = () => {
  const config2 = Config.getInstance();
  const { environmentId, apiHost } = config2.get();
  if (!environmentId || !apiHost) {
    throw new Error("formbricks.init() must be called before getApi()");
  }
  return new FormbricksAPI({
    apiHost,
    environmentId
  });
};
let exitIntentListenerAdded = false;
let exitIntentListenerWrapper = async function(e2) {
  if (e2.clientY <= 0) {
    const trackResult = await trackAction("Exit Intent (Desktop)");
    if (trackResult.ok !== true) {
      return err(trackResult.error);
    }
  }
};
const addExitIntentListener = () => {
  if (typeof document !== "undefined" && !exitIntentListenerAdded) {
    document.querySelector("body").addEventListener("mouseleave", exitIntentListenerWrapper);
    exitIntentListenerAdded = true;
  }
};
const removeExitIntentListener = () => {
  if (exitIntentListenerAdded) {
    document.removeEventListener("mouseleave", exitIntentListenerWrapper);
    exitIntentListenerAdded = false;
  }
};
let scrollDepthListenerAdded = false;
let scrollDepthTriggered = false;
let scrollDepthListenerWrapper = async () => {
  const scrollPosition = window.scrollY;
  const windowSize = window.innerHeight;
  const bodyHeight = document.documentElement.scrollHeight;
  if (scrollPosition === 0) {
    scrollDepthTriggered = false;
  }
  if (!scrollDepthTriggered && scrollPosition / (bodyHeight - windowSize) >= 0.5) {
    scrollDepthTriggered = true;
    const trackResult = await trackAction("50% Scroll");
    if (trackResult.ok !== true) {
      return err(trackResult.error);
    }
  }
};
const addScrollDepthListener = () => {
  if (typeof window !== "undefined" && !scrollDepthListenerAdded) {
    window.addEventListener("load", () => {
      window.addEventListener("scroll", scrollDepthListenerWrapper);
    });
    scrollDepthListenerAdded = true;
  }
};
const removeScrollDepthListener = () => {
  if (scrollDepthListenerAdded) {
    window.removeEventListener("scroll", scrollDepthListenerWrapper);
    scrollDepthListenerAdded = false;
  }
};
const config$2 = Config.getInstance();
const logger$3 = Logger.getInstance();
const errorHandler = ErrorHandler.getInstance();
const checkPageUrl = async () => {
  var _a;
  logger$3.debug(`Checking page url: ${window.location.href}`);
  const { state } = config$2.get();
  if ((state == null ? void 0 : state.noCodeActionClasses) === void 0) {
    return okVoid();
  }
  const pageUrlEvents = ((state == null ? void 0 : state.noCodeActionClasses) || []).filter((action) => {
    const { innerHtml, cssSelector, pageUrl } = action.noCodeConfig || {};
    return pageUrl && !innerHtml && !cssSelector;
  });
  if (pageUrlEvents.length === 0) {
    return okVoid();
  }
  for (const event of pageUrlEvents) {
    if (!((_a = event.noCodeConfig) == null ? void 0 : _a.pageUrl)) {
      continue;
    }
    const {
      noCodeConfig: { pageUrl }
    } = event;
    const match2 = checkUrlMatch(window.location.href, pageUrl.value, pageUrl.rule);
    if (match2.ok !== true)
      return err(match2.error);
    if (match2.value === false)
      continue;
    const trackResult = await trackAction(event.name);
    if (trackResult.ok !== true)
      return err(trackResult.error);
  }
  return okVoid();
};
let arePageUrlEventListenersAdded = false;
const checkPageUrlWrapper = () => checkPageUrl();
const addPageUrlEventListeners = () => {
  if (typeof window === "undefined" || arePageUrlEventListenersAdded)
    return;
  window.addEventListener("hashchange", checkPageUrlWrapper);
  window.addEventListener("popstate", checkPageUrlWrapper);
  window.addEventListener("pushstate", checkPageUrlWrapper);
  window.addEventListener("replacestate", checkPageUrlWrapper);
  window.addEventListener("load", checkPageUrlWrapper);
  arePageUrlEventListenersAdded = true;
};
const removePageUrlEventListeners = () => {
  if (typeof window === "undefined" || !arePageUrlEventListenersAdded)
    return;
  window.removeEventListener("hashchange", checkPageUrlWrapper);
  window.removeEventListener("popstate", checkPageUrlWrapper);
  window.removeEventListener("pushstate", checkPageUrlWrapper);
  window.removeEventListener("replacestate", checkPageUrlWrapper);
  window.removeEventListener("load", checkPageUrlWrapper);
  arePageUrlEventListenersAdded = false;
};
function checkUrlMatch(url, pageUrlValue, pageUrlRule) {
  switch (pageUrlRule) {
    case "exactMatch":
      return ok(url === pageUrlValue);
    case "contains":
      return ok(url.includes(pageUrlValue));
    case "startsWith":
      return ok(url.startsWith(pageUrlValue));
    case "endsWith":
      return ok(url.endsWith(pageUrlValue));
    case "notMatch":
      return ok(url !== pageUrlValue);
    case "notContains":
      return ok(!url.includes(pageUrlValue));
    default:
      return err({
        code: "invalid_match_type",
        message: "Invalid match type"
      });
  }
}
const checkClickMatch = (event) => {
  const { state } = config$2.get();
  if (!state) {
    return;
  }
  const targetElement = event.target;
  ((state == null ? void 0 : state.noCodeActionClasses) || []).forEach((action) => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const innerHtml = (_b = (_a = action.noCodeConfig) == null ? void 0 : _a.innerHtml) == null ? void 0 : _b.value;
    const cssSelectors = (_d = (_c = action.noCodeConfig) == null ? void 0 : _c.cssSelector) == null ? void 0 : _d.value;
    const pageUrl = (_f = (_e = action.noCodeConfig) == null ? void 0 : _e.pageUrl) == null ? void 0 : _f.value;
    const pageUrlRule = (_h = (_g = action.noCodeConfig) == null ? void 0 : _g.pageUrl) == null ? void 0 : _h.rule;
    if (!innerHtml && !cssSelectors && !pageUrl) {
      return;
    }
    if (innerHtml && targetElement.innerHTML !== innerHtml) {
      return;
    }
    if (cssSelectors) {
      const individualSelectors = cssSelectors.split(/\s*(?=[.#])/);
      for (let selector of individualSelectors) {
        if (!targetElement.matches(selector)) {
          return;
        }
      }
    }
    if (pageUrl && pageUrlRule) {
      const urlMatch = checkUrlMatch(window.location.href, pageUrl, pageUrlRule);
      if (!urlMatch.ok || !urlMatch.value) {
        return;
      }
    }
    trackAction(action.name).then((res) => {
      match(
        res,
        (_value) => {
        },
        (err2) => {
          errorHandler.handle(err2);
        }
      );
    });
  });
};
let isClickEventListenerAdded = false;
const checkClickMatchWrapper = (e2) => checkClickMatch(e2);
const addClickEventListener = () => {
  if (typeof window === "undefined" || isClickEventListenerAdded)
    return;
  document.addEventListener("click", checkClickMatchWrapper);
  isClickEventListenerAdded = true;
};
const removeClickEventListener = () => {
  if (!isClickEventListenerAdded)
    return;
  document.removeEventListener("click", checkClickMatchWrapper);
  isClickEventListenerAdded = false;
};
let areRemoveEventListenersAdded = false;
const addEventListeners = () => {
  addExpiryCheckListener();
  addPageUrlEventListeners();
  addClickEventListener();
  addExitIntentListener();
  addScrollDepthListener();
};
const addCleanupEventListeners = () => {
  if (areRemoveEventListenersAdded)
    return;
  window.addEventListener("beforeunload", () => {
    removeExpiryCheckListener();
    removePageUrlEventListeners();
    removeClickEventListener();
    removeExitIntentListener();
    removeScrollDepthListener();
  });
  areRemoveEventListenersAdded = true;
};
const removeCleanupEventListeners = () => {
  if (!areRemoveEventListenersAdded)
    return;
  window.removeEventListener("beforeunload", () => {
    removeExpiryCheckListener();
    removePageUrlEventListeners();
    removeClickEventListener();
    removeExitIntentListener();
    removeScrollDepthListener();
  });
  areRemoveEventListenersAdded = false;
};
const removeAllEventListeners = () => {
  removeExpiryCheckListener();
  removePageUrlEventListeners();
  removeClickEventListener();
  removeExitIntentListener();
  removeScrollDepthListener();
  removeCleanupEventListeners();
};
const config$1 = Config.getInstance();
const logger$2 = Logger.getInstance();
const updatePersonAttribute = async (key, value) => {
  const { apiHost, environmentId, userId } = config$1.get();
  if (!userId) {
    return err({
      code: "missing_person",
      message: "Unable to update attribute. User identification deactivated. No userId set."
    });
  }
  const input = {
    attributes: {
      [key]: value
    }
  };
  const api = new FormbricksAPI({
    apiHost: config$1.get().apiHost,
    environmentId: config$1.get().environmentId
  });
  const res = await api.client.people.update(userId, input);
  if (!res.ok) {
    return err({
      code: "network_error",
      status: 500,
      message: `Error updating person with userId ${userId}`,
      url: `${config$1.get().apiHost}/api/v1/client/${environmentId}/people/${userId}`,
      responseMessage: res.error.message
    });
  }
  logger$2.debug("Attribute updated. Syncing...");
  await sync({
    environmentId,
    apiHost,
    userId
  });
  return okVoid();
};
const updatePersonAttributes = async (apiHost, environmentId, userId, attributes) => {
  var _a, _b;
  if (!userId) {
    return err({
      code: "missing_person",
      message: "Unable to update attribute. User identification deactivated. No userId set."
    });
  }
  const updatedAttributes = { ...attributes };
  try {
    const existingAttributes = (_b = (_a = config$1.get()) == null ? void 0 : _a.state) == null ? void 0 : _b.attributes;
    if (existingAttributes) {
      for (const [key, value] of Object.entries(existingAttributes)) {
        if (updatedAttributes[key] === value) {
          delete updatedAttributes[key];
        }
      }
    }
  } catch (e2) {
    logger$2.debug("config not set; sending all attributes to backend");
  }
  if (Object.keys(updatedAttributes).length === 0) {
    logger$2.debug("No attributes to update. Skipping update.");
    return ok(updatedAttributes);
  }
  logger$2.debug("Updating attributes: " + JSON.stringify(updatedAttributes));
  const input = {
    attributes: updatedAttributes
  };
  const api = new FormbricksAPI({
    apiHost,
    environmentId
  });
  const res = await api.client.people.update(userId, input);
  if (res.ok) {
    return ok(updatedAttributes);
  }
  return err({
    code: "network_error",
    status: 500,
    message: `Error updating person with userId ${userId}`,
    url: `${apiHost}/api/v1/client/${environmentId}/people/${userId}`,
    responseMessage: res.error.message
  });
};
const isExistingAttribute = (key, value) => {
  if (config$1.get().state.attributes[key] === value) {
    return true;
  }
  return false;
};
const setPersonUserId = async () => {
  logger$2.error("'setUserId' is no longer supported. Please set the userId in the init call instead.");
  return okVoid();
};
const setPersonAttribute = async (key, value) => {
  logger$2.debug("Setting attribute: " + key + " to value: " + value);
  if (isExistingAttribute(key, value.toString())) {
    logger$2.debug("Attribute already set to this value. Skipping update.");
    return okVoid();
  }
  const result = await updatePersonAttribute(key, value.toString());
  if (result.ok) {
    config$1.update({
      environmentId: config$1.get().environmentId,
      apiHost: config$1.get().apiHost,
      userId: config$1.get().userId,
      state: {
        ...config$1.get().state,
        attributes: {
          ...config$1.get().state.attributes,
          [key]: value.toString()
        }
      }
    });
    return okVoid();
  }
  return err(result.error);
};
const logoutPerson = async () => {
  deinitalize();
};
const resetPerson = async () => {
  logger$2.debug("Resetting state & getting new state from backend");
  closeSurvey();
  const syncParams = {
    environmentId: config$1.get().environmentId,
    apiHost: config$1.get().apiHost,
    userId: config$1.get().userId
  };
  await logoutPerson();
  try {
    await initialize(syncParams);
    return okVoid();
  } catch (e2) {
    return err(e2);
  }
};
const config = Config.getInstance();
const logger$1 = Logger.getInstance();
let isInitialized = false;
const setDebugLevel = (c2) => {
  if (c2.debug) {
    logger$1.debug(`Setting log level to debug`);
    logger$1.configure({ logLevel: "debug" });
  }
};
const initialize = async (c2) => {
  if (isInitialized) {
    logger$1.debug("Already initialized, skipping initialization.");
    return okVoid();
  }
  setDebugLevel(c2);
  ErrorHandler.getInstance().printStatus();
  logger$1.debug("Start initialize");
  if (!c2.environmentId) {
    logger$1.debug("No environmentId provided");
    return err({
      code: "missing_field",
      field: "environmentId"
    });
  }
  if (!c2.apiHost) {
    logger$1.debug("No apiHost provided");
    return err({
      code: "missing_field",
      field: "apiHost"
    });
  }
  logger$1.debug("Adding widget container to DOM");
  addWidgetContainer();
  if (!c2.userId && c2.attributes) {
    logger$1.error("No userId provided but attributes. Cannot update attributes without userId.");
    return err({
      code: "missing_field",
      field: "userId"
    });
  }
  let updatedAttributes = null;
  if (c2.userId && c2.attributes) {
    const res = await updatePersonAttributes(c2.apiHost, c2.environmentId, c2.userId, c2.attributes);
    if (res.ok !== true) {
      return err(res.error);
    }
    updatedAttributes = res.value;
  }
  let existingConfig;
  try {
    existingConfig = config.get();
  } catch (e2) {
    logger$1.debug("No existing configuration found.");
  }
  if (existingConfig && existingConfig.state && existingConfig.environmentId === c2.environmentId && existingConfig.apiHost === c2.apiHost && existingConfig.userId === c2.userId && existingConfig.expiresAt) {
    logger$1.debug("Found existing configuration.");
    if (existingConfig.expiresAt < /* @__PURE__ */ new Date()) {
      logger$1.debug("Configuration expired.");
      await sync({
        apiHost: c2.apiHost,
        environmentId: c2.environmentId,
        userId: c2.userId
      });
    } else {
      logger$1.debug("Configuration not expired. Extending expiration.");
      config.update(existingConfig);
    }
  } else {
    logger$1.debug("No valid configuration found or it has been expired. Creating new config.");
    logger$1.debug("Syncing.");
    await sync({
      apiHost: c2.apiHost,
      environmentId: c2.environmentId,
      userId: c2.userId
    });
    await trackAction("New Session");
  }
  if (updatedAttributes && Object.keys(updatedAttributes).length > 0) {
    config.update({
      environmentId: config.get().environmentId,
      apiHost: config.get().apiHost,
      userId: config.get().userId,
      state: {
        ...config.get().state,
        attributes: { ...config.get().state.attributes, ...c2.attributes }
      }
    });
  }
  logger$1.debug("Adding event listeners");
  addEventListeners();
  addCleanupEventListeners();
  isInitialized = true;
  logger$1.debug("Initialized");
  checkPageUrl();
  return okVoid();
};
const checkInitialized = () => {
  logger$1.debug("Check if initialized");
  if (!isInitialized || !ErrorHandler.initialized) {
    return err({
      code: "not_initialized",
      message: "Formbricks not initialized. Call initialize() first."
    });
  }
  return okVoid();
};
const deinitalize = () => {
  logger$1.debug("Deinitializing");
  closeSurvey();
  removeAllEventListeners();
  config.resetConfig();
  isInitialized = false;
};
class CommandQueue {
  constructor() {
    this.queue = [];
    this.running = false;
    this.resolvePromise = null;
    this.commandPromise = null;
  }
  add(checkInitialized2 = true, command, ...args) {
    this.queue.push({ command, checkInitialized: checkInitialized2, commandArgs: args });
    if (!this.running) {
      this.commandPromise = new Promise((resolve) => {
        this.resolvePromise = resolve;
        this.run();
      });
    }
  }
  async wait() {
    if (this.running) {
      await this.commandPromise;
    }
  }
  async run() {
    this.running = true;
    while (this.queue.length > 0) {
      const errorHandler2 = ErrorHandler.getInstance();
      const currentItem = this.queue.shift();
      if (!currentItem)
        continue;
      if (currentItem.checkInitialized) {
        const initResult = checkInitialized();
        if (initResult && initResult.ok !== true)
          errorHandler2.handle(initResult.error);
      }
      const result = await (currentItem == null ? void 0 : currentItem.command.apply(null, currentItem == null ? void 0 : currentItem.commandArgs));
      if (!result)
        continue;
      if (result.ok !== true)
        errorHandler2.handle(result.error);
    }
    this.running = false;
    if (this.resolvePromise) {
      this.resolvePromise();
      this.resolvePromise = null;
      this.commandPromise = null;
    }
  }
}
const logger = Logger.getInstance();
logger.debug("Create command queue");
const queue = new CommandQueue();
const init = async (initConfig) => {
  ErrorHandler.init(initConfig.errorHandler);
  queue.add(false, initialize, initConfig);
  await queue.wait();
};
const setUserId = async () => {
  queue.add(true, setPersonUserId);
  await queue.wait();
};
const setEmail = async (email) => {
  setAttribute("email", email);
  await queue.wait();
};
const setAttribute = async (key, value) => {
  queue.add(true, setPersonAttribute, key, value);
  await queue.wait();
};
const logout = async () => {
  queue.add(true, logoutPerson);
  await queue.wait();
};
const reset = async () => {
  queue.add(true, resetPerson);
  await queue.wait();
};
const track = async (name, properties = {}) => {
  queue.add(true, trackAction, name, properties);
  await queue.wait();
};
const registerRouteChange = async () => {
  queue.add(true, checkPageUrl);
  await queue.wait();
};
const formbricks = {
  init,
  setUserId,
  setEmail,
  setAttribute,
  track,
  logout,
  reset,
  registerRouteChange,
  getApi
};
export {
  formbricks as default
};
//# sourceMappingURL=index.mjs.map
